@@ -19,8 +19,13 @@
 
 # Headers in this file shall remain intact.
 
+"""utilities used by flumotion-command"""
+
 from flumotion.twisted.defer import defer_generator
-from flumotion.common import componentui, errors
+from flumotion.common import errors
+
+__version__ = "$Rev: 7162 $"
+
 
 def avatarId(string):
     split = string.split('/')
@@ -28,6 +33,7 @@ def avatarId(string):
     assert not split[0]
     return split[1:]
 
+
 def avatarPath(string):
     split = string.split('/')
     assert not split[0]
@@ -41,7 +47,8 @@ def avatarPath(string):
             return ['root']
         else:
             return ['flow'] + split[1:]
-    
+
+
 def find_component(planet, avatarId):
     if avatarId[0] == 'atmosphere':
         for c in planet.get('atmosphere').get('components'):
@@ -50,7 +57,7 @@ def find_component(planet, avatarId):
         print ('Could not find component named %s in flow %s'
                % (avatarId[1], avatarId[0]))
         return None
-    
+
     for f in planet.get('flows'):
         if f.get('name') == avatarId[0]:
             for c in f.get('components'):
@@ -60,6 +67,7 @@ def find_component(planet, avatarId):
            % (avatarId[1], avatarId[0]))
     return None
 
+
 def get_component_uistate(model, avatarId, component=None, quiet=False):
     if not component:
         d = model.callRemote('getPlanetState')
