@@ -19,17 +19,44 @@
 
 # Headers in this file shall remain intact.
 
-"""
-objects and functions used in dealing with packages
+"""objects and functions used in dealing with packages
 """
 
 import ihooks
+import glob
 import os
 import sys
-import glob
+
+from twisted.python import rebuild, reflect
 
 from flumotion.common import log, common
-from twisted.python import rebuild, reflect
+from flumotion.configure import configure
+
+__version__ = "$Rev: 7927 $"
+
+
+class _PatchedModuleImporter(ihooks.ModuleImporter):
+    """
+    I am overriding ihook's ModuleImporter's import_module() method to
+    accept (and ignore) the 'level' keyword argument that appeared in
+    the built-in __import__() function in python2.5.
+
+    While no built-in modules in python2.5 seem to use that keyword
+    argument, 'encodings' module in python2.6 does and so it breaks if
+    used together with ihooks.
+
+    I make no attempt to properly support the 'level' argument -
+    ihooks didn't make it into py3k, and the only use in python2.6
+    we've seen so far, in 'encodings', serves as a performance hint
+    and it seems that can be ignored with no difference in behaviour.
+    """
+
+    def import_module(self, name, globals=None, locals=None, fromlist=None,
+                      level=-1):
+        # all we do is drop 'level' as ihooks don't support it, anyway
+        return ihooks.ModuleImporter.import_module(self, name, globals,
+                                                   locals, fromlist)
+
 
 class PackageHooks(ihooks.Hooks):
     """
@@ -52,7 +79,8 @@ class PackageHooks(ihooks.Hooks):
             return ret
 
         # get full paths to the package
-        paths = [os.path.join(path, name.replace('.', os.sep)) for path in packagePaths]
+        paths = [os.path.join(path, name.replace('.', os.sep))
+                 for path in packagePaths]
         for path in paths:
             if not path in m.__path__:
                 log.log('packager', 'adding path %s for package %s' % (
@@ -61,6 +89,7 @@ class PackageHooks(ihooks.Hooks):
 
         return ret
 
+
 class Packager(log.Loggable):
     """
     I am an object through which package paths can be registered, to support
@@ -81,7 +110,12 @@ class Packager(log.Loggable):
         self.debug('installing custom importer')
         self._hooks = PackageHooks()
         self._hooks.packager = self
-        self._importer = ihooks.ModuleImporter()
+        if sys.version_info < (2, 6):
+            self._importer = ihooks.ModuleImporter()
+        else:
+            self.debug('python2.6 or later detected - using patched'
+                       ' ModuleImporter')
+            self._importer = _PatchedModuleImporter()
         self._importer.set_hooks(self._hooks)
         self._importer.install()
 
@@ -90,12 +124,12 @@ class Packager(log.Loggable):
         Return all absolute paths to the top level of a tree from which
         (part of) the given package name can be imported.
         """
-        if not packageName in self._packages.keys():
+        if packageName not in self._packages:
             return None
-            
+
         return [self._paths[key] for key in self._packages[packageName]]
 
-    def registerPackagePath(self, packagePath, key, prefix='flumotion'):
+    def registerPackagePath(self, packagePath, key, prefix=configure.PACKAGE):
         """
         Register a given path as a path that can be imported from.
         Used to support partition of bundled code or import code from various
@@ -122,20 +156,25 @@ class Packager(log.Loggable):
         self.log('registering packagePath %s' % packagePath)
 
         # check if a packagePath for this bundle was already registered
-        if key in self._paths.keys():
+        if key in self._paths:
             oldPath = self._paths[key]
             if packagePath == oldPath:
                 self.log('already registered %s for key %s' % (
                     packagePath, key))
                 return
             new = False
-                
+
         # Find the packages in the path and sort them,
         # the following algorithm only works if they're sorted.
         # By sorting the list we can ensure that a parent package
         # is always processed before one of its children
-        packageNames = _findPackageCandidates(packagePath, prefix)
-           
+        if not os.path.isdir(packagePath):
+            log.warning('bundle', 'package path not a dir: %s',
+                        packagePath)
+            packageNames = []
+        else:
+            packageNames = _findPackageCandidates(packagePath, prefix)
+
         if not packageNames:
             log.log('bundle',
                 'packagePath %s does not have candidates starting with %s' %
@@ -170,19 +209,19 @@ class Packager(log.Loggable):
 
         # update our name->keys cache
         for name in packageNames:
-            if not name in self._packages.keys():
+            if name not in self._packages:
                 self._packages[name] = [key]
             else:
                 self._packages[name].insert(0, key)
-                
+
         self.log('packagePath %s has packageNames %r' % (
-            packagePath, packageNames)) 
+            packagePath, packageNames))
         # since we want sub-modules to be fixed up before parent packages,
         # we reverse the list
         packageNames.reverse()
 
         for packageName in packageNames:
-            if packageName not in sys.modules.keys():
+            if packageName not in sys.modules:
                 continue
             self.log('fixing up %s ...' % packageName)
 
@@ -219,7 +258,7 @@ class Packager(log.Loggable):
                 self.log('path %s already at start of %s.__path__' % (
                     newPath, packageName))
                 continue
-                
+
             if newPath in package.__path__:
                 package.__path__.remove(newPath)
                 self.log('moving %s to front of %s.__path__' % (
@@ -236,13 +275,19 @@ class Packager(log.Loggable):
             # rebuild.rebuild(package)
             # self.log('rebuilt package %s with paths %r' % (packageName,
             #     package.__path__))
-            self.log('fixed up %s, __path__ %s ...' % (packageName, package.__path__))
+            self.log('fixed up %s, __path__ %s ...' % (
+                packageName, package.__path__))
 
         # now rebuild all non-package modules in this packagePath if this
         # is not a new package
         if not new:
             self.log('finding end module candidates')
-            moduleNames = findEndModuleCandidates(packagePath, prefix)
+            if not os.path.isdir(packagePath):
+                log.warning('bundle', 'package path not a dir: %s',
+                            path)
+                moduleNames = []
+            else:
+                moduleNames = findEndModuleCandidates(packagePath, prefix)
             self.log('end module candidates to rebuild: %r' % moduleNames)
             for name in moduleNames:
                 if name in sys.modules:
@@ -278,19 +323,16 @@ class Packager(log.Loggable):
         self.debug('uninstalling custom importer')
         self._importer.uninstall()
 
+
 def _listDirRecursively(path):
     """
     I'm similar to os.listdir, but I work recursively and only return
     directories containing python code.
-    
+
     @param path: the path
     @type  path: string
     """
     retval = []
-    # files are never returned, only directories
-    if not os.path.isdir(path):
-        return retval
-
     try:
         files = os.listdir(path)
     except OSError:
@@ -298,18 +340,21 @@ def _listDirRecursively(path):
     else:
         for f in files:
             # this only adds directories since files are not returned
-            retval += _listDirRecursively(os.path.join(path, f))
+            p = os.path.join(path, f)
+            if os.path.isdir(p) and f != '.svn':
+                retval += _listDirRecursively(p)
 
     if glob.glob(os.path.join(path, '*.py*')):
         retval.append(path)
-            
+
     return retval
 
+
 def _listPyFileRecursively(path):
     """
     I'm similar to os.listdir, but I work recursively and only return
     files representing python non-package modules.
-    
+
     @param path: the path
     @type  path: string
 
@@ -321,9 +366,9 @@ def _listPyFileRecursively(path):
     # get all the dirs containing python code
     dirs = _listDirRecursively(path)
 
-    for dir in dirs:
-        pyfiles = glob.glob(os.path.join(dir, '*.py*'))
-        dontkeep = glob.glob(os.path.join(dir, '*__init__.py*'))
+    for directory in dirs:
+        pyfiles = glob.glob(os.path.join(directory, '*.py*'))
+        dontkeep = glob.glob(os.path.join(directory, '*__init__.py*'))
         for f in dontkeep:
             if f in pyfiles:
                 pyfiles.remove(f)
@@ -332,7 +377,8 @@ def _listPyFileRecursively(path):
 
     return retval
 
-def _findPackageCandidates(path, prefix='flumotion'):
+
+def _findPackageCandidates(path, prefix=configure.PACKAGE):
     """
     I take a directory and return a list of candidate python packages
     under that directory that start with the given prefix.
@@ -350,10 +396,8 @@ def _findPackageCandidates(path, prefix='flumotion'):
     bundlePaths = [x[len(path) + 1:] for x in dirs]
 
     # remove some common candidates, like .svn subdirs, or containing -
-    isNotSvn = lambda x: x.find('.svn') == -1
-    bundlePaths = filter(isNotSvn, bundlePaths)
-    isNotDashed = lambda x: x.find('-') == -1
-    bundlePaths = filter(isNotDashed, bundlePaths)
+    bundlePaths = [path for path in bundlePaths if path.find('.svn') == -1]
+    bundlePaths = [path for path in bundlePaths if path.find('-') == -1]
 
     # convert paths to module namespace
     bundlePackages = [".".join(x.split(os.path.sep)) for x in bundlePaths]
@@ -371,22 +415,23 @@ def _findPackageCandidates(path, prefix='flumotion'):
             else:
                 build = build + "." + p
             packages[build] = 1
-            
+
     bundlePackages = packages.keys()
 
     # sort them so that depending packages are after higher-up packages
     bundlePackages.sort()
-        
+
     return bundlePackages
 
-def findEndModuleCandidates(path, prefix='flumotion'):
+
+def findEndModuleCandidates(path, prefix=configure.PACKAGE):
     """
     I take a directory and return a list of candidate python end modules
     (i.e., non-package modules) for the given module prefix.
 
     @param path:   the path under which to search for end modules
     @type  path:   string
-    @param prefix: module prefix to check candidates under 
+    @param prefix: module prefix to check candidates under
     @type  prefix: string
     """
     pathPrefix = "/".join(prefix.split("."))
@@ -396,30 +441,30 @@ def findEndModuleCandidates(path, prefix='flumotion'):
     importPaths = [x[len(path) + 1:] for x in files]
 
     # remove some common candidates, like .svn subdirs, or containing -
-    isNotSvn = lambda x: x.find('.svn') == -1
-    importPaths = filter(isNotSvn, importPaths)
-    isNotDashed = lambda x: x.find('-') == -1
-    importPaths = filter(isNotDashed, importPaths)
+    importPaths = [path for path in importPaths if path.find('.svn') == -1]
+    importPaths = [path for path in importPaths if path.find('-') == -1]
 
     # convert paths to module namespace
     endModules = [common.pathToModuleName(x) for x in importPaths]
 
     # remove all not starting with prefix
-    isInPrefix = lambda x: x and x.startswith(prefix)
-    endModules = filter(isInPrefix, endModules)
+    endModules = [module for module in endModules
+                             if module and module.startswith(prefix)]
 
     # sort them so that depending packages are after higher-up packages
     endModules.sort()
 
     # make unique
     res = {}
-    for b in endModules: res[b] = 1
+    for b in endModules:
+        res[b] = 1
 
     return res.keys()
 
 # singleton factory function
 __packager = None
 
+
 def getPackager():
     """
     Return the (unique) packager.
