@@ -19,9 +19,12 @@
 
 # Headers in this file shall remain intact.
 
+"""initalizing logging and package paths.
 """
-Code related to setup and used as part of the bootstrapping of binaries.
-"""
+
+__version__ = "$Rev: 7162 $"
+
+
 def setup():
     """
     Set up the logging system.
@@ -29,6 +32,7 @@ def setup():
     from flumotion.common import log
     log.init()
 
+
 def setupPackagePath():
     """
     set up all project paths specified in the FLU_PROJECT_PATH environment
@@ -42,10 +46,10 @@ def setupPackagePath():
     from flumotion.configure import configure
 
     registryPaths = [configure.pythondir, ]
-    if os.environ.has_key('FLU_PROJECT_PATH'):
+    if 'FLU_PROJECT_PATH' in os.environ:
         paths = os.environ['FLU_PROJECT_PATH']
         registryPaths += paths.split(':')
-    
+
     log.debug('setup', 'registry paths: %s' % ", ".join(registryPaths))
     for path in registryPaths:
         log.debug('setup', 'registering package path: %s' % path)
@@ -53,4 +57,3 @@ def setupPackagePath():
         # these aren't meant to be replaced
         package.getPackager().registerPackagePath(path,
             "FLU_PROJECT_PATH_" + path)
-
