@@ -19,9 +19,16 @@
 
 # Headers in this file shall remain intact.
 
+"""abstract data types with built-in notification support
+"""
+
+__version__ = "$Rev: 7162 $"
+
 
 def _make_watched(type, *mutators):
+
     class Watched(type):
+
         def __init__(self):
             type.__init__(self)
             self.watch_id = 0
@@ -40,6 +47,7 @@ def _make_watched(type, *mutators):
                 proc(self)
 
     def mutate(method):
+
         def do_mutate(self, *args, **kwargs):
             method(self, *args, **kwargs)
             self.notify_changed()
@@ -51,3 +59,5 @@ def _make_watched(type, *mutators):
 
 WatchedList = _make_watched(list, 'append', 'insert', 'remove', 'pop',
                             'sort', 'reverse')
+WatchedDict = _make_watched(dict, '__setitem__', '__delitem__', 'pop',
+                            'popitem', 'update')
