@@ -19,8 +19,7 @@
 
 # Headers in this file shall remain intact.
 
-"""
-Objects related to the state of workers.
+"""objects related to the state of workers.
 """
 
 import os
@@ -29,13 +28,16 @@ import signal
 from twisted.spread import pb
 from twisted.internet import protocol
 
-from flumotion.twisted import flavors
 from flumotion.common import log, errors, messages
+from flumotion.common.i18n import N_, gettexter
+from flumotion.twisted import flavors
+
+__version__ = "$Rev: 7162 $"
+T_ = gettexter()
 
-from flumotion.common.messages import N_
-T_ = messages.gettexter('flumotion')
 
 class ProcessProtocol(protocol.ProcessProtocol):
+
     def __init__(self, loggable, avatarId, processType, where):
         self.loggable = loggable
         self.avatarId = avatarId
@@ -93,7 +95,7 @@ class ProcessProtocol(protocol.ProcessProtocol):
                     # if we find any of these, possibly special-case them too
                     obj.info("Reaped child with pid %s signaled by "
                              "signal %d.", pid, signum)
-                    
+
                 if not os.WCOREDUMP(status.value.status):
                     obj.warning("No core dump generated. "
                                 "Were core dumps enabled at the start ?")
@@ -101,6 +103,8 @@ class ProcessProtocol(protocol.ProcessProtocol):
                         "However, no core dump was generated. "
                         "You may need to configure the environment "
                         "if you want to further debug this problem.")))
+                    #message.description = T_(N_(
+                    #    "Learn how to enable core dumps."))
                 else:
                     obj.info("Core dumped.")
                     corepath = os.path.join(os.getcwd(), 'core.%s' % pid)
@@ -112,6 +116,10 @@ class ProcessProtocol(protocol.ProcessProtocol):
                         # FIXME: add an action that runs gdb and produces a
                         # backtrace; or produce it here and attach to the
                         # message as debug info.
+                        message.description = T_(N_(
+                            "Learn how to analyze core dumps."))
+                        message.section = 'chapter-debug'
+                        message.anchor = 'section-os-analyze-core-dumps'
 
         if message:
             obj.debug('sending message to manager/admin')
@@ -119,20 +127,27 @@ class ProcessProtocol(protocol.ProcessProtocol):
 
         self.setPid(None)
 
+
 class PortSet(log.Loggable):
     """
     A list of ports that keeps track of which are available for use on a
     given machine.
     """
     # not very efficient mkay
-    def __init__(self, logName, ports):
+
+    def __init__(self, logName, ports, randomPorts=False):
         self.logName = logName
         self.ports = ports
         self.used = [0] * len(ports)
+        self.random = randomPorts
 
     def reservePorts(self, numPorts):
         ret = []
         while numPorts > 0:
+            if self.random:
+                ret.append(0)
+                numPorts -= 1
+                continue
             if not 0 in self.used:
                 raise errors.ComponentStartError(
                     'could not allocate port on worker %s' % self.logName)
@@ -146,14 +161,14 @@ class PortSet(log.Loggable):
         for port in ports:
             try:
                 i = self.ports.index(port)
-            except IndexError:
+            except ValueError:
                 self.warning('portset does not include port %d', port)
             else:
                 if self.used[i]:
                     self.warning('port %d already in use!', port)
                 else:
                     self.used[i] = 1
-        
+
     def releasePorts(self, ports):
         """
         @param ports: list of ports to release
@@ -171,11 +186,13 @@ class PortSet(log.Loggable):
 
     def numFree(self):
         return len(self.ports) - self.numUsed()
-    
+
     def numUsed(self):
         return len(filter(None, self.used))
-    
+
 # worker heaven state proxy objects
+
+
 class ManagerWorkerHeavenState(flavors.StateCacheable):
     """
     I represent the state of the worker heaven on the manager.
@@ -185,6 +202,7 @@ class ManagerWorkerHeavenState(flavors.StateCacheable):
      - names   (list): list of worker names that we have state for
      - workers (list): list of L{ManagerWorkerState}
     """
+
     def __init__(self):
         flavors.StateCacheable.__init__(self)
         self.addListKey('names', [])
@@ -193,6 +211,7 @@ class ManagerWorkerHeavenState(flavors.StateCacheable):
     def __repr__(self):
         return "%r" % self._dict
 
+
 class AdminWorkerHeavenState(flavors.StateRemoteCache):
     """
     I represent the state of the worker heaven in the admin.
@@ -202,6 +221,7 @@ class AdminWorkerHeavenState(flavors.StateRemoteCache):
 
 pb.setUnjellyableForClass(ManagerWorkerHeavenState, AdminWorkerHeavenState)
 
+
 class ManagerWorkerState(flavors.StateCacheable):
     """
     I represent the state of a worker in the manager.
@@ -209,6 +229,7 @@ class ManagerWorkerState(flavors.StateCacheable):
      - name: name of the worker
      - host: the IP address of the worker as seen by the manager
     """
+
     def __init__(self, **kwargs):
         flavors.StateCacheable.__init__(self)
         self.addKey('name')
@@ -220,6 +241,7 @@ class ManagerWorkerState(flavors.StateCacheable):
         return ("<ManagerWorkerState for %s on %s>"
                 % (self.get('name'), self.get('host')))
 
+
 class AdminWorkerState(flavors.StateRemoteCache):
     """
     I represent the state of a worker in the admin.
