@@ -21,22 +21,27 @@
 
 import os
 
+import gettext
 import gtk
 
 from twisted.internet import defer
 
 from flumotion.common import errors, componentui
 from flumotion.twisted import flavors
-from flumotion.twisted.compat import implements
-from flumotion.component.base.admin_gtk import BaseAdminGtk, BaseAdminGtkNode
+from flumotion.component.base.admin_gtk import BaseAdminGtk
+from flumotion.component.base.baseadminnode import BaseAdminGtkNode
 
+_ = gettext.gettext
+__version__ = "$Rev: 7162 $"
 (
   COLUMN_ID,
   COLUMN_USER,
   COLUMN_ADDRESS,
 ) = range(3)
 
+
 class KeycardsNode(BaseAdminGtkNode):
+
     def render(self):
         self._iters = {} # iter -> data dict mapping
         self.model = gtk.ListStore(str, str, str)
@@ -47,9 +52,15 @@ class KeycardsNode(BaseAdminGtkNode):
         d.addCallback(self._loadGladeFileCallback)
         return d
 
+    def stateAppend(self, state, key, value):
+        self._append(value)
+
+    def stateRemove(self, state, key, value):
+        self._remove(value)
+
     def _loadGladeFileCallback(self, widgetTree):
         self.wtree = widgetTree
-        
+
         self.widget = self.wtree.get_widget('keycards-widget')
         self.tree = self.wtree.get_widget('keycards-treeview')
         self.tree.set_model(self.model)
@@ -80,74 +91,53 @@ class KeycardsNode(BaseAdminGtkNode):
         self._uiState = result
         keycardsData = result.get('keycards')
         self.debug('_gotState: got %d keycards' % len(keycardsData))
-        
+
         for data in keycardsData:
             self._append(data)
 
-        def append(object, key, value):
-            self._append(value)
-        def remove(object, key, value):
-            self._remove(value)
-
-        self._uiState.addListener(self, append=append, remove=remove)
-
     def _expire_clicked(self, button, treeselection):
         (model, pathlist) = treeselection.get_selected_rows()
         ids = []
         for path in pathlist:
-            iter = model.get_iter(path)
-            id = model.get_value(iter, COLUMN_ID)
-            ids.append(id)
+            model_iter = model.get_iter(path)
+            ids.append(model.get_value(model_iter, COLUMN_ID))
 
         self.debug('expiring %d keycards' % len(ids))
 
         d = defer.succeed(None)
-        for id in ids:
+        for keycard_id in ids:
             # we need to pass in i as well, to make sure we actually iterate
             # instead of adding a bunch of lambdas with the same id to expire
             d.addCallback(lambda res, i: self.callRemote('expireKeycardId', i),
-                id)
+                keycard_id)
 
         return d
-        
+
     def _append(self, data):
-        id = data['id']
-        iter = self.model.append()
+        keycard_id = data['id']
+        model_iter = self.model.append()
         # GtkListStore garantuees validity of iter as long as row lives
-        self._iters[id] = iter
-        self.model.set_value(iter, COLUMN_ID, id)
+        self._iters[keycard_id] = model_iter
+        self.model.set_value(model_iter, COLUMN_ID, keycard_id)
 
         if 'username' in data.keys():
-            self.model.set_value(iter, COLUMN_USER, data['username'])
+            self.model.set_value(model_iter, COLUMN_USER, data['username'])
         if 'address' in data.keys():
-            self.model.set_value(iter, COLUMN_ADDRESS, data['address'])
+            self.model.set_value(model_iter, COLUMN_ADDRESS, data['address'])
 
     def _remove(self, data):
-        id = data['id']
-        iter = self._iters[id]
-        del self._iters[id]
-        self.model.remove(iter)
+        keycard_id = data['id']
+        model_iter = self._iters[keycard_id]
+        del self._iters[keycard_id]
+        self.model.remove(model_iter)
 
-    def propertyChanged(self, name, value):
-        if name == "pattern":
-            self.debug("pattern changed to %r" % value)
-            c = self.combobox_pattern
-            id = self.pattern_changed_id
-            c.handler_block(id)
-            c.set_active(value)
-            c.handler_unblock(id)
-
-    def cleanup(self):
-        self._uiState.removeListener(self)
 
 class HTPasswdCryptAdminGtk(BaseAdminGtk):
+
     def setup(self):
         # FIXME: have constructor take self instead ?
-        keycards = KeycardsNode(self.state, self.admin)
+        keycards = KeycardsNode(self.state, self.admin, _("Keycards"))
         self.nodes['Keycards'] = keycards
         return BaseAdminGtk.setup(self)
 
-    def cleanup(self):
-        self._nodes['Keycards'].cleanup()
-
 GUIClass = HTPasswdCryptAdminGtk
