@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007,2008 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -24,28 +24,28 @@ worker-side objects for components
 """
 
 import os
-import sys
 import time
 import socket
 
-import gobject
-
 from twisted.internet import reactor, error, defer
-from twisted.cred import error as crederror
 from twisted.spread import pb
 from twisted.python import reflect
+from zope.interface import implements
 
-from flumotion.common import interfaces, errors, log, planet, medium, pygobject
-from flumotion.common import componentui, common, registry, messages, interfaces
-from flumotion.common.planet import moods
 from flumotion.configure import configure
+from flumotion.common import interfaces, errors, log, planet, medium
+from flumotion.common import componentui, common, messages
+from flumotion.common import interfaces, reflectcall, debug
+from flumotion.common.i18n import N_, gettexter
+from flumotion.common.planet import moods
+from flumotion.common.poller import Poller
 from flumotion.twisted import credentials
 from flumotion.twisted import pb as fpb
-from flumotion.twisted.compat import implements
-from flumotion.common.pygobject import gsignal
 
-from flumotion.common.messages import N_
-T_ = messages.gettexter('flumotion')
+
+__version__ = "$Rev: 7162 $"
+T_ = gettexter()
+
 
 class ComponentClientFactory(fpb.ReconnectingFPBClientFactory):
     """
@@ -53,13 +53,14 @@ class ComponentClientFactory(fpb.ReconnectingFPBClientFactory):
     """
     logCategory = 'component'
     perspectiveInterface = interfaces.IComponentMedium
+
     def __init__(self, component):
         """
         @param component: L{flumotion.component.component.BaseComponent}
         """
         # doing this as a class method triggers a doc error
         fpb.ReconnectingFPBClientFactory.__init__(self)
-        
+
         self.component = component
         # make a medium to interface with the manager
         self.medium = component.componentMediumClass(component)
@@ -72,8 +73,14 @@ class ComponentClientFactory(fpb.ReconnectingFPBClientFactory):
 
         self.logName = component.name
 
+    def clientConnectionMade(self, broker):
+        self.medium.broker = broker
+        fpb.ReconnectingFPBClientFactory.clientConnectionMade(self, broker)
+
     # vmethod implementation
+
     def gotDeferredLogin(self, d):
+
         def remoteDisconnected(remoteReference):
             if reactor.killed:
                 self.log('Connection to manager lost due to shutdown')
@@ -84,39 +91,73 @@ class ComponentClientFactory(fpb.ReconnectingFPBClientFactory):
         def loginCallback(reference):
             self.info("Logged in to manager")
             self.debug("remote reference %r" % reference)
-            self._previously_connected = True
-        
+
             self.medium.setRemoteReference(reference)
             reference.notifyOnDisconnect(remoteDisconnected)
 
+        def loginFailedDisconnect(failure):
+            # We _always_ disconnect. Then, we may log a more specific failure
+            # message at a higher warning level.
+            self.debug('Login failed, reason: %s, disconnecting', failure)
+            self.disconnect()
+            return failure
+
         def accessDeniedErrback(failure):
-            failure.trap(crederror.UnauthorizedLogin)
+            failure.trap(errors.NotAuthenticatedError)
             self.warning('Access denied.')
-            
+
         def connectionRefusedErrback(failure):
             failure.trap(error.ConnectionRefusedError)
             self.warning('Connection to manager refused.')
-                                                          
+
         def alreadyLoggedInErrback(failure):
             failure.trap(errors.AlreadyConnectedError)
             self.warning('Component with id %s is already logged in.',
                 self.medium.authenticator.avatarId)
-                                                          
+
         def loginFailedErrback(failure):
             self.warning('Login failed, reason: %s' % failure)
 
         d.addCallback(loginCallback)
+        d.addErrback(loginFailedDisconnect)
         d.addErrback(accessDeniedErrback)
         d.addErrback(connectionRefusedErrback)
         d.addErrback(alreadyLoggedInErrback)
         d.addErrback(loginFailedErrback)
 
     # we want to save the authenticator
+
     def startLogin(self, authenticator):
         self.medium.setAuthenticator(authenticator)
         return fpb.ReconnectingFPBClientFactory.startLogin(self, authenticator)
 
+
+def _maybeDeferredChain(procs, *args, **kwargs):
+    """
+    Creates a deferred chain created by chaining calls to the given
+    procedures, each of them made with the given args and kwargs.
+    Only the result of the last procedure is returned; results for the
+    other procedures are discarded.
+
+    Failures triggered during any of the procedure short-circuit execution
+    of the other procedures and should be handled by the errbacks attached
+    to the deferred returned here.
+
+    @rtype: L{twisted.internet.defer.Deferred}
+    """
+
+    def call_proc(_, p):
+        log.debug('', 'calling %r', p)
+        return p(*args, **kwargs)
+    p, procs = procs[0], procs[1:]
+    d = defer.maybeDeferred(call_proc, None, p)
+    for p in procs:
+        d.addCallback(call_proc, p)
+    return d
+
 # needs to be before BaseComponent because BaseComponent references it
+
+
 class BaseComponentMedium(medium.PingingMedium):
     """
     I am a medium interfacing with a manager-side avatar.
@@ -137,8 +178,14 @@ class BaseComponentMedium(medium.PingingMedium):
         """
         self.comp = component
         self.authenticator = None
+        self.broker = None
+
+    def setRemoteReference(self, reference):
+        self.broker = None # We no longer need that reference
+        medium.PingingMedium.setRemoteReference(self, reference)
 
     ### our methods
+
     def setup(self, config):
         pass
 
@@ -146,8 +193,9 @@ class BaseComponentMedium(medium.PingingMedium):
         """
         Return the manager IP as seen by us.
         """
-        assert self.remote
-        peer = self.remote.broker.transport.getPeer()
+        assert self.remote or self.broker
+        broker = self.broker or self.remote.broker
+        peer = broker.transport.getPeer()
         try:
             host = peer.host
         except AttributeError:
@@ -161,7 +209,7 @@ class BaseComponentMedium(medium.PingingMedium):
         """
         Return the IP of this component based on connection to the manager.
 
-        Note: this is insufficient in general, and should be replaced by 
+        Note: this is insufficient in general, and should be replaced by
         network mapping stuff later.
         """
         assert self.remote
@@ -182,6 +230,7 @@ class BaseComponentMedium(medium.PingingMedium):
 
     ### pb.Referenceable remote methods
     ### called from manager by our avatar
+
     def remote_getState(self):
         """
         Return the state of the component, which will be serialized to a
@@ -194,7 +243,7 @@ class BaseComponentMedium(medium.PingingMedium):
         # here
         self.comp.state.set('manager-ip', self.getManagerIP())
         return self.comp.state
-        
+
     def remote_getConfig(self):
         """
         Return the configuration of the component.
@@ -202,48 +251,16 @@ class BaseComponentMedium(medium.PingingMedium):
         @rtype:   dict
         @returns: component's current configuration
         """
-        try:
-            return self.comp.config
-        except AttributeError:
-            self.debug('getConfig(), but component is not set up yet')
-            return None
-        
-    def remote_setup(self, config):
-        """
-        Set up the component and the component's medium with the given config,
-        in that order.
-        """
-        d = self.comp.setup(config)
-        d.addCallback(lambda r, c: self.setup(c), config)
-        return d
-        
-    def remote_start(self, *args, **kwargs):
-        return self.comp.start(*args, **kwargs)
-       
+        return self.comp.config
+
     def remote_stop(self):
         self.info('Stopping component')
         return self.comp.stop()
 
     def remote_reloadComponent(self):
         """Reload modules in the component."""
-        import sys
-        from twisted.python.rebuild import rebuild
-        from twisted.python.reflect import filenameToModuleName
-        name = filenameToModuleName(__file__)
-
-        ## fixme: re-fetch bundles
-
-        # reload ourselves first
-        rebuild(sys.modules[name])
-
-        # now rebuild relevant modules
-        import flumotion.common.reload
-        rebuild(sys.modules['flumotion.common'])
-        try:
-            flumotion.common.reload.reload()
-        except SyntaxError, msg:
-            raise errors.ReloadSyntaxError(msg)
-        self._reloaded()
+        from flumotion.common.reload import reloadFlumotion
+        reloadFlumotion()
 
     def remote_getUIState(self):
         """Get a WorkerComponentUIState containing details needed to
@@ -251,26 +268,45 @@ class BaseComponentMedium(medium.PingingMedium):
         """
         return self.comp.uiState
 
-    # separate method so it runs the newly reloaded one :)
-    def _reloaded(self):
-        self.info('reloaded module code for %s' % __name__)
+    def remote_getMasterClockInfo(self):
+        """
+        Base implementation of getMasterClockInfo, can be overridden by
+        subclasses. By default, just returns None.
+        """
+        return None
 
-    def remote_callMethod(self, methodName, *args, **kwargs):
-        method = getattr(self.comp, 'remote_' + methodName, None)
-        if method:
-            return method(*args, **kwargs)
-        msg = "%r doesn't have method remote_%s" % (self.comp, methodName)
-        self.warning(msg)
-        raise errors.MoMethodError(msg)
+    def remote_getVersions(self):
+        return debug.getVersions()
 
-class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
+    def remote_setFluDebug(self, debug):
+        """
+        Sets the Flumotion debugging levels based on the passed debug string.
+
+        @since: 0.6.0
+        """
+        self.debug('Setting Flumotion debug level to %s' % debug)
+        log.setDebug(debug)
+
+
+class BaseComponent(common.InitMixin, log.Loggable):
     """
     I am the base class for all Flumotion components.
 
-    @ivar name:   the name of the component
-    @type name:   string
-    @ivar medium: the component's medium
-    @type medium: L{BaseComponentMedium}
+    @ivar name:    the name of the component
+    @type name:    string
+    @ivar medium:  the component's medium
+    @type medium:  L{BaseComponentMedium}
+    @ivar uiState: state of the component to be shown in a UI.
+                   Contains at least the following keys.
+                    - cpu-percent:  percentage of CPU use in last interval
+                    - start-time:   time when component was started, in epoch
+                                    seconds
+                    - current-time: current time in epoch seconds, as seen on
+                                    component's machine, which might be out of
+                                    sync
+                    - virtual-size: virtual memory size in bytes
+                   Subclasses can add additional keys for their respective UI.
+    @type uiState: L{componentui.WorkerComponentUIState}
 
     @cvar componentMediumClass: the medium class to use for this component
     @type componentMediumClass: child class of L{BaseComponentMedium}
@@ -278,24 +314,27 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
 
     logCategory = 'basecomp'
     componentMediumClass = BaseComponentMedium
-    
-    def __init__(self):
-        # FIXME: name is unique where ? only in flow, so not in worker
-        # need to use full path maybe ?
+
+    def __init__(self, config, haveError=None):
         """
         Subclasses should not override __init__ at all.
-        
+
         Instead, they should implement init(), which will be called
         by this implementation automatically.
 
-        See L{flumotion.common.common.InitMixin} for more details.
+        L{flumotion.common.common.InitMixin} for more details.
         """
-        gobject.GObject.__init__(self)
+        self.debug("initializing %r with config %r", type(self), config)
+        self.config = config
+        self._haveError = haveError
 
         # this will call self.init() for all implementors of init()
         common.InitMixin.__init__(self)
 
+        self.setup()
+
     # BaseComponent interface for subclasses related to component protocol
+
     def init(self):
         """
         A subclass should do as little as possible in its init method.
@@ -307,26 +346,28 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
         """
         self.state = planet.WorkerJobState()
 
-        self.name = None
-        
-        #self.state.set('name', name)
+        self.name = self.config['name']
+
         self.state.set('pid', os.getpid())
-        self.state.set('mood', moods.waking.value)
+        self.setMood(moods.waking)
 
         self.medium = None # the medium connecting us to the manager's avatar
- 
-        self.uiState = componentui.WorkerComponentUIState()
 
-        # FIXME: when we need this somewhere else, put this in a class and
-        # use it that way
-        self.baseTime = time.time()
-        self.lastTime = time.time()
-        self.lastClock = time.clock()
+        self.uiState = componentui.WorkerComponentUIState()
+        self.uiState.addKey('cpu-percent')
+        self.uiState.addKey('start-time')
+        self.uiState.addKey('current-time')
+        self.uiState.addKey('virtual-size')
 
         self.plugs = {}
 
+        self._happyWaits = []
+
         # Start the cpu-usage updating.
-        self._cpuCallLater = reactor.callLater(5, self._updateCPUUsage)
+        self._lastTime = time.time()
+        self._lastClock = time.clock()
+        self._cpuPoller = Poller(self._pollCPU, 5)
+        self._memoryPoller = Poller(self._pollMemory, 60)
 
         self._shutdownHook = None
 
@@ -336,19 +377,40 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
         performs setup.
 
         Messages can be added to the component state's 'messages' list key.
-        Any error messages added will trigger the component going to sad
-        an L{flumotion.common.errors.ComponentSetupError} being raised;
-        do_setup() will not be called.
+        Any error messages added will trigger the component going to sad,
+        with L{flumotion.common.errors.ComponentSetupError} being raised
+        before getting to setup stage; do_setup() will not be called.
 
         In the event of a fatal problem that can't be expressed through an
-        error message, this method should set the mood to sad and raise the
-        error on its own.
+        error message, this method should raise an exception or return a
+        failure.
 
-        self.config will be set before this is called.
-
-        @Returns: L{twisted.internet.defer.Deferred}
+        It is not necessary to chain up in this function. The return
+        value may be a deferred.
         """
-        return defer.succeed(None)
+        return defer.maybeDeferred(self.check_properties,
+                                   self.config['properties'],
+                                   self.addMessage)
+
+    def check_properties(self, properties, addMessage):
+        """
+        BaseComponent convenience vmethod for running checks.
+
+        A component implementation can override this method to run any
+        checks that it needs to. Typically, a check_properties
+        implementation will call the provided addMessage() callback to
+        note warnings or errors. For errors, addMessage() will set
+        component's mood to sad, which will abort the init process
+        before getting to do_setup().
+
+        @param properties: The component's properties
+        @type properties: dict of string => object
+        @param addMessage: Thunk to add a message to the component
+                           state. Will raise an exception if the
+                           message is of level ERROR.
+        @type addMessage: L{flumotion.common.messages.Message} -> None
+        """
+        pass
 
     def do_setup(self):
         """
@@ -358,24 +420,37 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
         Non-programming errors should not be raised, but returned as a
         failing deferred.
 
-        self.config will be set before this is called.
-
-        @Returns: L{twisted.internet.defer.Deferred}
+        The return value may be a deferred.
         """
-        return defer.succeed(None)
+        for socket, plugs in self.config['plugs'].items():
+            self.plugs[socket] = []
+            for plug in plugs:
+                entry = plug['entries']['default']
+                instance = reflectcall.reflectCall(entry['module-name'],
+                                                   entry['function-name'],
+                                                   plug)
+                self.plugs[socket].append(instance)
+                self.debug('Starting plug %r on socket %s',
+                           instance, socket)
+                instance.start(self)
 
-    def do_start(self, *args, **kwargs):
-        """
-        BaseComponent vmethod for starting up. If you override this
-        method, you are responsible for arranging that the component
-        becomes happy.
+        # Call check methods, starting from the base class and working down to
+        # subclasses.
+        checks = common.get_all_methods(self, 'do_check', False)
+
+        def checkErrorCallback(result):
+            # if the mood is now sad, it means an error was encountered
+            # during check, and we should return a failure here.
+            # since the checks are responsible for adding a message,
+            # this is a handled error.
+            current = self.state.get('mood')
+            if current == moods.sad.value:
+                self.warning('Running checks made the component sad.')
+                raise errors.ComponentSetupHandledError()
+
+        checks.append(checkErrorCallback)
+        return _maybeDeferredChain(checks, self)
 
-        @Returns: L{twisted.internet.defer.Deferred}
-        """
-        # default behavior
-        self.setMood(moods.happy)
-        return defer.succeed(None)
-       
     def do_stop(self):
         """
         BaseComponent vmethod for stopping.
@@ -384,130 +459,67 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
 
         @Returns: L{twisted.internet.defer.Deferred}
         """
-        return defer.succeed(None)
- 
+        for socket, plugs in self.plugs.items():
+            for plug in plugs:
+                self.debug('Stopping plug %r on socket %s', plug, socket)
+                plug.stop(self)
+
+        for message in self.state.get('messages'):
+            # FIXME: not necessary
+            self.state.remove('messages', message)
+
+        if self._cpuPoller:
+            self._cpuPoller.stop()
+            self._cpuPoller = None
+        if self._memoryPoller:
+            self._memoryPoller.stop()
+            self._memoryPoller = None
+
+        if self._shutdownHook:
+            self.debug('_stoppedCallback: firing shutdown hook')
+            self._shutdownHook()
+
     ### BaseComponent implementation related to compoment protocol
-    ### called by manager through medium
-    def setup(self, config, *args, **kwargs):
+
+    def setup(self):
         """
-        Sets up the component with the given config.  Called by the manager
-        through the medium.
-
-        @Returns: L{twisted.internet.defer.Deferred}
-        @raise    flumotion.common.errors.ComponentSetupError:
-                  when an error happened during setup of the component
+        Sets up the component.  Called during __init__, so be sure not
+        to raise exceptions, instead adding messages to the component
+        state.
         """
-        def setup_plugs():
-            # by this time we have a medium, so we can load bundles
-            reg = registry.getRegistry()
 
-            def load_bundles():
-                modules = {}
-                for plugs in config['plugs'].values():
-                    for plug in plugs:
-                        modules[plug['type']] = True
-                for plugtype in modules.keys():
-                    # we got this far, it should work
-                    entry = reg.getPlug(plugtype).getEntry()
-                    modules[plugtype] = entry.getModuleName()
-                if not modules:
-                    return defer.succeed(True) # shortcut
-                elif not self.medium:
-                    self.warning('Not connected to a medium, cannot '
-                                 'load bundles -- assuming all modules '
-                                 'are available')
-                    return defer.succeed(True)
-                else:
-                    loader = self.medium.bundleLoader
-                    return loader.getBundles(moduleName=modules.values())
-                
-            def make_plugs():
-                for socket, plugs in config['plugs'].items():
-                    self.plugs[socket] = []
-                    for plug in plugs:
-                        entry = reg.getPlug(plug['type']).getEntry()
-                        module = reflect.namedAny(entry.getModuleName())
-                        proc = getattr(module, entry.getFunction())
-                        instance = proc(plug)
-                        self.plugs[socket].append(instance)
+        def run_setups():
+            setups = common.get_all_methods(self, 'do_setup', False)
+            return _maybeDeferredChain(setups, self)
 
-            try:
-                d = load_bundles()
-                d.addCallback(lambda x: make_plugs())
-                return d
-            except Exception, e:
-                self.debug("Exception while loading bundles: %s" % 
-                    log.getExceptionMessage(e))
-                return defer.fail(e)
+        def setup_complete(_):
+            self.debug('setup completed')
+            self.setup_completed()
 
-        def checkErrorCallback(result):
-            # if the mood is now sad, it means an error was encountered
-            # during check, and we should return a failure here.
-            # since the checks are responsible for adding a message,
-            # this is a handled error.
-            current = self.state.get('mood')            
-            if current == moods.sad.value:
-                self.warning('Running checks made the component sad.')
-                raise errors.ComponentSetupHandledError()
+        def got_error(failure):
+            txt = log.getFailureMessage(failure)
+            self.debug('got_error: %s', txt)
+            if not failure.check(errors.ComponentSetupHandledError):
+                self.warning('Setup failed: %s', txt)
+                m = messages.Error(T_(N_("Could not setup component.")),
+                                   debug=txt,
+                                   mid="component-setup-%s" % self.name)
+                # will call setMood(moods.sad)
+                self.addMessage(m)
+
+            # swallow
+            return None
 
-        self.debug("setup() called with config %r", config)
         self.setMood(moods.waking)
-        self._setConfig(config)
-        # now we have a name, set it on the medium too
-        if self.medium:
-            self.medium.logName = self.getName()
-        d = setup_plugs()
-        d.addCallback(lambda r: self.do_check())
-        d.addCallback(checkErrorCallback)
-        d.addCallback(lambda r: self.do_setup())
-        def setupErrback(failure):
-            # pass through handled errors
-            if failure.check(errors.ComponentSetupHandledError):
-                return failure
+        self.uiState.set('start-time', time.time())
 
-            self.warning('Could not set up component: %s',
-                log.getFailureMessage(failure))
-            m = messages.Error(T_(N_("Could not setup component.")),
-                debug=log.getFailureMessage(failure),
-                id="component-setup-%s" % self.name)
-            self.state.append('messages', m)
-            self.setMood(moods.sad)
-            raise errors.ComponentSetupHandledError(
-                'Could not set up component')
+        d = run_setups()
+        d.addCallbacks(setup_complete, got_error)
+        # all status info via messages and the mood
 
-        d.addErrback(setupErrback)
-        return d
-
-    def start(self, *args, **kwargs):
-        """
-        Tell the component to start.  This is called when all its dependencies
-        are already started.
-
-        To hook onto this method, implement your own do_start method.
-        See BaseComponent.do_start() for what your do_start method is
-        responsible for doing.
-
-        Again, don't override this method. Thanks.
-        """
-        self.debug('BaseComponent.start')
-
-        def start_plugs():
-            for socket, plugs in self.plugs.items():
-                for plug in plugs:
-                    self.debug('Starting plug %r on socket %s', plug, socket)
-                    plug.start(self)
-
-        try:
-            start_plugs()
-            ret = self.do_start(*args, **kwargs)
-            assert isinstance(ret, defer.Deferred), \
-                   "do_start %r must return a deferred" % self.do_start
-            self.debug('start: returning value %s' % ret)
-            return ret
-        except Exception, e:
-            self.debug("Exception during component do_start: %s" % 
-                log.getExceptionMessage(e))
-            return defer.fail(e)
+    def setup_completed(self):
+        self.debug('turning happy')
+        self.setMood(moods.happy)
 
     def setShutdownHook(self, shutdownHook):
         """
@@ -515,7 +527,7 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
         When a component is stopped, then this hook will be fired.
         """
         self._shutdownHook = shutdownHook
-        
+
     def stop(self):
         """
         Tell the component to stop.
@@ -524,40 +536,15 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
         """
         self.debug('BaseComponent.stop')
 
-        def stop_plugs(ret):
-            for socket, plugs in self.plugs.items():
-                for plug in plugs:
-                    self.debug('Stopping plug %r on socket %s', plug, socket)
-                    plug.stop(self)
-            return ret
-
-        def fireShutdownHook(ret):
-            if self._shutdownHook:
-                self.debug('_stoppedCallback: firing shutdown hook')
-                self._shutdownHook()
-
+        # Set ourselves to waking while we're shutting down.
         self.setMood(moods.waking)
-        for message in self.state.get('messages'):
-            self.state.remove('messages', message)
 
-        if self._cpuCallLater:
-            self._cpuCallLater.cancel()
-            self._cpuCallLater = None
+        # Run stop methods, starting from the subclass, up to this base class.
+        stops = common.get_all_methods(self, 'do_stop', True)
+        return _maybeDeferredChain(stops, self)
 
-        d = self.do_stop()
-        d.addCallback(stop_plugs)
-        d.addBoth(fireShutdownHook)
-        return d
-
-    ### GObject methods
-    def emit(self, name, *args):
-        if 'uninitialized' in str(self):
-            self.warning('Uninitialized object!')
-            #self.__gobject_init__()
-        else:
-            gobject.GObject.emit(self, name, *args)
-        
     ### BaseComponent public methods
+
     def getName(self):
         return self.name
 
@@ -570,6 +557,7 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
     def setMedium(self, medium):
         assert isinstance(medium, BaseComponentMedium)
         self.medium = medium
+        self.medium.logName = self.getName()
 
     def setMood(self, mood):
         """
@@ -585,9 +573,16 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
             self.info('tried to set mood to %r, but already sad :-(' % mood)
             return
 
-        self.debug('MOOD changed to %r' % mood)
+        self.doLog(log.DEBUG, -2, 'MOOD changed to %r by caller', mood)
         self.state.set('mood', mood.value)
 
+        if mood == moods.happy:
+            while self._happyWaits:
+                self._happyWaits.pop(0).callback(None)
+        elif mood == moods.sad:
+            while self._happyWaits:
+                self._happyWaits.pop(0).errback(errors.ComponentStartError())
+
     def getMood(self):
         """
         Gets the mood on the component.
@@ -596,7 +591,17 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
         """
         return self.state.get('mood')
 
-        
+    def waitForHappy(self):
+        mood = self.getMood()
+        if mood == moods.happy.value:
+            return defer.succeed(None)
+        elif mood == moods.sad.value:
+            return defer.fail(errors.ComponentStartError())
+        else:
+            d = defer.Deferred()
+            self._happyWaits.append(d)
+            return d
+
     def addMessage(self, message):
         """
         Add a message to the component.
@@ -608,7 +613,9 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
         if message.level == messages.ERROR:
             self.debug('error message, turning sad')
             self.setMood(moods.sad)
-        
+            if self._haveError:
+                self._haveError(message)
+
     def fixRenamedProperties(self, properties, list):
         """
         Fix properties that have been renamed from a previous version,
@@ -621,14 +628,14 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
         """
         found = []
         for old, new in list:
-            if properties.has_key(old):
+            if old in properties:
                 found.append((old, new))
 
         if found:
             m = messages.Warning(T_(N_(
                 "Your configuration uses deprecated properties.  "
                 "Please update your configuration and correct them.\n")),
-                id = "deprecated")
+                mid="deprecated")
             for old, new in found:
                 m.add(T_(N_(
                 "Please rename '%s' to '%s'.\n"),
@@ -659,29 +666,31 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
                        'no manager.'
                        % (methodName, args, kwargs))
 
-    # private methods
-    def _setConfig(self, config):
-        if self.name:
-            assert config['name'] == self.name, \
-                   "Can't change name while running"
-        self.config = config
-        self.name = config['name']
-
-    def _updateCPUUsage(self):
+    def _pollCPU(self):
         # update CPU time stats
         nowTime = time.time()
         nowClock = time.clock()
-        deltaTime = nowTime - self.lastTime
-        deltaClock = nowClock - self.lastClock
-        CPU = deltaClock/deltaTime
-        self.log('latest CPU use: %r' % CPU)
-        self.state.set('cpu', CPU)
-        deltaTime = nowTime - self.baseTime
-        deltaClock = nowClock
-        CPU = deltaClock/deltaTime
-        self.lastTime = nowTime
-        self.lastClock = nowClock
+        deltaTime = nowTime - self._lastTime
+        deltaClock = nowClock - self._lastClock
+        self._lastTime = nowTime
+        self._lastClock = nowClock
+        # deltaClock can be < 0 if time.clock() wrapped around
+        if deltaClock >= 0:
+            CPU = deltaClock/deltaTime
+            self.log('latest CPU use: %r', CPU)
+            self.uiState.set('cpu-percent', CPU)
 
-        self._cpuCallLater = reactor.callLater(5, self._updateCPUUsage)
+        self.uiState.set('current-time', nowTime)
 
-pygobject.type_register(BaseComponent)
+    def _pollMemory(self):
+        # Figure out our virtual memory size and report that.
+        # I don't know a nicer way to find vsize than groping /proc/
+        handle = open('/proc/%d/stat' % os.getpid())
+        line = handle.read()
+        handle.close()
+        fields = line.split()
+        # field 1 (comm) could potentially contain spaces and thus split over
+        # multiple list items, but our processes do not contain spaces
+        vsize = int(fields[22])
+        self.log('vsize is %d', vsize)
+        self.uiState.set('virtual-size', vsize)
