@@ -20,8 +20,15 @@
 # Headers in this file shall remain intact.
 
 from flumotion.component import feedcomponent
+from flumotion.common import messages, gstreamer
+from flumotion.common.i18n import N_, gettexter
+
+__version__ = "$Rev: 8057 $"
+T_ = gettexter()
+
 
 class Smoke(feedcomponent.ParseLaunchComponent):
+
     def get_pipeline_string(self, properties):
         return 'ffmpegcolorspace ! smokeenc name=encoder'
 
@@ -31,3 +38,11 @@ class Smoke(feedcomponent.ParseLaunchComponent):
         for p in ('qmin', 'qmax', 'threshold', 'keyframe'):
             if p in properties:
                 element.set_property(p, properties[p])
+
+        jpegVersion = gstreamer.get_plugin_version('jpeg')
+        if jpegVersion < (0, 10, 11, 1):
+            m = messages.Warning(
+                T_(N_("The 'smoke' encoder has a bug in versions previous "
+                      "to 0.10.11. It will not work unless it is updated.")),
+                mid="smokeenc-bug")
+            self.addMessage(m)
