@@ -21,11 +21,15 @@
 
 from flumotion.component import feedcomponent
 
+__version__ = "$Rev: 7162 $"
+
+
 class Speex(feedcomponent.ParseLaunchComponent):
+
     def get_pipeline_string(self, properties):
         return 'speexenc name=encoder'
 
     def configure_pipeline(self, pipeline, properties):
         element = pipeline.get_by_name('encoder')
-        if properties.has_key('bitrate'):
+        if 'bitrate' in properties:
             element.set_property('bitrate', properties['bitrate'])
