@@ -19,11 +19,15 @@
 
 # Headers in this file shall remain intact.
 
-from flumotion.component import feedcomponent
 from flumotion.common import messages
+from flumotion.common.i18n import N_, gettexter
+from flumotion.component import feedcomponent
+from flumotion.worker.checks import check
+
+
+__version__ = "$Rev: 7161 $"
+T_ = gettexter()
 
-from flumotion.common.messages import N_
-T_ = messages.gettexter('flumotion')
 
 class Theora(feedcomponent.ParseLaunchComponent):
     checkTimestamp = True
@@ -32,15 +36,7 @@ class Theora(feedcomponent.ParseLaunchComponent):
     def do_check(self):
         self.debug('running Theora check')
         from flumotion.worker.checks import encoder
-        d = encoder.checkTheora()
-
-        d.addCallback(self._checkCallback)
-
-        return d
-
-    def _checkCallback(self, result):
-        for m in result.messages:
-            self.addMessage(m)
+        return check.do_check(self, encoder.checkTheora)
 
     def get_pipeline_string(self, properties):
         return "ffmpegcolorspace ! theoraenc name=encoder"
@@ -50,12 +46,12 @@ class Theora(feedcomponent.ParseLaunchComponent):
 
         props = ('bitrate',
                  'quality',
-                 'keyframe-threshold',
-                 'keyframe-mindistance', 
+                 'keyframe-mindistance',
                  ('quick-compress', 'quick'),
                  ('keyframe-maxdistance', 'keyframe-freq'),
                  ('keyframe-maxdistance', 'keyframe-force'),
-                 'noise-sensitivity')
+                 'noise-sensitivity',
+                 'sharpness')
 
         # F0.6: remove this code
         # before 0.3.2, bitrate was interpreted as kbps, inconsistent
@@ -67,22 +63,25 @@ class Theora(feedcomponent.ParseLaunchComponent):
                 self.addMessage(
                     messages.Warning(T_(N_(
                         "Your configuration uses 'bitrate' expressed in "
-                        "kbit/sec.  Please convert it to a value in bit/sec by "
-                        "multiplying the value by 1000.")), id='bitrate'))
+                        "kbit/sec.  Please convert it to a value in bit/sec by"
+                        " multiplying the value by 1000.")), mid='bitrate'))
                 properties['bitrate'] *= 1000
 
         for p in props:
-            pproperty = isinstance(p, tuple) and p[0] or p
-            eproperty = isinstance(p, tuple) and p[1] or p
+            if isinstance(p, tuple):
+                pproperty, eproperty = p
+            else:
+                pproperty = eproperty = p
 
-            if pproperty in properties:
-                self.debug('Setting GStreamer property %s to %r' % (
-                    eproperty, properties[pproperty]))
-                # FIXME: GStreamer 0.10 has bitrate in kbps, inconsistent
-                # with all other elements, so fix it up
-                if pproperty == 'bitrate':
-                    element.set_property(eproperty, 
-                        int(properties[pproperty]/1000))
-                else:
-                    element.set_property(eproperty, properties[pproperty])
+            if not pproperty in properties:
+                continue
 
+            value = properties[pproperty]
+            self.debug('Setting GStreamer property %s to %r' % (
+                eproperty, value))
+
+            # FIXME: GStreamer 0.10 has bitrate in kbps, inconsistent
+            # with all other elements, so fix it up
+            if pproperty == 'bitrate':
+                value = int(value/1000)
+            element.set_property(eproperty, value)
