@@ -19,12 +19,15 @@
 
 # Headers in this file shall remain intact.
 
-
 import gst
 
+from flumotion.common import gstreamer
 from flumotion.component import feedcomponent
 from vorbisutils import get_max_sample_rate, get_preferred_sample_rate
 
+__version__ = "$Rev: 7889 $"
+
+
 class Vorbis(feedcomponent.ParseLaunchComponent):
     checkTimestamp = True
     checkOffset = True
@@ -35,7 +38,7 @@ class Vorbis(feedcomponent.ParseLaunchComponent):
         d = encoder.checkVorbis()
 
         d.addCallback(self._checkCallback)
-        
+
         return d
 
     def _checkCallback(self, result):
@@ -46,9 +49,11 @@ class Vorbis(feedcomponent.ParseLaunchComponent):
         self.bitrate = properties.get('bitrate', -1)
         self.quality = properties.get('quality', 0.3)
         self.channels = properties.get('channels', 2)
-
-        return ('audioresample name=ar ! audioconvert ! capsfilter name=cf '
-                '! vorbisenc name=enc')
+        resampler = 'audioresample'
+        if gstreamer.element_factory_exists('legacyresample'):
+            resampler = 'legacyresample'
+        return ('%s name=ar ! audioconvert ! capsfilter name=cf '
+                '! vorbisenc name=enc' % resampler)
 
     def configure_pipeline(self, pipeline, properties):
         enc = pipeline.get_by_name('enc')
@@ -73,12 +78,14 @@ class Vorbis(feedcomponent.ParseLaunchComponent):
             # now do necessary filtercaps
             rate = in_rate
             if self.bitrate > -1:
-                maxsamplerate = get_max_sample_rate(self.bitrate, self.channels)
+                maxsamplerate = get_max_sample_rate(
+                    self.bitrate, self.channels)
                 if in_rate > maxsamplerate:
                     rate = get_preferred_sample_rate(maxsamplerate)
                     self.debug(
-                        'rate %d > max rate %d (for %d kbit/sec), selecting rate %d instead' % (
-                            in_rate, maxsamplerate, self.bitrate, rate))
+                        'rate %d > max rate %d (for %d kbit/sec), '
+                        'selecting rate %d instead' % (
+                        in_rate, maxsamplerate, self.bitrate, rate))
 
             caps_str = 'audio/x-raw-float, rate=%d, channels=%d' % (rate,
                         self.channels)
