@@ -21,12 +21,16 @@
 
 from gettext import gettext as _
 
-from flumotion.component.base import admin_gtk
-from flumotion.component.effects.volume import admin_gtk as vadmin_gtk
+from flumotion.component.base.admin_gtk import BaseAdminGtk
+from flumotion.component.base.baseadminnode import BaseAdminGtkNode
+from flumotion.component.effects.volume.admin_gtk import VolumeAdminGtkNode
 
-class AudioTestAdminGtkNode(admin_gtk.BaseAdminGtkNode):
+__version__ = "$Rev: 7888 $"
+
+
+class AudioTestAdminGtkNode(BaseAdminGtkNode):
     logCategory = 'audiotest'
-    glade_file = 'flumotion/component/producers/audiotest/audiotest.glade'
+    gladeFile = 'flumotion/component/producers/audiotest/audiotest.glade'
     uiStateHandlers = None
 
     def haveWidgetTree(self):
@@ -34,6 +38,8 @@ class AudioTestAdminGtkNode(admin_gtk.BaseAdminGtkNode):
         self._scale = self.wtree.get_widget('scale-frequency')
         self._spinbutton = self.wtree.get_widget('spinbutton-frequency')
         self._combobox = self.wtree.get_widget('combobox-wave')
+        self._combobox.prefill(['Sine', 'Square', 'Saw'])
+
         self._scale_change_id = self._scale.connect('value-changed',
             self.frequency_changed_cb)
         self._spinbutton_change_id = self._spinbutton.connect('value-changed',
@@ -42,14 +48,14 @@ class AudioTestAdminGtkNode(admin_gtk.BaseAdminGtkNode):
             self.wave_changed_cb)
 
     def setUIState(self, state):
-        admin_gtk.BaseAdminGtkNode.setUIState(self, state)
+        BaseAdminGtkNode.setUIState(self, state)
         if not self.uiStateHandlers:
             self.uiStateHandlers = {'wave': self.waveSet,
                                     'frequency': self.frequencySet,
-                                    'rate': self.rateSet}
+                                    'samplerate': self.samplerateSet}
         for k, handler in self.uiStateHandlers.items():
             handler(state.get(k))
-                                    
+
     def frequency_changed_cb(self, widget):
         value = widget.get_value()
         d = self.callRemote("setFrequency", value)
@@ -64,11 +70,11 @@ class AudioTestAdminGtkNode(admin_gtk.BaseAdminGtkNode):
         waveName = widget.get_active()
         d = self.callRemote("setWave", waveName)
         d.addErrback(self.warningFailure)
-        
-    def rateSet(self, rate):
-        self._scale.set_range(1, rate)
-        self._spinbutton.set_range(1, rate)
-        
+
+    def samplerateSet(self, samplerate):
+        self._scale.set_range(1, samplerate)
+        self._spinbutton.set_range(1, samplerate)
+
     def frequencySet(self, value):
         self._scale.handler_block(self._scale_change_id)
         self._scale.set_value(value)
@@ -82,12 +88,14 @@ class AudioTestAdminGtkNode(admin_gtk.BaseAdminGtkNode):
         self._combobox.set_active(value)
         self._combobox.handler_unblock(self._combobox_change_id)
 
-class AudioTestAdminGtk(admin_gtk.BaseAdminGtk):
+
+class AudioTestAdminGtk(BaseAdminGtk):
+
     def setup(self):
-        volume = vadmin_gtk.VolumeAdminGtkNode(self.state, self.admin,
-                                               'volume', title=_("Volume"))
+        volume = VolumeAdminGtkNode(self.state, self.admin,
+                                    'volume', title=_("Volume"))
         self.nodes['Volume'] = volume
         audiotest = AudioTestAdminGtkNode(self.state, self.admin,
                                           title=_("Audio Test"))
         self.nodes['Audio Test'] = audiotest
-        return admin_gtk.BaseAdminGtk.setup(self)
+        return BaseAdminGtk.setup(self)
