@@ -21,14 +21,20 @@
 
 import gst
 
+from twisted.internet import defer
+
+from flumotion.common import errors, gstreamer, messages
+from flumotion.common.i18n import N_, gettexter
 from flumotion.component import feedcomponent
-from flumotion.common import errors, gstreamer, messages
 from flumotion.component.effects.volume import volume
+from flumotion.worker.checks import check
+
+__version__ = "$Rev: 7649 $"
+T_ = gettexter()
 
-from flumotion.common.messages import N_
-T_ = messages.gettexter('flumotion')
 
 class AudioTestMedium(feedcomponent.FeedComponentMedium):
+
     def remote_setFrequency(self, frequency):
         """
         @type frequency: int
@@ -41,6 +47,7 @@ class AudioTestMedium(feedcomponent.FeedComponentMedium):
         """
         return self.comp.setWave(wave)
 
+
 class AudioTest(feedcomponent.ParseLaunchComponent):
     componentMediumClass = AudioTestMedium
 
@@ -49,11 +56,19 @@ class AudioTest(feedcomponent.ParseLaunchComponent):
                             ['Sine', 'Square', 'Saw', 'Triangle'])
         self.uiState.addKey('wave', 0)
         self.uiState.addKey('frequency', 440)
-        self.uiState.addKey('rate', 8000)
+        self.uiState.addKey('samplerate', 441000)
+
+    def do_check(self):
+        levelD = check.do_check(self, check.checkPlugin, 'level', 'level')
+        audiotestD = check.do_check(self, check.checkPlugin, 'audiotestsrc',
+            'audiotestsrc')
+        volumeD = check.do_check(self, check.checkPlugin, 'volume', 'volume')
+        dl = defer.DeferredList([levelD, audiotestD, volumeD])
+        return dl
 
     def get_pipeline_string(self, properties):
-        rate = properties.get('rate', 8000)
-        self.rate = rate
+        samplerate = properties.get('samplerate', 44100)
+        self.samplerate = samplerate
         volume = properties.get('volume', 1.0)
 
         is_live = 'is-live=true'
@@ -65,16 +80,16 @@ class AudioTest(feedcomponent.ParseLaunchComponent):
         return ('%s name=source %s ! identity name=identity silent=TRUE ! ' \
             'audio/x-raw-int,rate=%d ! ' \
             'volume name=volume volume=%f ! level name=level'
-                % (source, is_live, rate, volume))
+                % (source, is_live, samplerate, volume))
 
     def configure_pipeline(self, pipeline, properties):
 
-        self.fixRenamedProperties(properties, [ 
-             ('freq', 'frequency'), 
-             ]) 
+        self.fixRenamedProperties(properties, [
+             ('freq', 'frequency'),
+             ])
 
         element = self.get_element('source')
-        if properties.has_key('frequency'):
+        if 'frequency' in properties:
             element.set_property('freq', properties['frequency'])
             self.uiState.set('frequency', properties['frequency'])
 
@@ -86,8 +101,8 @@ class AudioTest(feedcomponent.ParseLaunchComponent):
                 self.addMessage(
                     messages.Warning(T_(N_(
                         "The 'drop-probability' property is specified, but "
-                        "it only works with GStreamer core newer than 0.10.12. "
-                        "You should update your version of GStreamer."))))
+                        "it only works with GStreamer core newer than 0.10.12."
+                        " You should update your version of GStreamer."))))
             else:
                 drop_probability = properties['drop-probability']
                 if drop_probability < 0.0 or drop_probability > 1.0:
@@ -100,7 +115,7 @@ class AudioTest(feedcomponent.ParseLaunchComponent):
                     identity.set_property('drop-probability',
                         drop_probability)
 
-        self.uiState.set('rate', self.rate)
+        self.uiState.set('samplerate', self.samplerate)
         self.uiState.set('wave', int(element.get_property('wave')))
 
         level = pipeline.get_by_name('level')
@@ -115,7 +130,7 @@ class AudioTest(feedcomponent.ParseLaunchComponent):
     def getVolume(self):
         element = self.get_element('volume')
         return element.get_property('volume')
-    
+
     def setFrequency(self, frequency):
         element = self.get_element('source')
         element.set_property('freq', frequency)
