@@ -20,21 +20,19 @@
 # Headers in this file shall remain intact.
 
 from flumotion.component.base import admin_gtk
-from flumotion.component.effects.colorbalance.admin_gtk import ColorbalanceAdminGtkNode
+from flumotion.component.effects.colorbalance.admin_gtk import \
+     ColorbalanceAdminGtkNode
+
+__version__ = "$Rev: 7162 $"
+
 
 class BTTVAdminGtk(admin_gtk.BaseAdminGtk):
+
     def setup(self):
         colorbalance = ColorbalanceAdminGtkNode(self.state, self.admin,
                                                 'outputColorbalance',
                                                 'Output Colorbalance')
         self.nodes['Colorbalance'] = colorbalance
-
-    def component_effectPropertyChanged(self, effectName, propertyName, value):
-        if not effectName == "outputColorbalance":
-            self.warning("Unknown effect '%s'" % effectName)
-            return
-
-        cb = self.nodes['Colorbalance']
-        cb.propertyChanged(propertyName, value)
+        return admin_gtk.BaseAdminGtk.setup(self)
 
 GUIClass = BTTVAdminGtk
