@@ -23,12 +23,15 @@ import gst
 import gobject
 
 from flumotion.common import errors, messages
+from flumotion.common.i18n import N_, gettexter
 from flumotion.component import feedcomponent
 
-from flumotion.common.messages import N_
-T_ = messages.gettexter('flumotion')
+__version__ = "$Rev: 7832 $"
+T_ = gettexter()
+
 
 class LooperMedium(feedcomponent.FeedComponentMedium):
+
     def __init__(self, comp):
         feedcomponent.FeedComponentMedium.__init__(self, comp)
 
@@ -52,6 +55,7 @@ class LooperMedium(feedcomponent.FeedComponentMedium):
 # To loop a segment, when you get the segment_done message
 # asynchronously, just do a new segment seek.
 
+
 class Looper(feedcomponent.ParseLaunchComponent):
 
     componentMediumClass = LooperMedium
@@ -72,11 +76,22 @@ class Looper(feedcomponent.ParseLaunchComponent):
         self.uiState.addKey('position', 0)
 
     def do_check(self):
+
         def on_result(result):
             for m in result.messages:
                 self.addMessage(m)
 
         from flumotion.component.producers import checks
+        version = checks.get_pygst_version(gst)
+        if version >= (0, 10, 11, 0) and version < (0, 10, 14, 0):
+            # if it's going to segfault it won't have time to deliver
+            # messages to manager, otherwise we don't need to show it!
+            # but we can add a log message
+            self.warning('the version of gst-python you are using is known to '
+                         'cause segfault in the looper component, please '
+                         'update to the latest release')
+            self.warning('... just so you know, in case it crashes')
+
         d = checks.checkTicket349()
         d.addCallback(on_result)
         return d
@@ -85,17 +100,19 @@ class Looper(feedcomponent.ParseLaunchComponent):
         # setup the properties
         self.bus = None
         self.videowidth = properties.get('width', 240)
-        self.videoheight = properties.get('height', int(576 * self.videowidth/720.))
+        self.videoheight = properties.get(
+            'height', int(576 * self.videowidth/720.))
         self.videoframerate = properties.get('framerate', (25, 2))
         self.filelocation = properties.get('location')
 
-        vstruct = gst.structure_from_string("video/x-raw-yuv,width=%(width)d,height=%(height)d" %
-                                            dict (width=self.videowidth, height=self.videoheight))
+        vstruct = gst.structure_from_string(
+            "video/x-raw-yuv,width=%(width)d,height=%(height)d" %
+            dict(width=self.videowidth, height=self.videoheight))
         vstruct['framerate'] = gst.Fraction(self.videoframerate[0],
                                             self.videoframerate[1])
 
         vcaps = gst.Caps(vstruct)
-        
+
         self.run_discoverer()
 
         template = (
@@ -106,17 +123,29 @@ class Looper(feedcomponent.ParseLaunchComponent):
             '       ! videorate name=videorate'
             '       ! videoscale'
             '       ! %(vcaps)s'
-            '       ! identity name=vident sync=true silent=true ! @feeder::video@'
+            '       ! identity name=vident sync=true silent=true'
+            '       ! @feeder:video@'
             '    demux. ! queue ! vorbisdec name=vorbisdec'
             '       ! identity name=audiolive single-segment=true silent=true'
             '       ! audioconvert'
             '       ! audio/x-raw-int,width=16,depth=16,signed=(boolean)true'
-            '       ! identity name=aident sync=true silent=true ! @feeder::audio@'
+            '       ! identity name=aident sync=true silent=true'
+            '       ! @feeder:audio@'
             % dict(location=self.filelocation, vcaps=vcaps))
 
         return template
 
+    def make_message_for_gstreamer_error(self, gerror, debug):
+        if gerror.domain == 'gst-resource-error-quark':
+            return messages.Error(T_(N_(
+                "Could not open file '%s' for reading."), self.filelocation),
+                debug='%s\n%s' % (gerror.message, debug),
+                mid=gerror.domain, priority=40)
+        base = feedcomponent.ParseLaunchComponent
+        return base.make_message_for_gstreamer_error(gerror, debug)
+
     def run_discoverer(self):
+
         def discovered(d, ismedia):
             self.uiState.set('info-location', self.filelocation)
             self.uiState.set('info-duration',
@@ -136,12 +165,12 @@ class Looper(feedcomponent.ParseLaunchComponent):
         d = discoverer.Discoverer(self.filelocation)
         d.connect('discovered', discovered)
         d.discover()
- 
+
     def on_segment_done(self):
         self.do_seek(False)
         self.nbiterations += 1
         self.uiState.set('num-iterations', self.nbiterations)
-        
+
     def on_pads_blocked(self):
         for src, sink in self.pads_to_link:
             src.link(sink)
@@ -151,20 +180,22 @@ class Looper(feedcomponent.ParseLaunchComponent):
         self.pads_to_link = []
         self.nbiterations = 0
         self.uiState.set('num-iterations', self.nbiterations)
-        
+
     def configure_pipeline(self, pipeline, properties):
+
         def on_message(bus, message):
-            handlers = {(pipeline, gst.MESSAGE_SEGMENT_DONE): 
+            handlers = {(pipeline, gst.MESSAGE_SEGMENT_DONE):
                         self.on_segment_done,
                         (pipeline, gst.MESSAGE_APPLICATION):
                         self.on_pads_blocked}
 
             if (message.src, message.type) in handlers:
                 handlers[(message.src, message.type)]()
-            
+
         self.oggdemux = pipeline.get_by_name("demux")
 
         for name in 'aident', 'vident':
+
             def blocked(x, is_blocked):
                 if not x in self.pads_awaiting_block:
                     return
@@ -191,7 +222,7 @@ class Looper(feedcomponent.ParseLaunchComponent):
     def do_seek(self, flushing):
         """
         Restarts the looping.
-        
+
         Returns True if the seeking was accepted,
         Returns False otherwiser
         """
@@ -200,7 +231,8 @@ class Looper(feedcomponent.ParseLaunchComponent):
         return self.oggdemux.seek(1.0, gst.FORMAT_TIME, flags,
                                   gst.SEEK_TYPE_SET, 0, gst.SEEK_TYPE_END, 0)
 
-    def do_start(self, *args, **kwargs):
+    def do_setup(self):
+
         def check_time():
             self.log("checking position")
             try:
@@ -211,15 +243,8 @@ class Looper(feedcomponent.ParseLaunchComponent):
                 self.uiState.set('position', pos)
             return True
 
-        def on_start(result):
-            if not self.timeoutid:
-                self.timeoutid = gobject.timeout_add(500, check_time)
-            return result
-
-        d = feedcomponent.ParseLaunchComponent.do_start(self, *args, **kwargs)
-        d.addCallback(on_start)
-
-        return d
+        if not self.timeoutid:
+            self.timeoutid = gobject.timeout_add(500, check_time)
 
     def do_stop(self):
         if self.bus:
@@ -231,5 +256,3 @@ class Looper(feedcomponent.ParseLaunchComponent):
             self.timeoutid = 0
 
         self.nbiterations = 0
-
-        return feedcomponent.ParseLaunchComponent.do_stop(self)
