@@ -25,9 +25,12 @@ import string
 
 from twisted.internet import defer
 
+__version__ = "$Rev: 7162 $"
+
+
 class VideoTestAdminText(BaseAdminText):
-    commands = [ 'setpattern', 'getpattern' ]
-    patterns = [ 'smpte', 'snow', 'black' ]
+    commands = ['setpattern', 'getpattern']
+    patterns = ['smpte', 'snow', 'black']
 
     def setup(self):
         pass
@@ -46,7 +49,7 @@ class VideoTestAdminText(BaseAdminText):
                 for p in self.patterns:
                     if p.startswith(string.lower(input_split[1])):
                         available_commands.append(p)
-        
+
         return available_commands
 
     def runCommand(self, command):
@@ -62,21 +65,18 @@ class VideoTestAdminText(BaseAdminText):
                 elif string.lower(command_split[1]) == 'black':
                     pattern = 2
                 if pattern > -1:
-                    d = self.callRemote("setElementProperty", "source", "pattern", pattern)
+                    d = self.callRemote("setPattern", pattern)
                     return d
         elif string.lower(command_split[0]) == 'getpattern':
             # get pattern
-            d = self.callRemote("getElementProperty", "source", "pattern")
-            newd = defer.Deferred()
-            d.addCallback(self._getpattern_cb, newd) 
-            return newd
+
+            def getpattern_cb(uiState):
+                return self.patterns[uiState.get('pattern')]
+            d = self.callRemote("getUIState")
+            d.addCallback(getpattern_cb)
+            return d
         else:
             return None
 
-    def _getpattern_cb(self, result, newd):
-        pattern = self.patterns[result]
-        newd.callback(pattern)
-                
-            
+
 UIClass = VideoTestAdminText
-        
