@@ -36,7 +36,7 @@ different configdir, logdir and rundir.
 @type isinstalled: boolean
 
 @var  cachedir:      directory where cached code is stored
-@type cachedir:      string
+@type cachedir:      stringed
 @var  configdir:     directory where configuration files are stored
 @type configdir:     string
 @var  daemondir:     directory where daemonized programs should run
@@ -64,25 +64,42 @@ different configdir, logdir and rundir.
 @type defaultTCPManagerPort:  int
 @var  defaultSSLManagerPort:  the default manager port for SSL communication
 @type defaultSSLManagerPort:  int
-@var  defaultStreamPortRange: the default range of external streaming ports
-@type defaultStreamPortRange: list of ints
+@var  defaultHTTPStreamPort:  the default external http streaming port
+@type defaultHTTPStreamPort:  int
 @var  defaultGstPortRange:    the default range of internal GStreamer ports
 @type defaultGstPortRange:    list of ints
 
+@var  PACKAGE:      Flumotion package
+@type PACKAGE:      string
 @var  version:      Flumotion version number
 @type version:      string
 @var  versionTuple: Flumotion version number
 @type versionTuple: 4-tuple of integers
+@var  branchName:   Flumotion branch name
+@type branchName:   string
+
+# default values for service-related stuff
+
+@var  processTermWait: how long to wait before timing out term signals
+@type processTermWait int
+@var  processKillWait: how long to wait before timing out kill signals
+@type processKillWait int
+@var  heartbeatInterval: component heartbeat interval, in seconds
+@type heartbeatInterval: int
+@var  pingTimeoutMultiplier: how long to wait before assuming a lost
+                             connection, specified as a multiple of the
+                             heartbeatInterval
+@type pingTimeoutMultiplier: float
 '''
 
 # Note: This module is loaded very early on, so
 #       don't add any extra flumotion imports unless you
 #       really know what you're doing
 
-# FIXME: document all the module variables
-
 import os
 
+__version__ = "$Rev: 7372 $"
+
 # where am I on the disk ?
 __thisdir = os.path.dirname(os.path.abspath(__file__))
 
@@ -93,27 +110,43 @@ else:
     from flumotion.configure import installed
     _config = installed.get()
 
-# default values for ports
-_config['defaultTCPManagerPort'] = 8642
-_config['defaultSSLManagerPort'] = 7531
-_config['defaultStreamPortRange'] = range(8800, 8844 + 1)
-_config['defaultGstPortRange'] = range(8600, 8639 + 1)
-
-# default values for service-related stuff
-# how long to wait before timing out term and kill signals
-_config['processTermWait'] = 5
-_config['processKillWait'] = 5
-
-# default value for component heartbeat interval, in seconds
-_config['heartbeatInterval'] = 5
 
 def _versionStringToTuple(versionString):
     t = tuple(map(int, versionString.split('.')))
-    if len (t) < 4:
-        t = t + (0,)
+    if len(t) < 4:
+        t = t + (0, )
     return t
-_config['versionTuple'] = _versionStringToTuple(_config['version'])
 
-for key, value in _config.items():
-    dictionary = locals()
-    dictionary[key] = value
+isinstalled = _config['isinstalled']
+
+cachedir = _config['cachedir']
+configdir = _config['configdir']
+daemondir = _config['daemondir']
+datadir = _config['datadir']
+gladedir = _config['gladedir']
+imagedir = _config['imagedir']
+logdir = _config['logdir']
+localedatadir = _config['localedatadir']
+pythondir = _config['pythondir']
+registrydir = _config['registrydir']
+rundir = _config['rundir']
+bindir = _config['bindir']
+sbindir = _config['sbindir']
+
+defaultTCPManagerPort = 8642
+defaultSSLManagerPort = 7531
+defaultHTTPStreamPort = 8800
+defaultGstPortRange = range(8600, 8639 + 1)
+
+PACKAGE = 'flumotion'
+version = _config['version']
+versionTuple = _versionStringToTuple(version)
+branchName = 'trunk'
+
+processTermWait = 20
+processKillWait = 10
+heartbeatInterval = 5
+# FIXME: bring pingTimeoutMultiplier back to 2.5 once the ping
+# problems are fixed properly (possibly that value won't be needed at
+# that point at all)
+pingTimeoutMultiplier = 6.0
