@@ -25,24 +25,31 @@ manager-side objects to handle administrative clients
 
 import re
 import os
+import errno
 from StringIO import StringIO
 
-from twisted.internet import reactor, defer
-from twisted.spread import pb
+from twisted.internet import reactor
 from twisted.python import failure
+from twisted.spread import pb
+from zope.interface import implements
 
 from flumotion.manager import base
-from flumotion.common import errors, interfaces, log, planet, registry
+from flumotion.common import errors, interfaces, log, planet, registry, debug
+from flumotion.common.python import makedirs
 
 # make Result and Message proxyable
 from flumotion.common import messages
 
 # make ComponentState proxyable
 from flumotion.twisted import flavors
-from flumotion.twisted.compat import implements
 from flumotion.common import componentui
 
+__version__ = "$Rev: 7981 $"
+
+
 # FIXME: rename to Avatar since we are in the admin. namespace ?
+
+
 class AdminAvatar(base.ManagerAvatar):
     """
     I am an avatar created for an administrative client interface.
@@ -51,97 +58,23 @@ class AdminAvatar(base.ManagerAvatar):
     I live in the manager.
     """
     logCategory = 'admin-avatar'
-       
-    # override base methods
-    def attached(self, mind):
-        self.info('admin client "%s" logged in' % self.avatarId)
-        base.ManagerAvatar.attached(self, mind)
 
-    def detached(self, mind):
-        self.info('admin client "%s" logged out' % self.avatarId)
-        base.ManagerAvatar.detached(self, mind)
-
-    # FIXME: instead of doing this, give a RemoteCache of the heaven state ?
-    def getComponentStates(self):
-        """
-        Return all component states logged in to the manager.
-        The list gets serialized to a list of
-        L{flumotion.common.planet.AdminComponentState}
-        
-        @rtype: list of L{planet.ManagerComponentState}
-        """
-        return self.vishnu.getComponentStates()
-
-    def sendLog(self, category, type, message):
-        """
-        Send the given log message to the peer.
-        """
-        # don't send if we don't have a remote reference yet.
-        # this avoids recursion from the remote caller trying to warn
-        if self.hasRemoteReference():
-            self.mindCallRemote('log', category, type, message)
-        
     # override pb.Avatar implementation so we can run admin actions
+
     def perspectiveMessageReceived(self, broker, message, args, kwargs):
-        args = broker.unserialize(args, self)
-        kwargs = broker.unserialize(kwargs, self)
-        method = getattr(self, "perspective_%s" % message)
+        benignMethods = ('ping', )
 
-        level = log.DEBUG
-        if message == 'ping': level = log.LOG
-        debugClass = self.logCategory.upper()
-        startArgs = [self.remoteLogName, debugClass, message]
-        format, debugArgs = log.getFormatArgs(
-            '%s --> %s: perspective_%s(', startArgs,
-            ')', (), args, kwargs)
+        args = broker.unserialize(args)
+        kwargs = broker.unserialize(kwargs)
 
-        # log going into the method
-        logKwArgs = self.doLog(level, method, format, *debugArgs)
- 
-        benignMethods = ('ping',)
         if message not in benignMethods:
             self.vishnu.adminAction(self.remoteIdentity, message, args, kwargs)
 
-        try:
-            state = method(*args, **kwargs)
-        except TypeError:
-            self.debug("%s didn't accept %s and %s" % (method, args, kwargs))
-            raise
-        except pb.Error, e:
-            format, debugArgs = log.getFormatArgs(
-                '%s <-- %s: perspective_%s(', startArgs,
-                '): pb.Error %r', (e, ), args, kwargs)
-            self.doLog(level, -1, format, *debugArgs, **logKwArgs)
-            raise e
-
-        # log coming out of the method
-        if isinstance(state, defer.Deferred):
-            # for a deferred, we currently can't log a better location than
-            # the def line for the function/instance that we called above
-            def callback(result):
-                format, debugArgs = log.getFormatArgs(
-                    '%s <-- %s: perspective_%s(', startArgs,
-                    '): %r', (result, ), args, kwargs)
-                self.doLog(level, -1, format, *debugArgs, **logKwArgs)
-                return result
-            def errback(failure):
-                format, debugArgs = log.getFormatArgs(
-                    '%s <-- %s: perspective_%s(', startArgs,
-                    '): failure %r', (failure, ), args, kwargs)
-                self.doLog(level, -1, format, *debugArgs, **logKwArgs)
-                return failure
-
-            state.addCallback(callback)
-            state.addErrback(errback)
-        else:
-            format, debugArgs = log.getFormatArgs(
-                '%s <-- %s: perspective_%s(', startArgs,
-                '): %r', (state, ), args, kwargs)
-            self.doLog(level, -1, format, *debugArgs, **logKwArgs)
-
-        return broker.serialize(state, self, method, args, kwargs)
+        return base.ManagerAvatar.perspectiveMessageReceivedUnserialised(
+            self, broker, message, args, kwargs)
 
     ### pb.Avatar IPerspective methods
+
     def perspective_getPlanetState(self):
         """
         Get the planet state.
@@ -160,13 +93,6 @@ class AdminAvatar(base.ManagerAvatar):
         self.debug("returning worker heaven state %r" % self.vishnu.state)
         return self.vishnu.workerHeaven.state
 
-    def perspective_shutdown(self):
-        """
-        Shut down the manager.
-        """
-        self.warning("Shutdown requested, shutting down...")
-        reactor.stop()
-
     def perspective_componentStart(self, componentState):
         """
         Start the given component.  The component should be sleeping before
@@ -176,7 +102,7 @@ class AdminAvatar(base.ManagerAvatar):
         """
         self.debug('perspective_componentStart(%r)' % componentState)
         return self.vishnu.componentCreate(componentState)
-        
+
     def perspective_componentStop(self, componentState):
         """
         Stop the given component.
@@ -187,7 +113,7 @@ class AdminAvatar(base.ManagerAvatar):
         """
         self.debug('perspective_componentStop(%r)' % componentState)
         return self.vishnu.componentStop(componentState)
-        
+
     def perspective_componentRestart(self, componentState):
         """
         Restart the given component.
@@ -196,15 +122,17 @@ class AdminAvatar(base.ManagerAvatar):
         """
         self.debug('perspective_componentRestart(%r)' % componentState)
         d = self.perspective_componentStop(componentState)
-        d.addCallback(lambda *x: self.perspective_componentStart(componentState))
+        d.addCallback(lambda *x: self.perspective_componentStart(
+            componentState))
         return d
-        
+
     # Generic interface to call into a component
+
     def perspective_componentCallRemote(self, componentState, methodName,
                                         *args, **kwargs):
         """
         Call a method on the given component on behalf of an admin client.
-        
+
         @param componentState: state of the component to call the method on
         @type  componentState: L{planet.ManagerComponentState}
         @param methodName:     name of the method to call.  Gets proxied to
@@ -214,19 +142,24 @@ class AdminAvatar(base.ManagerAvatar):
 
         @rtype: L{twisted.internet.defer.Deferred}
         """
-        assert isinstance(componentState, planet.ManagerComponentState)
+        assert isinstance(componentState, planet.ManagerComponentState), \
+            "%r is not a componentState" % componentState
 
         if methodName == "start":
             self.warning('forwarding "start" to perspective_componentStart')
             return self.perspective_componentStart(componentState)
 
         m = self.vishnu.getComponentMapper(componentState)
+        if not m:
+            self.warning('Component not mapped. Maybe deleted.')
+            raise errors.UnknownComponentError(componentState)
+
         avatar = m.avatar
 
         if not avatar:
             self.warning('No avatar for %s, cannot call remote' %
                 componentState.get('name'))
-            raise errors.SleepingComponentError()
+            raise errors.SleepingComponentError(componentState)
 
         # XXX: Maybe we need to have a prefix, so we can limit what an
         # admin interface can call on a component
@@ -236,7 +169,7 @@ class AdminAvatar(base.ManagerAvatar):
             msg = "exception on remote call %s: %s" % (methodName,
                 log.getExceptionMessage(e))
             self.warning(msg)
-            raise errors.RemoteMethodError(methodName, 
+            raise errors.RemoteMethodError(methodName,
                 log.getExceptionMessage(e))
 
     def perspective_workerCallRemote(self, workerName, methodName,
@@ -249,15 +182,15 @@ class AdminAvatar(base.ManagerAvatar):
         @param workerName: the worker to call
         @type  workerName: str
         @param methodName: Name of the method to call.  Gets proxied to
-                           L{flumotion.worker.worker.WorkerMedium} 's
+                           L{flumotion.worker.medium.WorkerMedium} 's
                            remote_(methodName)
         @type  methodName: str
         """
-        
+
         self.debug('AdminAvatar.workerCallRemote(%r, %r)' % (
             workerName, methodName))
         workerAvatar = self.vishnu.workerHeaven.getAvatar(workerName)
-        
+
         # XXX: Maybe we need to a prefix, so we can limit what an admin
         # interface can call on a worker
         try:
@@ -267,73 +200,65 @@ class AdminAvatar(base.ManagerAvatar):
                 log.getExceptionMessage(e))
             return failure.Failure(errors.RemoteMethodError(methodName,
                 log.getExceptionMessage(e)))
-        
-    def perspective_getEntryByType(self, componentState, type):
+
+    def perspective_getEntryByType(self, componentType, entryType):
         """
-        Get the entry point for a piece of bundled code by the type.
-
-        Returns: a (filename, methodName) tuple, or raises a Failure
+        Get the entry point for a piece of bundled code in a component by type.
+        @param componentType: the component
+        @type componentType: a string
+        @param entryType: location of the entry point
+        @type entryType: a string
+        Returns: a (filename, methodName) tuple, or raises::
+          - NoBundleError if the entry location does not exist
         """
-        m = self.vishnu.getComponentMapper(componentState)
-        componentName = componentState.get('name')
+        assert componentType is not None
 
-        if not m.avatar:
-            self.debug('component %s not logged in yet, no entry' %
-                componentName)
-            raise errors.SleepingComponentError(componentName)
+        self.debug('getting entry of type %s for component type %s',
+                   entryType, componentType)
 
-        componentType = m.avatar.getType()
-        self.debug('getting entry of type %s for component %s of type %s' % (
-            type, componentName, componentType))
         try:
             componentRegistryEntry = registry.getRegistry().getComponent(
                 componentType)
             # FIXME: add logic here for default entry points and functions
-            entry = componentRegistryEntry.getEntryByType(type)
+            entry = componentRegistryEntry.getEntryByType(entryType)
         except KeyError:
             self.warning("Could not find bundle for %s(%s)" % (
-                componentType, type))
+                componentType, entryType))
             raise errors.NoBundleError("entry type %s in component type %s" %
-                (type, componentType))
+                (entryType, componentType))
 
         filename = os.path.join(componentRegistryEntry.base, entry.location)
         self.debug('entry point is in file path %s and function %s' % (
             filename, entry.function))
         return (filename, entry.function)
 
-    def perspective_reloadComponent(self, componentState):
+    def perspective_getPlugEntry(self, plugType, entryType):
         """
-        Reload modules in the given component.
-
-        @param componentState: state of the component to reload
-        @type  componentState: L{planet.ManagerComponentState}
+        Get the entry point for a piece of bundled code in a plug by type.
+        @param plugType: the plug
+        @type plugType: a string
+        @param entryType: location of the entry point
+        @type entryType: a string
+        Returns: a (filename, methodName) tuple, or raises::
+          - NoBundleError if the entry location does not exist
         """
-        def _reloaded(result, self, name):
-            self.info("reloaded component %s code" % name)
+        assert plugType is not None
 
-        name = componentState.get('name')
-        self.info("reloading component %s code" % name)
-        m = self.vishnu.getComponentMapper(componentState)
-        avatar = m.avatar
-        d = avatar.reloadComponent()
-        d.addCallback(_reloaded, self, name)
-        return d
+        self.debug('getting entry of type %s for plug type %s',
+                   entryType, plugType)
 
-    def perspective_reloadManager(self):
-        """
-        Reload modules in the manager.
-        """
-        import sys
-        from twisted.python.rebuild import rebuild
-        self.info('reloading manager code')
-        # reload ourselves first
-        rebuild(sys.modules[__name__])
+        try:
+            plugRegistryEntry = registry.getRegistry().getPlug(plugType)
+            entry = plugRegistryEntry.getEntryByType(entryType)
+        except KeyError:
+            self.warning("Could not find bundle for %s(%s)" % (
+                plugType, entryType))
+            raise errors.NoBundleError("entry type %s in plug type %s" %
+                                       (entryType, plugType))
 
-        # now rebuild relevant modules
-        import flumotion.common.reload
-        rebuild(sys.modules['flumotion.common'])
-        flumotion.common.reload.reload()
-        self._reloaded()
+        self.debug('entry point is in file path %s and function %s' % (
+           entry.location, entry.function))
+        return (entry.location, entry.function)
 
     def perspective_getConfiguration(self):
         """
@@ -343,6 +268,47 @@ class AdminAvatar(base.ManagerAvatar):
         """
         return self.vishnu.getConfiguration()
 
+    def perspective_getScenarioByType(self, scenarioType, entryType):
+        """
+        Remote method that gets the scenario of a given type.
+
+        @param scenarioType: the component
+        @type scenarioType: a string
+        Returns: a (filename, methodName) tuple, or raises::
+          - NoBundleError if the entry location does not exist
+        """
+        assert scenarioType is not None
+
+        self.debug('getting entry of type %s for scenario type %s',
+                   entryType, scenarioType)
+
+        try:
+            scenarioRegistryEntry = registry.getRegistry().getScenarioByType(
+                scenarioType)
+            # FIXME: add logic here for default entry points and functions
+            entry = scenarioRegistryEntry.getEntryByType(entryType)
+        except KeyError:
+            self.warning("Could not find bundle for %s(%s)" % (
+                scenarioType, entryType))
+            raise errors.NoBundleError("entry type %s in component type %s" %
+                (entryType, scenarioType))
+
+        filename = os.path.join(scenarioRegistryEntry.getBase(),
+                                entry.getLocation())
+        self.debug('entry point is in file path %s and function %s' % (
+            filename, entry.function))
+
+        return (filename, entry.getFunction())
+
+    def perspective_getScenarios(self):
+        """
+        Get all the scenarios defined on the registry.
+
+        @rtype     : List of L{IScenarioAssistantPlugin}
+        """
+        r = registry.getRegistry()
+        return r.getScenarios()
+
     def _saveFlowFile(self, filename):
         """Opens a file that the flow should be written to.
 
@@ -352,22 +318,23 @@ class AdminAvatar(base.ManagerAvatar):
         """
         self.vishnu.adminAction(self.remoteIdentity,
                                 '_saveFlowFile', (), {})
+
         def ensure_sane(name, extra=''):
             if not re.match('^[a-zA-Z0-9_' + extra + '-]+$', name):
                 raise errors.ConfigError, \
                       'Invalid planet or saveAs name: %s' % name
-        
+
         ensure_sane(self.vishnu.configDir, '/')
         ensure_sane(filename)
-        dir = os.path.join(self.vishnu.configDir, "flows")
-        self.debug('told to save flow as %s/%s.xml', dir, filename)
-        try: 
-            os.makedirs(dir, 0770) 
-        except OSError, e: 
-            if e.errno != 17: # 17 == EEXIST
+        directory = os.path.join(self.vishnu.configDir, "flows")
+        self.debug('told to save flow as %s/%s.xml', directory, filename)
+        try:
+            makedirs(directory, 0770)
+        except OSError, e:
+            if e.errno != errno.EEXIST:
                 raise e
         prev = os.umask(0007)
-        output = open(os.path.join(dir, filename + '.xml'), 'a')
+        output = open(os.path.join(directory, filename + '.xml'), 'a')
         os.umask(prev)
         return output
 
@@ -385,12 +352,17 @@ class AdminAvatar(base.ManagerAvatar):
 
         if saveAs:
             output = self._saveFlowFile(saveAs)
- 
+
+        # Update the registry if needed, so that new/changed component types
+        # can be parsed.
+        registry.getRegistry().verify()
+
         f = StringIO(xml)
-        res = self.vishnu.loadConfigurationXML(f, self.remoteIdentity)
+        res = self.vishnu.loadComponentConfigurationXML(f, self.remoteIdentity)
         f.close()
 
         if saveAs:
+
             def success(res):
                 self.debug('loadConfiguration succeeded, writing flow to %r',
                            output)
@@ -398,6 +370,7 @@ class AdminAvatar(base.ManagerAvatar):
                 output.write(xml)
                 output.close()
                 return res
+
             def failure(res):
                 self.debug('loadConfiguration failed, leaving %r as it was',
                            output)
@@ -407,6 +380,51 @@ class AdminAvatar(base.ManagerAvatar):
 
         return res
 
+    def perspective_loadComponent(self, componentType, componentId,
+                                  componentLabel, properties, workerName,
+                                  plugs=None, eaters=None,
+                                  isClockMaster=None, virtualFeeds=None):
+        """
+        Load a component into the manager configuration.
+        Returns a deferred that will be called with the component state.
+
+        @param componentType:  The registered type of the component to be added
+        @type  componentType:  str
+        @param componentId:    The identifier of the component to add,
+                               should be created by the function
+                               L{flumotion.common.common.componentId}
+        @type  componentId:    str
+        @param componentLabel: The human-readable label of the component.
+                               if None, no label will be set.
+        @type  componentLabel: str or None
+        @param properties:     List of property name-value pairs.
+                               See L{flumotion.common.config.buildPropertyDict}
+        @type  properties:     list of (str, object)
+        @param workerName:     the name of the worker where the added
+                               component should run.
+        @type  workerName:     str
+        @param plugs:          List of plugs, as type-propertyList pairs.
+                               See {flumotion.manager.config.buildPlugsSet}.
+        @type  plugs:          [(str, [(str, object)])]
+        @param eaters:         List of (eater name, feed ID) pairs.
+                               See L{flumotion.manager.config.buildEatersDict}
+        @type  eaters:         [(str, str)]
+        @param isClockMaster:  True if the component to be added must be
+                               a clock master. Passing False here means
+                               that the manager will choose what
+                               component, if any, will be clock master
+                               for this flow.
+        @type  isClockMaster:  bool
+        @param virtualFeeds:   List of (virtual feed, feeder name) pairs.
+            See L{flumotion.manager.config.buildVirtualFeeds}
+        @type  virtualFeeds:   [(str, str)]
+        """
+        return self.vishnu.loadComponent(self.remoteIdentity, componentType,
+                                         componentId, componentLabel,
+                                         properties, workerName,
+                                         plugs or [], eaters or [],
+                                         isClockMaster, virtualFeeds or [])
+
     def perspective_deleteFlow(self, flowName):
         return self.vishnu.deleteFlow(flowName)
 
@@ -422,13 +440,70 @@ class AdminAvatar(base.ManagerAvatar):
         """
         return self.vishnu.deleteComponent(componentState)
 
-    # Deprecated -- remove me when no one uses me any more
+    def perspective_getVersions(self):
+        return debug.getVersions()
+
     def perspective_cleanComponents(self):
         return self.vishnu.emptyPlanet()
 
-    # separate method so it runs the newly reloaded one :)
-    def _reloaded(self):
-        self.info('reloaded manager code')
+    def perspective_getWizardEntries(self, types=None, provides=None,
+                                     accepts=None):
+        """
+        Fetches the wizard entries which matches the parameters sent in
+
+        @param types: list of component types to fetch, is usually
+          something like ['video-producer'] or ['audio-encoder']
+        @type  types: list of strings
+        @param provides: formats provided, eg ['jpeg', 'speex']
+        @type  provides: list of strings
+        @param accepts: formats accepted, eg ['theora']
+        @type  accepts: list of strings
+        @returns: L{componentui.WizardEntryState}
+        """
+
+        def extract(wizards):
+            for wizard in wizards:
+                if types is not None:
+                    if wizard.type not in types:
+                        continue
+                if provides is not None:
+                    for format in wizard.provides:
+                        if format.media_type in provides:
+                            break
+                    else:
+                        continue
+                if accepts is not None:
+                    for format in wizard.accepts:
+                        if format.media_type in accepts:
+                            break
+                    else:
+                        continue
+                yield wizard
+
+        retval = []
+        r = registry.getRegistry()
+        for component in r.getComponents():
+            retval += extract(component.wizards)
+        for plug in r.getPlugs():
+            retval += extract(plug.wizards)
+        del r
+
+        return retval
+
+    def perspective_getComponentEntry(self, componentType):
+        """Fetches a ComponentRegistryEntry given a componentType
+        @param componentType: component type
+        @type componentType: string
+        @returns: the component
+        @rtype: L{ComponentRegistryEntry}
+        """
+        try:
+            componentRegistryEntry = registry.getRegistry().getComponent(
+                componentType)
+        except KeyError:
+            return None
+        return componentRegistryEntry
+
 
 class AdminHeaven(base.ManagerHeaven):
     """
@@ -440,40 +515,3 @@ class AdminHeaven(base.ManagerHeaven):
     logCategory = "admin-heaven"
     implements(interfaces.IHeaven)
     avatarClass = AdminAvatar
-
-    def __init__(self, vishnu):
-        # doc in base class
-        base.ManagerHeaven.__init__(self, vishnu)
-        #FIXME: don't add a log handler here until we have a good way
-        #of filtering client-side again
-        #log.addLogHandler(self.logHandler)
-        self._logcache = []
-
-    #def logHandler(self, category, type, message):
-    #    self.logcache.append((category, type, message))
-    #    for avatar in self.getAvatars():
-    #        avatar.sendLog(category, type, message)
-
-    #def sendCache(self, avatar):
-    #    if not avatar.hasRemoteReference():
-    #        reactor.callLater(0.25, self.sendCache, avatar)
-    #        return
-        
-        # FIXME: do this on request only
-        #self.debug('sending logcache to client (%d messages)' % len(self.logcache))
-        #for category, type, message in self.logcache:
-        #    avatar.sendLog(category, type, message)
-        
-    ### my methods
-
-    def avatarsCallRemote(self, methodName, *args, **kwargs):
-        """
-        Call a remote method on all AdminAvatars in this heaven.
-
-        @param methodName: Name of the method to call.  Gets proxied to
-                           L{flumotion.admin.admin.AdminModel}'s
-                           remote_(methodName)
-        @type  methodName: str
-        """
-        for avatar in self.getAvatars():
-            avatar.mindCallRemote(methodName, *args, **kwargs)
