@@ -25,11 +25,13 @@ manager-side objects to handle worker clients
 
 from twisted.internet import defer
 
-# FIXME: rename to base
 from flumotion.manager import base
 from flumotion.common import errors, interfaces, log, registry
-from flumotion.common import config, worker, common
-from flumotion.twisted.defer import defer_generator_method
+from flumotion.common import worker, common
+from flumotion.common.vfs import registerVFSJelly
+
+__version__ = "$Rev: 7162 $"
+
 
 class WorkerAvatar(base.ManagerAvatar):
     """
@@ -45,40 +47,43 @@ class WorkerAvatar(base.ManagerAvatar):
     _portSet = None
     feedServerPort = None
 
-    def getName(self):
-        return self.avatarId
+    def __init__(self, heaven, avatarId, remoteIdentity, mind,
+                 feedServerPort, ports, randomPorts):
+        base.ManagerAvatar.__init__(self, heaven, avatarId,
+                                    remoteIdentity, mind)
+        self.feedServerPort = feedServerPort
 
-    def attached(self, mind):
-        # doc in base class
-        self.info('worker "%s" logged in' % self.getName())
-        base.ManagerAvatar.attached(self, mind)
-
-        self.debug('MANAGER -> WORKER: getFeedServerPort()')
-        d = self.mindCallRemote('getFeedServerPort')
-        yield d
-        # this can be None if no feed server is configured
-        self.feedServerPort = d.value()
-        self.debug('WORKER -> MANAGER: getFeedServerPort(): %r' %
-            self.feedServerPort)
-
-        self.debug('MANAGER -> WORKER: getPorts()')
-        d = self.mindCallRemote('getPorts')
-        yield d
-        ports = d.value()
-        self.debug('WORKER -> MANAGER: getPorts(): %r' % ports)
-        self._portSet = worker.PortSet(self.avatarId, ports)
+        self._portSet = worker.PortSet(self.avatarId, ports, randomPorts)
 
         self.heaven.workerAttached(self)
         self.vishnu.workerAttached(self)
-    attached = defer_generator_method(attached)
 
-    def detached(self, mind):
-        # doc in base class
-        self.info('worker "%s" logged out' % self.getName())
-        base.ManagerAvatar.detached(self, mind)
+        registerVFSJelly()
+
+    def getName(self):
+        return self.avatarId
+
+    def makeAvatarInitArgs(klass, heaven, avatarId, remoteIdentity,
+                           mind):
+
+        def havePorts(res):
+            log.debug('worker-avatar', 'got port information')
+            (_s1, feedServerPort), (_s2, (ports, random)) = res
+            return (heaven, avatarId, remoteIdentity, mind,
+                    feedServerPort, ports, random)
+        log.debug('worker-avatar', 'calling mind for port information')
+        d = defer.DeferredList([mind.callRemote('getFeedServerPort'),
+                                mind.callRemote('getPorts')],
+                               fireOnOneErrback=True)
+        d.addCallback(havePorts)
+        return d
+    makeAvatarInitArgs = classmethod(makeAvatarInitArgs)
+
+    def onShutdown(self):
         self.heaven.workerDetached(self)
         self.vishnu.workerDetached(self)
-    
+        base.ManagerAvatar.onShutdown(self)
+
     def reservePorts(self, numPorts):
         """
         Reserve the given number of ports on the worker.
@@ -91,13 +96,13 @@ class WorkerAvatar(base.ManagerAvatar):
     def releasePorts(self, ports):
         """
         Release the given list of ports on the worker.
-    
+
         @param ports: list of ports to release
         @type  ports: list of int
         """
         self._portSet.releasePorts(ports)
 
-    def createComponent(self, avatarId, type, nice):
+    def createComponent(self, avatarId, type, nice, conf):
         """
         Create a component of the given type with the given nice level.
 
@@ -107,12 +112,14 @@ class WorkerAvatar(base.ManagerAvatar):
         @type  type:     str
         @param nice:     the nice level to create the component at
         @type  nice:     int
+        @param conf:     the component's config dict
+        @type  conf:     dict
 
         @returns: a deferred that will give the avatarId the component
                   will use to log in to the manager
         """
-        self.debug('creating %s (%s) on worker %s with nice level %d' % (
-            avatarId, type, self.avatarId, nice))
+        self.debug('creating %s (%s) on worker %s with nice level %d',
+                   avatarId, type, self.avatarId, nice)
         defs = registry.getRegistry().getComponent(type)
         try:
             entry = defs.getEntryByType('component')
@@ -127,7 +134,7 @@ class WorkerAvatar(base.ManagerAvatar):
 
         self.debug('call remote create')
         return self.mindCallRemote('create', avatarId, type, moduleName,
-            methodName, nice)
+            methodName, nice, conf)
 
     def getComponents(self):
         """
@@ -136,16 +143,17 @@ class WorkerAvatar(base.ManagerAvatar):
         @returns: a deferred that will give the avatarIds running on the
                   worker
         """
-        self.debug('getting component list from worker %s' % 
+        self.debug('getting component list from worker %s' %
             self.avatarId)
         return self.mindCallRemote('getComponents')
 
     ### IPerspective methods, called by the worker's component
+
     def perspective_componentAddMessage(self, avatarId, message):
         """
         Called by the worker to tell the manager to add a given message to
         the given component.
-        
+
         Useful in cases where the component can't report messages itself,
         for example because it crashed.
 
@@ -155,22 +163,23 @@ class WorkerAvatar(base.ManagerAvatar):
         self.debug('received message from component %s' % avatarId)
         self.vishnu.componentAddMessage(avatarId, message)
 
+
 class WorkerHeaven(base.ManagerHeaven):
     """
     I interface between the Manager and worker clients.
     For each worker client I create an L{WorkerAvatar} to handle requests.
     I live in the manager.
     """
-    
+
     logCategory = "workerheaven"
     avatarClass = WorkerAvatar
-    
+
     def __init__(self, vishnu):
         base.ManagerHeaven.__init__(self, vishnu)
-        self.conf = None
         self.state = worker.ManagerWorkerHeavenState()
-        
+
     ### my methods
+
     def workerAttached(self, workerAvatar):
         """
         Notify the heaven that the given worker has logged in.
@@ -185,9 +194,9 @@ class WorkerHeaven(base.ManagerHeaven):
             self.state.append('names', workerName)
             self.state.append('workers', state)
         else:
-            # FIXME: what if it was already there ?
-            self.warning('worker %s was already registered in the heaven' %
-                workerName)
+            self.warning('worker %s was already registered in the heaven',
+                         workerName)
+            raise errors.AlreadyConnectedError()
 
     def workerDetached(self, workerAvatar):
         """
@@ -202,5 +211,5 @@ class WorkerHeaven(base.ManagerHeaven):
                 if state.get('name') == workerName:
                     self.state.remove('workers', state)
         except ValueError:
-            self.warning('worker %s was never registered in the heaven' %
-                workerName)
+            self.warning('worker %s was never registered in the heaven',
+                         workerName)
