@@ -22,15 +22,20 @@
 import os
 import sys
 
-from flumotion.common import package, errors
+from flumotion.common import package, errors, log
+from flumotion.configure import configure
 
 import flumotion.project
 
+__version__ = "$Rev: 7986 $"
+
+
 def list():
     """
     Returns a list of all add-on projects seen by Flumotion.
     """
-    projects = [n for n in sys.modules.keys() if n.startswith('flumotion.project')]
+    projects = [n for n in sys.modules.keys()
+                      if n.startswith('flumotion.project')]
     paths = flumotion.project.__path__
     modules = []
     for path in paths:
@@ -42,16 +47,30 @@ def list():
 
     return [n[len('flumotion.project.'):] for n in modules]
 
+
 def get(project, attribute, default=None):
     """
     Get an attribute from a project's module.
     """
+    log.debug('project', 'Getting attribute %s from project %s',
+        attribute, project)
+
+    # short-circuit flumotion, the core project
+    if project == 'flumotion':
+        return getattr(configure, attribute, default)
+
     moduleName = "flumotion.project.%s" % project
     try:
         exec("import %s" % moduleName)
-    except ImportError:
-        raise errors.NoProjectError(moduleName)
-    except SyntaxError:
-        raise errors.NoProjectError(moduleName)
+    except ImportError, e:
+        msg = ('Could not load project %s: %s' %
+               (project, log.getExceptionMessage(e)))
+        log.warning('project', msg)
+        raise errors.NoProjectError(project, msg)
+    except SyntaxError, e:
+        msg = ('Syntax error while loading project %s: %s' %
+               (project, log.getExceptionMessage(e)))
+        log.warning('project', msg)
+        raise errors.NoProjectError(project, msg)
     m = sys.modules[moduleName]
     return getattr(m, attribute, default)
