@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007,2008 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This program is free software; you can redistribute it and/or modify
@@ -14,21 +14,28 @@
 # This program is also licensed under the Flumotion license.
 # See "LICENSE.Flumotion" in the source distribution for more information.
 
+"""
+Servicer object used in service scripts
+"""
+
 import os
 import glob
 import time
 
 from flumotion.configure import configure
-from flumotion.common import common, errors, log
+from flumotion.common import errors, log
+from flumotion.common.python import makedirs
+from flumotion.common.process import checkPidRunning, deletePidFile, getPid, \
+     killPid, termPid, waitPidFile
+
+__version__ = "$Rev: 7917 $"
+
 
-"""
-Servicer object used in service scripts
-"""
 class Servicer(log.Loggable):
     """
     I manage running managers and workers on behalf of a service script.
     """
-    
+
     logCategory = 'servicer'
 
     def __init__(self, configDir=None, logDir=None, runDir=None):
@@ -62,24 +69,24 @@ class Servicer(log.Loggable):
 
         which = args[0]
         if which not in ['manager', 'worker']:
-            raise errors.SystemError, 'Please specify either manager or worker'
-            
+            raise errors.FatalError, 'Please specify either manager or worker'
+
         if len(args) < 2:
-            raise errors.SystemError, 'Please specify which %s to %s' % (
+            raise errors.FatalError, 'Please specify which %s to %s' % (
                 which, command)
 
         name = args[1]
         if which == 'manager':
             managers = self.getManagers()
-            if not managers.has_key(name):
-                raise errors.SystemError, 'No manager "%s"' % name
+            if not name in managers:
+                raise errors.FatalError, 'No manager "%s"' % name
             managers = [name, ]
         elif which == 'worker':
             workers = self.getWorkers()
             if not name in workers:
-                raise errors.SystemError, 'No worker with name %s' % name
+                raise errors.FatalError, 'No worker with name %s' % name
             workers = [name, ]
-            
+
         return (managers, workers)
 
     def _getDirOptions(self):
@@ -140,10 +147,8 @@ class Servicer(log.Loggable):
         Start processes as given in the args.
 
         If nothing specified, start all managers and workers.
-        If first argument is "manager", start given manager,
-        or all if none specified.
-        If first argument is "worker", start given worker,
-        or all if none specified.
+        If first argument is "manager", start given manager.
+        If first argument is "worker", start given worker.
 
         @returns: an exit value reflecting the number of processes that failed
                   to start
@@ -167,10 +172,8 @@ class Servicer(log.Loggable):
         Stop processes as given in the args.
 
         If nothing specified, stop all managers and workers.
-        If first argument is "manager", stop given manager,
-        or all if none specified.
-        If first argument is "worker", stop given worker,
-        or all if none specified.
+        If first argument is "manager", stop given manager.
+        If first argument is "worker", stop given worker.
 
         @returns: an exit value reflecting the number of processes that failed
                   to stop
@@ -186,7 +189,7 @@ class Servicer(log.Loggable):
         for name in managers:
             if not self.stopManager(name):
                 exitvalue += 1
-            
+
         return exitvalue
 
     def status(self, args):
@@ -195,16 +198,16 @@ class Servicer(log.Loggable):
         """
         (managers, workers) = self._parseManagersWorkers('status', args)
         self.debug("Status managers %r and workers %r" % (managers, workers))
-        for type, list in [('manager', managers), ('worker', workers)]:
-            for name in list:
-                pid = common.getPid(type, name)
+        for kind, names in [('manager', managers), ('worker', workers)]:
+            for name in names:
+                pid = getPid(kind, name)
                 if not pid:
-                    print "%s %s not running" % (type, name)
+                    print "%s %s not running" % (kind, name)
                     continue
-                if common.checkPidRunning(pid):
-                    print "%s %s is running with pid %d" % (type, name, pid)
+                if checkPidRunning(pid):
+                    print "%s %s is running with pid %d" % (kind, name, pid)
                 else:
-                    print "%s %s dead (stale pid %d)" % (type, name, pid)
+                    print "%s %s dead (stale pid %d)" % (kind, name, pid)
 
     def clean(self, args):
         """
@@ -212,19 +215,63 @@ class Servicer(log.Loggable):
         """
         (managers, workers) = self._parseManagersWorkers('clean', args)
         self.debug("Clean managers %r and workers %r" % (managers, workers))
-        for type, list in [('manager', managers), ('worker', workers)]:
-            for name in list:
-                pid = common.getPid(type, name)
+        for kind, names in [('manager', managers), ('worker', workers)]:
+            for name in names:
+                pid = getPid(kind, name)
                 if not pid:
                     # may be a file that contains bogus data
-                    print "deleting bogus pid file for %s %s" % (type, name)
-                    common.deletePidFile(type, name)
+                    try:
+                        deletePidFile(kind, name)
+                        print "deleted bogus pid file for %s %s" % (kind, name)
+                    except OSError:
+                        print ("failed to delete pid file for %s %s "
+                               "- ignoring" % (kind, name))
                     continue
-                if not common.checkPidRunning(pid):
+                if not checkPidRunning(pid):
                     self.debug("Cleaning up stale pid %d for %s %s" % (
-                        pid, type, name))
-                    print "deleting stale pid file for %s %s" % (type, name)
-                    common.deletePidFile(type, name)
+                        pid, kind, name))
+                    print "deleting stale pid file for %s %s" % (kind, name)
+                    deletePidFile(kind, name)
+
+    def condrestart(self, args):
+        """
+        Restart running processes as given in the args.
+
+        If nothing specified, condrestart all managers and workers.
+        If first argument is "manager", condrestart given manager.
+        If first argument is "worker", condrestart given worker.
+
+        @returns: an exit value reflecting the number of processes that failed
+                  to start
+        """
+        (managers, workers) = self._parseManagersWorkers('condrestart', args)
+        self.debug("condrestart managers %r and workers %r" % (
+            managers, workers))
+        managersDict = self.getManagers()
+        exitvalue = 0
+
+        for kind, names in [('manager', managers), ('worker', workers)]:
+            for name in names:
+                pid = getPid(kind, name)
+                if not pid:
+                    continue
+                if checkPidRunning(pid):
+                    if kind == 'manager':
+                        if not self.stopManager(name):
+                            exitvalue += 1
+                            continue
+                        if not self.startManager(name, managersDict[name]):
+                            exitvalue += 1
+                    elif kind == 'worker':
+                        if not self.stopWorker(name):
+                            exitvalue += 1
+                            continue
+                        if not self.startWorker(name):
+                            exitvalue += 1
+                else:
+                    print "%s %s dead (stale pid %d)" % (kind, name, pid)
+
+        return exitvalue
 
     def create(self, args):
         # TODO: Andy suggested we should be able to customize the
@@ -235,24 +282,24 @@ class Servicer(log.Loggable):
         Create a default manager or worker config.
         """
         if len(args) == 0:
-            raise errors.SystemError, \
+            raise errors.FatalError, \
                 "Please specify 'manager' or 'worker' to create."
-        type = args[0]
+        kind = args[0]
         if len(args) == 1:
-            raise errors.SystemError, \
-                "Please specify name of %s to create." % type
+            raise errors.FatalError, \
+                "Please specify name of %s to create." % kind
         name = args[1]
 
         port = 7531
         if len(args) == 3:
             port = int(args[2])
 
-        if type == 'manager':
+        if kind == 'manager':
             self.createManager(name, port)
-        elif type == 'worker':
+        elif kind == 'worker':
             self.createWorker(name, managerPort=port, randomFeederports=True)
         else:
-            raise errors.SystemError, \
+            raise errors.FatalError, \
                 "Please specify 'manager' or 'worker' to create."
 
     def createManager(self, name, port=7531):
@@ -264,12 +311,26 @@ class Servicer(log.Loggable):
         self.info("Creating manager %s" % name)
         managerDir = os.path.join(self.managersDir, name)
         if os.path.exists(managerDir):
-            raise errors.SystemError, \
+            raise errors.FatalError, \
                 "Manager directory %s already exists" % managerDir
-        os.makedirs(managerDir)
+        makedirs(managerDir)
 
         planetFile = os.path.join(managerDir, 'planet.xml')
 
+        # create a default.pem file if it doesn't exist yet
+        pemFile = os.path.join(configure.configdir, 'default.pem')
+        if not os.path.exists(pemFile):
+            # files in datadir are usually not executable, so call through sh
+            retval = os.system("sh %s %s" % (
+                os.path.join(configure.datadir, 'make-dummy-cert'), pemFile))
+
+            # If we couldn't generate the file, it means that we probably
+            # don't have openssl installed. If so, don't include the complete
+            # to the pemfile which means that the the default pem file which
+            # is shipped with flumotion will be used instead.
+            if retval != 0:
+                pemFile = 'default.pem'
+
         # generate the file
         handle = open(planetFile, 'w')
         handle.write("""<planet>
@@ -280,9 +341,7 @@ class Servicer(log.Loggable):
     <transport>ssl</transport>
     <!-- certificate path can be relative to $sysconfdir/flumotion,
          or absolute -->
-<!--
-    <certificate>default.pem</certificate>
--->
+    <certificate>%(pemFile)s</certificate>
     <component name="manager-bouncer" type="htpasswdcrypt-bouncer">
       <property name="data"><![CDATA[
 user:PSfNpHTkpTx1M
@@ -293,12 +352,6 @@ user:PSfNpHTkpTx1M
 """ % locals())
         handle.close()
 
-        # create a default.pem file if it doesn't exist yet
-        pemFile = os.path.join(configure.configdir, 'default.pem')
-        if not os.path.exists(pemFile):
-            os.system("%s %s" % (
-                os.path.join(configure.datadir, 'make-dummy-cert'), pemFile))
-
         return True
 
     def createWorker(self, name, managerPort=7531, randomFeederports=False):
@@ -307,11 +360,11 @@ user:PSfNpHTkpTx1M
 
         @returns: whether or not the config was created.
         """
-        os.makedirs(self.workersDir)
+        makedirs(self.workersDir)
         self.info("Creating worker %s" % name)
         workerFile = os.path.join(self.workersDir, "%s.xml" % name)
         if os.path.exists(workerFile):
-            raise errors.SystemError, \
+            raise errors.FatalError, \
                 "Worker file %s already exists." % workerFile
 
         feederports = "  <!-- <feederports>8600-8639</feederports> -->"
@@ -341,7 +394,6 @@ user:PSfNpHTkpTx1M
 
         return True
 
- 
     def startManager(self, name, flowNames):
         """
         Start the manager as configured in the manager directory for the given
@@ -354,7 +406,7 @@ user:PSfNpHTkpTx1M
         managerDir = os.path.join(self.managersDir, name)
         planetFile = os.path.join(managerDir, 'planet.xml')
         if not os.path.exists(planetFile):
-            raise errors.SystemError, \
+            raise errors.FatalError, \
                 "Planet file %s does not exist" % planetFile
         self.info("Loading planet %s" % planetFile)
 
@@ -363,20 +415,20 @@ user:PSfNpHTkpTx1M
         for flowName in flowNames:
             flowFile = os.path.join(flowsDir, "%s.xml" % flowName)
             if not os.path.exists(flowFile):
-                raise errors.SystemError, \
+                raise errors.FatalError, \
                     "Flow file %s does not exist" % flowFile
             flowFiles.append(flowFile)
             self.info("Loading flow %s" % flowFile)
 
-        pid = common.getPid('manager', name)
+        pid = getPid('manager', name)
         if pid:
-            if common.checkPidRunning(pid):
-                raise errors.SystemError, \
+            if checkPidRunning(pid):
+                raise errors.FatalError, \
                     "Manager %s is already running (with pid %d)" % (name, pid)
             else:
-                raise errors.SystemError, \
+                raise errors.FatalError, \
                     "Manager %s is dead (stale pid %d)" % (name, pid)
-            
+
         dirOptions = self._getDirOptions()
         command = "flumotion-manager %s -D --daemonize-to %s " \
             "--service-name %s %s %s" % (
@@ -387,7 +439,7 @@ user:PSfNpHTkpTx1M
 
         if retval == 0:
             self.debug("Waiting for pid for manager %s" % name)
-            pid = common.waitPidFile('manager', name)
+            pid = waitPidFile('manager', name)
             if pid:
                 self.info("Started manager %s with pid %d" % (name, pid))
                 return True
@@ -409,18 +461,18 @@ user:PSfNpHTkpTx1M
         self.info("Starting worker %s" % name)
         workerFile = os.path.join(self.workersDir, "%s.xml" % name)
         if not os.path.exists(workerFile):
-            raise errors.SystemError, \
+            raise errors.FatalError, \
                 "Worker file %s does not exist" % workerFile
 
-        pid = common.getPid('worker', name)
+        pid = getPid('worker', name)
         if pid:
-            if common.checkPidRunning(pid):
-                raise errors.SystemError, \
+            if checkPidRunning(pid):
+                raise errors.FatalError, \
                     "Worker %s is already running (with pid %d)" % (name, pid)
             else:
-                raise errors.SystemError, \
+                raise errors.FatalError, \
                     "Worker %s is dead (stale pid %d)" % (name, pid)
-            
+
         # we are sure the worker is not running and there's no pid file
         self.info("Loading worker %s" % workerFile)
 
@@ -433,7 +485,7 @@ user:PSfNpHTkpTx1M
 
         if retval == 0:
             self.debug("Waiting for pid for worker %s" % name)
-            pid = common.waitPidFile('worker', name)
+            pid = waitPidFile('worker', name)
             if pid:
                 self.info("Started worker %s with pid %d" % (name, pid))
                 return True
@@ -463,12 +515,12 @@ user:PSfNpHTkpTx1M
         Stop the given manager if it is running.
         """
         self.info("Stopping manager %s" % name)
-        pid = common.getPid('manager', name)
+        pid = getPid('manager', name)
         if not pid:
             return True
 
         # FIXME: ensure a correct process is running this pid
-        if not common.checkPidRunning(pid):
+        if not checkPidRunning(pid):
             self.info("Manager %s is dead (stale pid %d)" % (name, pid))
             return False
 
@@ -484,13 +536,13 @@ user:PSfNpHTkpTx1M
         Stop the given worker if it is running.
         """
         self.info("Stopping worker %s" % name)
-        pid = common.getPid('worker', name)
+        pid = getPid('worker', name)
         if not pid:
             self.info("worker %s was not running" % name)
             return True
 
         # FIXME: ensure a correct process is running this pid
-        if not common.checkPidRunning(pid):
+        if not checkPidRunning(pid):
             self.info("Worker %s is dead (stale pid %d)" % (name, pid))
             return False
 
@@ -511,18 +563,19 @@ user:PSfNpHTkpTx1M
         killClock = termClock + configure.processKillWait
 
         self.debug('stopping process with pid %d' % pid)
-        if not common.termPid(pid):
+        if not termPid(pid):
             self.warning('No process with pid %d' % pid)
             return False
 
         # wait for the kill
-        while (common.checkPidRunning(pid)):
+        while (checkPidRunning(pid)):
             if time.clock() > termClock:
                 self.warning("Process with pid %d has not responded to TERM " \
                     "for %d seconds, killing" % (pid,
                         configure.processTermWait))
-                common.killPid(pid)
-                termClock = killClock + 1.0 # so it does not get triggered again
+                killPid(pid)
+                # so it does not get triggered again
+                termClock = killClock + 1.0
 
             if time.clock() > killClock:
                 self.warning("Process with pid %d has not responded to KILL " \
@@ -533,7 +586,7 @@ user:PSfNpHTkpTx1M
             # busy loop until kill is done
 
         return True
-  
+
     def list(self):
         """
         List all service parts managed.
@@ -544,9 +597,8 @@ user:PSfNpHTkpTx1M
             print "manager %s" % name
             if flows:
                 for flow in flows:
-                   print "        flow %s" % flow
+                    print "        flow %s" % flow
 
         workers = self.getWorkers()
         for worker in workers:
             print "worker  %s" % worker
-
