@@ -19,41 +19,38 @@
 
 # Headers in this file shall remain intact.
 
-import common
+from flumotion.common import testsuite
 from twisted.trial import unittest
 
 import crypt
 
-from twisted.cred import error
-
 from twisted.cred import credentials as tcredentials
 from flumotion.twisted import credentials, checkers
+from flumotion.common import errors
+
+
 
-import twisted.copyright #T1.3
 # Use some shorter names
 CredPlaintext = credentials.UsernameCryptPasswordPlaintext
 CredCrypt = credentials.UsernameCryptPasswordCrypt
 
-#T1.3
-def weHaveAnOldTwisted():
-    return twisted.copyright.version < '2.0.0'
 
-class TestFlexibleWithPassword(unittest.TestCase):
+class TestFlexibleWithPassword(testsuite.TestCase):
+
     def setUp(self):
         self.checker = checkers.FlexibleCredentialsChecker(user='test')
 
     def testCredPlaintextCorrect(self):
+
         def credPlaintextCorrectCallback(result):
             self.assertEquals(result, 'user')
         cred = tcredentials.UsernamePassword('user', 'test')
         d = self.checker.requestAvatarId(cred)
         d.addCallback(credPlaintextCorrectCallback)
-        if weHaveAnOldTwisted(): #T1.3
-            unittest.deferredResult(d)
-        else:
-            return d
+        return d
 
     def testCredPlaintextCorrectWithId(self):
+
         def credPlaintextCorrectWithIdCallback(result):
             self.assertEquals(result, 'requested')
             return True
@@ -61,41 +58,38 @@ class TestFlexibleWithPassword(unittest.TestCase):
         cred.avatarId = "requested"
         d = self.checker.requestAvatarId(cred)
         d.addCallback(credPlaintextCorrectWithIdCallback)
-        if weHaveAnOldTwisted(): #T1.3
-            unittest.deferredResult(d)
-        else:
-            return d
+        return d
 
     def testCredPlaintextWrong(self):
+
         def credPlaintextWrongErrback(failure):
-            self.assert_(isinstance(failure.type(), error.UnauthorizedLogin))
+            self.assert_(isinstance(failure.type(),
+                                    errors.NotAuthenticatedError))
             return True
         cred = tcredentials.UsernamePassword('user', 'wrong')
         d = self.checker.requestAvatarId(cred)
         d.addErrback(credPlaintextWrongErrback)
-        if weHaveAnOldTwisted(): #T1.3
-            unittest.deferredResult(d)
-        else:
-            return d
+        return d
+
+
+class TestFlexibleWithoutPassword(testsuite.TestCase):
 
-class TestFlexibleWithoutPassword(unittest.TestCase):
     def setUp(self):
         self.checker = checkers.FlexibleCredentialsChecker(user='test')
         self.checker.allowPasswordless(True)
 
     def testCredPlaintextCorrect(self):
+
         def credPlaintextCorrectCallback(result):
             self.assertEquals(result, 'user')
             return True
         cred = tcredentials.UsernamePassword('user', '')
         d = self.checker.requestAvatarId(cred)
         d.addCallback(credPlaintextCorrectCallback)
-        if weHaveAnOldTwisted(): #T1.3
-            unittest.deferredResult(d)
-        else:
-            return d
+        return d
 
     def testCredPlaintextCorrectWithId(self):
+
         def credPlaintextCorrectWithIdCallback(result):
             self.assertEquals(result, 'requested')
             return True
@@ -103,28 +97,27 @@ class TestFlexibleWithoutPassword(unittest.TestCase):
         cred.avatarId = "requested"
         d = self.checker.requestAvatarId(cred)
         d.addCallback(credPlaintextCorrectWithIdCallback)
-        if weHaveAnOldTwisted(): #T1.3
-            unittest.deferredResult(d)
-        else:
-            return d
+        return d
+
+
+class TestCryptCheckerInit(testsuite.TestCase):
 
-class TestCryptCheckerInit(unittest.TestCase):
     def setUp(self):
         self.checker = checkers.CryptChecker(user='qi1Lftt0GZC0o')
 
     def testCredPlaintext(self):
+
         def credPlaintextCallback(result):
             self.assertEquals(result, 'user')
             return True
         cred = CredPlaintext('user', 'test')
         d = self.checker.requestAvatarId(cred)
         d.addCallback(credPlaintextCallback)
-        if weHaveAnOldTwisted(): #T1.3
-            unittest.deferredResult(d)
-        else:
-            return d
+        return d
+
+
+class TestCryptCheckerAddUser(testsuite.TestCase):
 
-class TestCryptCheckerAddUser(unittest.TestCase):
     def setUp(self):
         username = 'user'
         cryptPassword = crypt.crypt('test', 'qi')
@@ -132,42 +125,39 @@ class TestCryptCheckerAddUser(unittest.TestCase):
         self.checker.addUser(username, cryptPassword)
 
     def testCredPlaintext(self):
+
         def credPlaintextCallback(result):
             self.assertEquals(result, 'user')
             return True
         cred = CredPlaintext('user', 'test')
         d = self.checker.requestAvatarId(cred)
         d.addCallback(credPlaintextCallback)
-        if weHaveAnOldTwisted(): #T1.3
-            unittest.deferredResult(d)
-        else:
-            return d
+        return d
 
     def testCredPlaintextWrongPassword(self):
+
         def credPlaintextWrongPasswordErrback(failure):
-            self.assert_(isinstance(failure.type(), error.UnauthorizedLogin))
+            self.assert_(isinstance(failure.type(),
+                                    errors.NotAuthenticatedError))
             return True
         cred = CredPlaintext('user', 'tes')
         d = self.checker.requestAvatarId(cred)
         d.addErrback(credPlaintextWrongPasswordErrback)
-        if weHaveAnOldTwisted(): #T1.3
-            unittest.deferredResult(d)
-        else:
-            return d
+        return d
 
     def testCredPlaintextWrongUser(self):
+
         def credPlaintextWrongUserErrback(failure):
-            self.assert_(isinstance(failure.type(), error.UnauthorizedLogin))
+            self.assert_(isinstance(failure.type(),
+                                    errors.NotAuthenticatedError))
             return True
         cred = CredPlaintext('wrong', 'test')
         d = self.checker.requestAvatarId(cred)
         d.addErrback(credPlaintextWrongUserErrback)
-        if weHaveAnOldTwisted(): #T1.3
-            unittest.deferredResult(d)
-        else:
-            return d
+        return d
 
     def testCredCrypt(self):
+
         def credCryptCallback(result):
             self.assertEquals(result, 'user')
             return True
@@ -176,49 +166,43 @@ class TestCryptCheckerAddUser(unittest.TestCase):
         cred = CredCrypt('user', crypted)
         d = self.checker.requestAvatarId(cred)
         d.addCallback(credCryptCallback)
-        if weHaveAnOldTwisted(): #T1.3
-            unittest.deferredResult(d)
-        else:
-            return d
+        return d
 
     def testCredCryptWrongSalt(self):
+
         def credCryptWrongSaltErrback(failure):
-            self.assert_(isinstance(failure.type(), error.UnauthorizedLogin))
+            self.assert_(isinstance(failure.type(),
+                                    errors.NotAuthenticatedError))
             return True
         crypted = crypt.crypt('test', 'aa')
         cred = CredCrypt('user', crypted)
         d = self.checker.requestAvatarId(cred)
         d.addErrback(credCryptWrongSaltErrback)
-        if weHaveAnOldTwisted(): #T1.3
-            unittest.deferredResult(d)
-        else:
-            return d
+        return d
 
     def testCredCryptWrongPassword(self):
+
         def credCryptWrongPasswordErrback(failure):
-            self.assert_(isinstance(failure.type(), error.UnauthorizedLogin))
+            self.assert_(isinstance(failure.type(),
+                                    errors.NotAuthenticatedError))
             return True
         crypted = crypt.crypt('wrong', 'qi')
         cred = CredCrypt('user', crypted)
         d = self.checker.requestAvatarId(cred)
         d.addErrback(credCryptWrongPasswordErrback)
-        if weHaveAnOldTwisted(): #T1.3
-            unittest.deferredResult(d)
-        else:
-            return d
+        return d
 
     def testCredCryptWrongUser(self):
+
         def credCryptWrongUserErrback(failure):
-            self.assert_(isinstance(failure.type(), error.UnauthorizedLogin))
+            self.assert_(isinstance(failure.type(),
+                                    errors.NotAuthenticatedError))
             return True
         crypted = crypt.crypt('test', 'qi')
         cred = CredCrypt('wronguser', crypted)
         d = self.checker.requestAvatarId(cred)
         d.addErrback(credCryptWrongUserErrback)
-        if weHaveAnOldTwisted(): #T1.3
-            unittest.deferredResult(d)
-        else:
-            return d
+        return d
 
 if __name__ == '__main__':
-     unittest.main()
+    unittest.main()
