@@ -21,26 +21,22 @@
 
 import os
 import sys
-import time
 import tempfile
 
-from twisted.trial import unittest
-from twisted.spread import pb
-from twisted.internet import reactor, address
-
 from flumotion.common import common, package
-
-import common as testcommon
+from flumotion.common import testsuite
 
 # for our simulation of failIfRaises
 import twisted.python.util
 from twisted.trial.unittest import FailTest
 
-class TestPackagePath(unittest.TestCase):
+
+class TestPackagePath(testsuite.TestCase):
+
     def setUp(self):
         # store our old sys.path
         self.syspath = sys.path[:]
-        
+
         self._assertions = 0
 
         self.cwd = os.getcwd()
@@ -64,7 +60,7 @@ class TestPackagePath(unittest.TestCase):
                 del sys.modules[n]
         sys.path = self.syspath
         os.system("rm -r %s" % self.tempdir)
-        
+
         # sanity checks
         self.failIf('mypackage' in globals())
         self.failIf('mypackage' in sys.modules.keys())
@@ -74,6 +70,7 @@ class TestPackagePath(unittest.TestCase):
         self.assertRaises(ImportError, self._import, "mypackage.B")
 
     # create the file system layout for our test
+
     def _createOne(self):
         self.onedir = os.path.join(self.tempdir, "one")
         os.mkdir(self.onedir)
@@ -109,27 +106,32 @@ class TestPackagePath(unittest.TestCase):
 
     # a way of making import statements into functions so we can
     # use assertRaises
+
     def _import(self, which):
         exec("import %s" % which)
 
     # a way of failing if a method call raises
+
     def _failIfRaises(self, exception, f, *args, **kwargs):
         self._assertions += 1
         try:
             if twisted.python.util.raises(exception, f, *args, **kwargs):
-                raise FailTest, '%s raised' % exception.__name__
+                raise FailTest(
+                    '%s raised' % exception.__name__)
         except FailTest, e:
             raise
         except:
             # import traceback; traceback.print_exc()
-            raise FailTest, '%s raised instead of %s' % (sys.exc_info()[0],
-                                                         exception.__name__)
+            raise FailTest(
+                '%s raised instead of %s' % (
+                sys.exc_info()[0],
+                exception.__name__))
 
     def testCurrent(self):
         packager = package.getPackager()
         packager.registerPackagePath(self.tempdir, 'tempdir')
         packager.unregister()
- 
+
     def testTwoStackedProjects(self):
         # we create two stacked projects both with a "mypackage" import space
         # project one:
@@ -138,7 +140,7 @@ class TestPackagePath(unittest.TestCase):
         # project two:
         #   tempdir/two/mypackage/__init__.py
         #   tempdir/two/mypackage/B.py
-        
+
         # each has parts of a common namespace
         # set up stuff so we can import from both
         # this shows we can develop uninstalled against uninstalled
@@ -252,10 +254,16 @@ class TestPackagePath(unittest.TestCase):
     def testPackagerWithNonePrefix(self):
         # see if import hooks don't mess up non prefix hooks
         packager = package.Packager()
-        from xml.dom.html import HTMLDOMImplementation
+        try:
+            from xml.dom.html import HTMLDOMImplementation
+        except ImportError:
+            return
+        HTMLDOMImplementation
         packager.unregister()
 
-class TestRecursively(unittest.TestCase):
+
+class TestRecursively(testsuite.TestCase):
+
     def testListDir(self):
         self.tempdir = tempfile.mkdtemp()
 
