@@ -21,17 +21,16 @@
 
 from twisted.trial import unittest
 from twisted.spread import jelly
-from twisted.internet import reactor
+from zope.interface import implements
 
-import common as mcommon
-
-from flumotion.twisted import flavors
-from flumotion.twisted.compat import implements
+from flumotion.common import testsuite
 from flumotion.common import planet
-
 from flumotion.common.planet import moods
+from flumotion.twisted import flavors
+
+
+class ManagerComponentStateTest(testsuite.TestCase):
 
-class ManagerComponentStateTest(unittest.TestCase):
     def setUp(self):
         self.state = planet.ManagerComponentState()
 
@@ -41,12 +40,15 @@ class ManagerComponentStateTest(unittest.TestCase):
         self.failUnlessEqual(self.state._dict['name'], 'mynameis')
 
     def testSetWrongArg(self):
+
         def assign(object):
             object.state.set('wrongarg', 'failure')
-        
+
         self.assertRaises(KeyError, assign, self)
-    
-class AllComponentStateTest(unittest.TestCase):
+
+
+class AllComponentStateTest(testsuite.TestCase):
+
     def setUp(self):
         self.mstate = planet.ManagerComponentState()
         self.astate = jelly.unjelly(jelly.jelly(self.mstate))
@@ -71,7 +73,9 @@ class AllComponentStateTest(unittest.TestCase):
         self.failUnlessEqual(self.mstate.get('mood'), moods.lost.value)
         self.failUnlessEqual(self.astate.get('mood'), moods.lost.value)
 
-class InvalidateTest(unittest.TestCase):
+
+class InvalidateTest(testsuite.TestCase):
+
     def testInvalidate(self):
         mcomp = planet.ManagerComponentState()
         mflow = planet.ManagerFlowState()
@@ -87,6 +91,7 @@ class InvalidateTest(unittest.TestCase):
         acomp, = aflow.get('components')
 
         invalidates = []
+
         def invalidate(obj):
             invalidates.append(obj)
 
@@ -97,24 +102,26 @@ class InvalidateTest(unittest.TestCase):
         self.assertEquals(invalidates, [])
         astate.invalidate()
         self.assertEquals(invalidates, [acomp, aflow, astate])
-        
+
 # FIXME: this test doesn't do anything since unjelly(jelly()) creates a
 # new one, instead of updating the old one.  Find a way to make the old
 # serialized object update first
-class ListenerTest(unittest.TestCase):
+
+
+class ListenerTest(testsuite.TestCase):
     implements(flavors.IStateListener)
 
     def setUp(self):
         self.mstate = planet.ManagerComponentState()
         self.astate = jelly.unjelly(jelly.jelly(self.mstate))
-        self.astate.addListener(self, set=self.stateSet)
+        self.astate.addListener(self, set_=self.stateSet)
         self.changes = []
 
     def stateSet(self, state, key, value):
         # listener "interface" function
         print "state set !"
         self.changes.append((state, key, value))
-        
+
     def mset(self, key, value):
         # helper function to set on job state and propagate
         self.mstate.set(key, value)
