@@ -19,24 +19,20 @@
 
 # Headers in this file shall remain intact.
 
-
-from twisted.trial import unittest
-
 from twisted.internet import defer, reactor
-from flumotion.twisted.defer import defer_generator
+
+from flumotion.common import errors
+from flumotion.common import testsuite
+from flumotion.twisted.defer import defer_generator, RetryingDeferred
 
 
-def run_until_finished():
-    while reactor.getDelayedCalls():
-        reactor.iterate(0.01)
-
-class TestDefer(unittest.TestCase):
+class TestDefer(testsuite.TestCase):
     result = None
     error = None
 
     def testYieldResultAtFirst(self):
         self.result = None
-        
+
         def gen():
             yield 42
         gen = defer_generator(gen)
@@ -52,7 +48,7 @@ class TestDefer(unittest.TestCase):
             d = defer.Deferred()
             reactor.callLater(0.1, lambda: d.callback(x*x))
             return d
-            
+
         def gen():
             yield square_later(0)
             yield 42
@@ -60,25 +56,31 @@ class TestDefer(unittest.TestCase):
 
         d = gen()
         d.addCallback(lambda x: setattr(self, 'result', x))
-        run_until_finished()
-        assert self.result == 42
+
+        def checkResult(res):
+            assert self.result == 42
+        d.addCallback(checkResult)
+        return d
 
     def testYieldNothing(self):
         self.result = 42
-        
+
         def square_later(x):
             d = defer.Deferred()
             reactor.callLater(0.1, lambda: d.callback(x*x))
             return d
-            
+
         def gen():
             yield square_later(0)
         gen = defer_generator(gen)
 
         d = gen()
         d.addCallback(lambda x: setattr(self, 'result', x))
-        run_until_finished()
-        assert self.result == None
+
+        def checkResult(res):
+            assert self.result == None
+        d.addCallback(checkResult)
+        return d
 
     def testValues(self):
         self.result = None
@@ -87,24 +89,28 @@ class TestDefer(unittest.TestCase):
             d = defer.Deferred()
             reactor.callLater(0.1, lambda: d.callback(x*x))
             return d
-            
+
         def gen():
             for i in range(10):
                 d = square_later(i)
                 yield d
-                
+
                 # .value() gets the result of the deferred
                 assert d.value() == i*i
-                
+
             yield d.value()
         gen = defer_generator(gen)
 
         d = gen()
         d.addCallback(lambda x: setattr(self, 'result', x))
-        run_until_finished()
-        assert self.result == 81
+
+        def checkResult(res):
+            assert self.result == 81
+        d.addCallback(checkResult)
+        return d
 
     def testBarfOnNongenerator(self):
+
         def nongen():
             pass
         try:
@@ -115,8 +121,10 @@ class TestDefer(unittest.TestCase):
 
     def testException(self):
         self.result = None
+
         def divide_later(x, y):
             d = defer.Deferred()
+
             def divide():
                 try:
                     d.callback(x/y)
@@ -136,19 +144,24 @@ class TestDefer(unittest.TestCase):
             except ZeroDivisionError:
                 d = divide_later(42, 1)
                 yield d
-                
+
                 assert d.value() == 42
                 yield True
         gen = defer_generator(gen)
 
         d = gen()
         d.addCallback(lambda x: setattr(self, 'result', x))
-        run_until_finished()
-        assert self.result == True
+
+        def checkResult(res):
+            assert self.result == True
+        d.addCallback(checkResult)
+        return d
 
     def testExceptionChain(self):
+
         def divide_later(x, y):
             d = defer.Deferred()
+
             def divide():
                 try:
                     d.callback(x/y)
@@ -186,6 +199,38 @@ class TestDefer(unittest.TestCase):
         d.addErrback(zerodivisionerrorback)
         d.addErrback(runtimeerrorback)
         d.addCallback(checkexceptionchain)
-        run_until_finished()
-        assert self.result == ['oserror', 'zerodivisionerror'],\
-               "Unexpected exception chain: %r" % (self.result,)
+
+        def checkResult(res):
+            assert self.result == ['oserror', 'zerodivisionerror'],\
+                   "Unexpected exception chain: %r" % (self.result, )
+        d.addCallback(checkResult)
+        return d
+
+
+class TestRetryingDeferred(testsuite.TestCase):
+
+    def testSimple(self):
+
+        def genDef():
+            return defer.succeed(True)
+
+        rd = RetryingDeferred(genDef)
+        d = rd.start()
+
+        return d
+
+    def testRetryOnce(self):
+        self.__first = True
+
+        def genDef():
+            if self.__first:
+                self.__first = False
+                return defer.fail(errors.FlumotionError())
+            else:
+                return defer.succeed(None)
+
+        rd = RetryingDeferred(genDef)
+        rd.initialDelay = 0.1 # Set it short so the test isn't long-running.
+        d = rd.start()
+
+        return d
