@@ -19,34 +19,44 @@
 
 # Headers in this file shall remain intact.
 
-import common
-import testclasses
-import twisted
-
-from twisted.trial import unittest
-
 from twisted.internet import reactor, defer
 from twisted.spread import pb
+from zope.interface import implements
+
+from flumotion.common import testsuite
 from flumotion.twisted import flavors
-from flumotion.twisted.compat import implements
+
 
 class TestStateCacheable(flavors.StateCacheable):
     pass
+
+
 class TestStateRemoteCache(flavors.StateRemoteCache):
     pass
 
 pb.setUnjellyableForClass(TestStateCacheable, TestStateRemoteCache)
 
-class FakeObject: pass
+
+class FakeObject:
+    pass
+
+
 class FakeListener:
     # listener interface
     implements(flavors.IStateListener)
-    
-    def stateSet(self, state, key, value): pass
-    def stateAppend(self, state, key, value): pass
-    def stateRemove(self, state, key, value): pass
 
-class TestRoot(testclasses.TestManagerRoot):
+    def stateSet(self, state, key, value):
+        pass
+
+    def stateAppend(self, state, key, value):
+        pass
+
+    def stateRemove(self, state, key, value):
+        pass
+
+
+class TestRoot(testsuite.TestManagerRoot):
+
     def remote_getState(self):
         self.state = TestStateCacheable()
         self.state.addKey('name', 'lois')
@@ -75,14 +85,16 @@ class TestRoot(testclasses.TestManagerRoot):
     def remote_haveAdopted(self, name):
         return self.state.remove('children', name)
 
-class StateTest(unittest.TestCase):
+
+class StateTest(testsuite.TestCase):
+
     def setUp(self):
         self.changes = []
         self.runServer()
 
     def tearDown(self):
         return self.stopServer()
-        
+
     def runClient(self):
         f = pb.PBClientFactory()
         self.cport = reactor.connectTCP("127.0.0.1", self.port, f)
@@ -110,7 +122,9 @@ class StateTest(unittest.TestCase):
         d = self.sport.stopListening()
         return d
 
+
 class TestStateSet(StateTest):
+
     def testStateSet(self):
         d = self.runClient()
         d.addCallback(lambda _: self.perspective.callRemote('getState'))
@@ -178,19 +192,22 @@ class TestStateSet(StateTest):
         return d
 
     def listen(self, state):
+
         def event(type):
-            return lambda *x: self.changes.append((type,) + x)
-        state.addListener(self, event('set'), event('append'),
-                          event('remove'), event('setitem'),
-                          event('delitem'))
+            return lambda *x: self.changes.append((type, ) + x)
+        state.addListener(self, set_=event('set'), append=event('append'),
+                          remove=event('remove'), setitem=event('setitem'),
+                          delitem=event('delitem'))
 
     # listener tests
+
     def testStateSetListener(self):
         # start everything and get the state
         d = self.runClient()
         d.addCallback(lambda _: self.perspective.callRemote('getState'))
 
         # ask server to set the name
+
         def add_listener_and_set_name(state):
             d.state = state # monkeypatch
             self.listen(state)
@@ -271,7 +288,9 @@ class TestStateSet(StateTest):
         d.addCallback(check_remove_results_and_stop)
         return d
 
+
 class TestFullListener(StateTest):
+
     def testStateSetListener(self):
         # start everything and get the state
         d = self.runClient()
@@ -281,9 +300,10 @@ class TestFullListener(StateTest):
             self.changes.append(('set', state, key, value))
 
         # ask server to set the name
+
         def add_listener_and_set_name(state):
             d.state = state # monkeypatch
-            state.addListener(self, set=customStateSet)
+            state.addListener(self, set_=customStateSet)
             return self.perspective.callRemote('setStateName', 'robin')
 
         def check_results(_):
@@ -336,9 +356,12 @@ class TestFullListener(StateTest):
 
     def testInvalidate(self):
         calls = []
+
         def check_invalidation(state):
+
             def invalidate(obj):
                 calls.append(('invalidate', obj))
+
             def unused(*args):
                 assert False, 'should not be reached'
             self.assertEquals(calls, [])
@@ -351,8 +374,8 @@ class TestFullListener(StateTest):
             state.addListener(2, invalidate=invalidate)
             self.assertEquals(calls, [('invalidate', state),
                                       ('invalidate', state)])
-            
-            state.addListener(3, set=unused)
+
+            state.addListener(3, set_=unused)
             self.assertEquals(calls, [('invalidate', state),
                                       ('invalidate', state)])
 
@@ -364,7 +387,8 @@ class TestFullListener(StateTest):
         return d
 
 
-class TestState(unittest.TestCase):
+class TestState(testsuite.TestCase):
+
     def testStateAddKey(self):
         c = flavors.StateCacheable()
 
@@ -385,7 +409,7 @@ class TestState(unittest.TestCase):
         l = c.get('two')
         self.assertEqual(len(l), 1)
         self.assertEqual(l[0], 'B')
-        
+
     def testStateGet(self):
         c = flavors.StateCacheable()
 
@@ -393,7 +417,7 @@ class TestState(unittest.TestCase):
         c.set('akey', 'avalue')
         self.assertEquals(c.get('akey'), 'avalue')
         self.assertRaises(KeyError, c.get, 'randomkey')
-  
+
     def testStateAppendRemove(self):
         c = flavors.StateCacheable()
 
