@@ -19,26 +19,25 @@
 
 # Headers in this file shall remain intact.
 
-import common
+from flumotion.common import testsuite
 from twisted.trial import unittest
 from twisted.internet import defer
 
 from flumotion.common import keycards
 from flumotion.component.bouncers import htpasswdcrypt
 
-import twisted.copyright #T1.3
-#T1.3
-def weHaveAnOldTwisted():
-    return twisted.copyright.version[0] < '2'
 
 bouncerconf = {'name': 'testbouncer',
                'plugs': {},
                'properties': {'data': "user:qi1Lftt0GZC0o"}}
 
 # this is a type that should not be allowed
-class TestHTPasswdCryptKeycard(unittest.TestCase):
+
+
+class TestHTPasswdCryptKeycard(testsuite.TestCase):
+
     def setUp(self):
-        self.bouncer = htpasswdcrypt.HTPasswdCrypt()
+        self.bouncer = htpasswdcrypt.HTPasswdCrypt(bouncerconf)
 
     def tearDown(self):
         self.bouncer.stop()
@@ -46,67 +45,57 @@ class TestHTPasswdCryptKeycard(unittest.TestCase):
     def testWrongKeycardClass(self):
         keycard = keycards.Keycard()
         d = defer.maybeDeferred(self.bouncer.authenticate, keycard)
+
         def wrongKeycardClassCallback(result):
             self.failIf(result)
-        if weHaveAnOldTwisted():
-            result = unittest.deferredResult(d)
-            self.failIf(result)
-        else:
-            d.addCallback(wrongKeycardClassCallback)
-            return d
+        d.addCallback(wrongKeycardClassCallback)
+        return d
+
+
+class TestHTPasswdCryptUACPP(testsuite.TestCase):
 
-class TestHTPasswdCryptUACPP(unittest.TestCase):
     def setUp(self):
-        self.bouncer = htpasswdcrypt.HTPasswdCrypt()
-        self.bouncer.setup(bouncerconf)
+        self.bouncer = htpasswdcrypt.HTPasswdCrypt(bouncerconf)
 
     def tearDown(self):
         self.bouncer.stop()
         del self.bouncer
-        
+
     def testInit(self):
         self.assert_(self.bouncer._checker)
-        
+
     def testOk(self):
         keycard = keycards.KeycardUACPP('user', 'test', '127.0.0.1')
         d = defer.maybeDeferred(self.bouncer.authenticate, keycard)
+
         def okCallback(result):
             self.assertEquals(result.state, keycards.AUTHENTICATED)
-        if weHaveAnOldTwisted():
-            result = unittest.deferredResult(d)
-            self.assertEquals(result.state, keycards.AUTHENTICATED)
-        else:
-            d.addCallback(okCallback)
-            return d
+        d.addCallback(okCallback)
+        return d
 
     def testWrongUser(self):
         keycard = keycards.KeycardUACPP('wronguser', 'test', '127.0.0.1')
         d = defer.maybeDeferred(self.bouncer.authenticate, keycard)
+
         def wrongUserCallback(result):
             self.failIf(result)
-        if weHaveAnOldTwisted():
-            result = unittest.deferredResult(d)
-            self.failIf(result)
-        else:
-            d.addCallback(wrongUserCallback)
-            return d
+        d.addCallback(wrongUserCallback)
+        return d
 
     def testWrongPassword(self):
         keycard = keycards.KeycardUACPP('test', 'wrongpass', '127.0.0.1')
         d = defer.maybeDeferred(self.bouncer.authenticate, keycard)
+
         def wrongPasswordCallback(result):
             self.assertEquals(result, None)
-        if weHaveAnOldTwisted():
-            result = unittest.deferredResult(d)
-            self.assertEquals(result, None)
-        else:
-            d.addCallback(wrongPasswordCallback)
-            return d
+        d.addCallback(wrongPasswordCallback)
+        return d
+
+
+class TestHTPasswdCryptUACPCC(testsuite.TestCase):
 
-class TestHTPasswdCryptUACPCC(unittest.TestCase):
     def setUp(self):
-        self.bouncer = htpasswdcrypt.HTPasswdCrypt()
-        self.bouncer.setup(bouncerconf)
+        self.bouncer = htpasswdcrypt.HTPasswdCrypt(bouncerconf)
 
     def tearDown(self):
         self.bouncer.stop()
@@ -119,27 +108,20 @@ class TestHTPasswdCryptUACPCC(unittest.TestCase):
 
         # submit for auth
         d = defer.maybeDeferred(self.bouncer.authenticate, keycard)
+
         def okCallback(result):
             self.assertEquals(result.state, keycards.REQUESTING)
             # respond to challenge and resubmit
             result.setPassword('test')
             dd = defer.maybeDeferred(self.bouncer.authenticate, keycard)
+
             def authenticatedCallback(result):
                 self.assertEquals(result.state, keycards.AUTHENTICATED)
             dd.addCallback(authenticatedCallback)
             return dd
-            
-        if weHaveAnOldTwisted():
-            result = unittest.deferredResult(d)
-            self.assertEquals(result.state, keycards.REQUESTING)
-            # respond to challenge and resubmit
-            result.setPassword('test')
-            d = defer.maybeDeferred(self.bouncer.authenticate, keycard)
-            result = unittest.deferredResult(d)
-            self.assertEquals(result.state, keycards.AUTHENTICATED)
-        else:
-            d.addCallback(okCallback)
-            return d
+
+        d.addCallback(okCallback)
+        return d
 
     def testTamperWithChallenge(self):
         # create challenger
@@ -149,28 +131,19 @@ class TestHTPasswdCryptUACPCC(unittest.TestCase):
 
         # submit for auth
         d = defer.maybeDeferred(self.bouncer.authenticate, keycard)
+
         def tamperCallback(result):
             self.assertEquals(result.state, keycards.REQUESTING)
             # mess with challenge, respond to challenge and resubmit
             result.challenge = "I am a h4x0r"
             result.setPassword('test')
             dd = defer.maybeDeferred(self.bouncer.authenticate, keycard)
+
             def tamperAuthenticateCallback(result):
                 self.failIf(result)
             dd.addCallback(tamperAuthenticateCallback)
             return dd
-        if weHaveAnOldTwisted():
-            result = unittest.deferredResult(d)
-            self.assertEquals(result.state, keycards.REQUESTING)
-        
-            # mess with challenge, respond to challenge and resubmit
-            result.challenge = "I am a h4x0r"
-            result.setPassword('test')
-            d = defer.maybeDeferred(self.bouncer.authenticate, keycard)
-            result = unittest.deferredResult(d)
-            self.failIf(result)
-        else:
-            d.addCallback(tamperCallback)
-            return d
+        d.addCallback(tamperCallback)
+        return d
 if __name__ == '__main__':
     unittest.main()
