@@ -19,40 +19,33 @@
 
 # Headers in this file shall remain intact.
 
-import os
-import sys
-import time
-import tempfile
-
-from twisted.trial import unittest
-from twisted.spread import pb
-from twisted.internet import reactor, address
-
-from flumotion.common import common
-from flumotion.configure import configure
-
 import gettext
 import locale
+import os
+
+from twisted.spread import jelly
+from flumotion.common import messages, testsuite
+from flumotion.common.i18n import N_, gettexter, ngettext, Translator
+from flumotion.configure import configure
 
 # marking/translating for singulars
-from gettext import gettext as _
-def N_(format): return format
+_ = gettext.gettext
+T_ = gettexter()
 
 # marking for plurals can only be done with a "fake" ngettext
 # just adding, say, NP_ to --keyword for xgettext doesn't pick up on
 # the plurality
 
-# so we use this global ngettext for marking, and gettext.ngettext for
-# actually translating
-def ngettext(singular, plural, count): return (singular, plural, count)
-
 # this test uses the class-based gettext API
-class TestSingularClassbased(unittest.TestCase):
+
+
+class TestSingularClassbased(testsuite.TestCase):
+
     def setUp(self):
         localedir = os.path.join(configure.localedatadir, 'locale')
-        mo = gettext.find("flumotion", localedir, ["nl_NL"])
+        mo = gettext.find(configure.PACKAGE, localedir, ["nl_NL"])
         self.failUnless(mo)
-        self.nl = gettext.translation("flumotion", localedir, ["nl_NL"])
+        self.nl = gettext.translation(configure.PACKAGE, localedir, ["nl_NL"])
         self.failUnless(self.nl)
         self.able = N_("I am a translatable string")
         self.ed = self.nl.gettext("I am a translated string")
@@ -68,15 +61,18 @@ class TestSingularClassbased(unittest.TestCase):
 # these tests use the regular gettext API
 
 # helper class for gettext API tests
-class TestGettext(unittest.TestCase):
+
+
+class TestGettext(testsuite.TestCase):
+
     def setUp(self):
-        self.oldlocaledir = gettext.bindtextdomain('flumotion')
+        self.oldlocaledir = gettext.bindtextdomain(configure.PACKAGE)
         self.oldlocale = locale.setlocale(locale.LC_MESSAGES)
 
         # switch to nl
         localedir = os.path.join(configure.localedatadir, 'locale')
-        gettext.bindtextdomain('flumotion', localedir)
-        gettext.textdomain('flumotion')
+        gettext.bindtextdomain(configure.PACKAGE, localedir)
+        gettext.textdomain(configure.PACKAGE)
         # FIXME: for some reason locale.setlocale does not work, only env
         #locale.setlocale(locale.LC_ALL, "nl_NL")
         os.environ['LANG'] = 'nl_NL'
@@ -85,10 +81,12 @@ class TestGettext(unittest.TestCase):
         os.environ['LANGUAGE'] = 'nl_NL'
 
     def tearDown(self):
-        gettext.bindtextdomain('flumotion', self.oldlocaledir)
+        gettext.bindtextdomain(configure.PACKAGE, self.oldlocaledir)
         locale.setlocale(locale.LC_MESSAGES, self.oldlocale)
- 
+
+
 class TestSingularGettext(TestGettext):
+
     def setUp(self):
         TestGettext.setUp(self)
         self.able = N_("I am a translatable string")
@@ -102,7 +100,9 @@ class TestSingularGettext(TestGettext):
     def testTranslated(self):
         self.assertEquals(self.ed, "Ik ben een vertaalde string")
 
+
 class TestPluralGettext(TestGettext):
+
     def setUp(self):
         TestGettext.setUp(self)
 
@@ -136,4 +136,66 @@ class TestPluralGettext(TestGettext):
         self.assertEquals(self.edone % 1, "Ik vertaalde 1 ding")
         self.assertEquals(self.edcount, "Ik vertaalde %d dingen")
         self.assertEquals(self.edcount % 5, "Ik vertaalde 5 dingen")
- 
+
+
+class TranslatableTest(testsuite.TestCase):
+
+    def testTranslatable(self):
+        t = T_(N_("%s can be translated"), "I")
+        self.assertEquals(t.domain, configure.PACKAGE)
+        self.assertEquals(t.format, "%s can be translated")
+        self.assertEquals(t.args, ("I", ))
+
+    def testTranslatablePlural(self):
+        # Andy 3 is a droid in the Andy series and doesn't need translating
+        t = T_(ngettext("%s %d has %d thing", "%s %d has %d things", 5),
+            "Andy", 3, 5)
+        self.assertEquals(t.domain, configure.PACKAGE)
+        self.assertEquals(t.singular, "%s %d has %d thing")
+        self.assertEquals(t.plural, "%s %d has %d things")
+        self.assertEquals(t.count, 5)
+        self.assertEquals(t.args, ("Andy", 3, 5))
+        self.assertEquals(t.plural % t.args, "Andy 3 has 5 things")
+
+        # now translate to nl_NL
+        localedir = os.path.join(configure.localedatadir, 'locale')
+        self.nl = gettext.translation(configure.PACKAGE, localedir, ["nl_NL"])
+        self.failUnless(self.nl)
+        text = self.nl.ngettext(t.singular, t.plural, t.count) % t.args
+        self.assertEquals(text, "Andy 3 heeft 5 dingen")
+
+
+class TranslatorTest(testsuite.TestCase):
+
+    def testTranslateOne(self):
+        t = T_(N_("%s can be translated"), "Andy")
+
+        translator = Translator()
+        localedir = os.path.join(configure.localedatadir, 'locale')
+        translator.addLocaleDir(configure.PACKAGE, localedir)
+        text = translator.translateTranslatable(t, lang=["nl_NL"])
+        self.assertEquals(text, 'Andy kan vertaald worden')
+
+    def testTranslateMessage(self):
+        cmsg = messages.Error(T_(N_("Something is really wrong. ")))
+        t = T_(N_("But does %s know what ?"), "Andy")
+        cmsg.add(t)
+        mmsg = jelly.unjelly(jelly.jelly(cmsg))
+
+        translator = Translator()
+        localedir = os.path.join(configure.localedatadir, 'locale')
+        translator.addLocaleDir(configure.PACKAGE, localedir)
+
+        text = translator.translate(mmsg, lang=["nl_NL"])
+        self.assertEquals(text, "Er is iets echt mis. Maar weet Andy wat ?")
+
+
+class TestFormat(testsuite.TestCase):
+
+    def testFormat(self):
+        t = T_('string with a %s format', 'X')
+        self.assertEquals(t.untranslated(), 'string with a X format')
+
+    def testFormatNoArgument(self):
+        t = T_('string with a %s format')
+        self.assertEquals(t.untranslated(), 'string with a %s format')
