@@ -19,28 +19,19 @@
 
 # Headers in this file shall remain intact.
 
-from common import deferred_result
-
-from twisted.trial import unittest
-
 import os
-import exceptions
 
 from twisted.spread import pb
-from twisted.internet import reactor, defer
+from twisted.internet import defer, reactor
+from zope.interface import implements
 
-from flumotion.common.planet import moods
-
-from flumotion.manager import component, manager
 from flumotion.common import log, planet, interfaces, common
-from flumotion.common import setup
+from flumotion.common import errors
+from flumotion.common import testsuite
+from flumotion.common.planet import moods
+from flumotion.manager import component, manager
 from flumotion.twisted import flavors
-from flumotion.twisted.compat import implements
 
-import twisted.copyright #T1.3
-#T1.3
-def weHaveAnOldTwisted():
-    return twisted.copyright.version[0] < '2'
 
 class MyListener(log.Loggable):
     # a helper object that you can get deferreds from that fire when
@@ -49,16 +40,16 @@ class MyListener(log.Loggable):
 
     def __init__(self, state):
         self._setters = {} # (state, key, value) tuple -> list of deferred
-        state.addListener(self, set=self.stateSet)
-        
+        state.addListener(self, set_=self.stateSet)
+
     def notifyOnSet(self, state, key, value):
         self.debug("notify on state %r key %r set to value %r" % (
             state, key, value))
-        
+
         if state.hasKey(key) and state.get(key) == value:
             self.debug("key already has the given value, firing")
             return defer.succeed(None)
-            
+
         d = defer.Deferred()
         t = (state, key, value)
         if not t in self._setters.keys():
@@ -69,7 +60,7 @@ class MyListener(log.Loggable):
 
     def stateSet(self, object, key, value):
         self.debug("%r key %r set to %r" % (object, key, value))
-        
+
         t = (object, key, value)
         if t in self._setters.keys():
             list = self._setters[t]
@@ -77,10 +68,12 @@ class MyListener(log.Loggable):
                 self.debug("firing deferred %s" % d)
                 d.callback(None)
             del self._setters[t]
-    
+
+
 class FakeComponentAvatar(log.Loggable):
     ### since we fake out componentavatar, eaters need to be specified fully
     ### for the tests, ie sourceComponentName:feedName
+
     def __init__(self, name='fake', parent='eve', eaters=[], port=-1,
                  listen_host='127.0.0.1'):
         self.name = name
@@ -89,7 +82,7 @@ class FakeComponentAvatar(log.Loggable):
         self.eaters = eaters
         self.port = port
         self.listen_host = listen_host
-        
+
     def getFeeders(self):
         return [self.name + ':default']
 
@@ -98,13 +91,13 @@ class FakeComponentAvatar(log.Loggable):
 
     def getEaters(self):
         return self.eaters
-    
+
     def getClientAddress(self):
         return self.listen_host
 
     def getListenPort(self, *args):
         return self.port
-    
+
     def getName(self):
         return self.name
 
@@ -114,7 +107,9 @@ class FakeComponentAvatar(log.Loggable):
     def cleanup(self):
         pass
 
-class TestComponentMapper(unittest.TestCase):
+
+class TestComponentMapper(testsuite.TestCase):
+
     def setUp(self):
         self._mappers = {}
         self.heaven = component.ComponentHeaven(manager.Vishnu('test'))
@@ -134,7 +129,7 @@ class TestComponentMapper(unittest.TestCase):
         id = '/adam/cain'
         m = self._mappers[state]
         self.assertEquals(m, mapper)
-        
+
         m.id = id
         # insert a id -> mapper ref
         self._mappers[id] = m
@@ -150,12 +145,15 @@ class TestComponentMapper(unittest.TestCase):
 
         # a componentAvatar gets created with this avatarId
         # lookup mapper and add
-        avatar = component.ComponentAvatar(self.heaven, id, None)
+
+        class FakeAvatar:
+            pass
+        avatar = FakeAvatar()
         m = self._mappers[id]
         m.avatar = avatar
         # insert an avatar -> mapper ref
         self._mappers[avatar] = m
- 
+
         # verify we can do avatar -> (state, id) and other way
         m = self._mappers[avatar]
         self.assertEquals(m.state, state)
@@ -185,87 +183,37 @@ class TestComponentMapper(unittest.TestCase):
         self.failIf(m.id)
         self.failIf(m.avatar)
         self.assertEquals(m.state, state)
-        
-class TestComponentHeaven(unittest.TestCase):
-    def setUp(self):
-        self.heaven = component.ComponentHeaven(manager.Vishnu('test'))
 
-    def testCreateAvatar(self):
-        p = self.heaven.createAvatar('foo-bar-baz', None)
-        self.failUnless(isinstance(p, component.ComponentAvatar))
-
-        #self.assertRaises(AssertionError,
-        #                  self.heaven.createAvatar, 'does-not-exist')
-        # make sure callbacks get cancelled
-        # we moved heartbeat checks to attached
-        # p.cleanup()
-
-    def testComponentIsLocal(self):
-        a = FakeComponentAvatar()
-        self.heaven.avatars['test'] = a
-        self.failUnless(self.heaven._componentIsLocal(a))
-
-        
-    def testGetComponent(self):
-        a = self.heaven.createAvatar('prod', None)
-        self.assertEqual(self.heaven.getAvatar('prod'), a)
-        a.cleanup()
-
-    def testHasComponent(self):
-        a = self.heaven.createAvatar('prod', None)
-        self.failUnless(self.heaven.hasAvatar('prod'))
-
-        self.heaven.removeComponent(a)
-        self.failIf(self.heaven.hasAvatar('prod'))
-        self.assertRaises(KeyError, self.heaven.removeComponent, a)
-
-        a.cleanup()
-
-    def testRemoveComponent(self):
-        self.failIf(self.heaven.hasAvatar('fake'))
-
-        a = FakeComponentAvatar('fake')
-        self.assertRaises(KeyError, self.heaven.removeComponent, a)
-        self.failIf(self.heaven.hasAvatar(a.avatarId))
-
-        self.heaven.avatars[a.avatarId] = a
-        self.failUnless(self.heaven.hasAvatar(a.avatarId))
-
-        self.heaven.removeComponent(a)
-        self.failIf(self.heaven.hasAvatar(a.avatarId))
-        self.assertRaises(KeyError, self.heaven.removeComponent, a)
-
-    def testComponentEatersEmpty(self):
-        a = FakeComponentAvatar('fake')
-        self.heaven.avatars[a.avatarId] = a
-        self.assertEquals(self.heaven._getComponentEatersData(a), [])
-        
-    def testComponentsEaters(self):
-        a = FakeComponentAvatar(name='foo',
-            eaters=['bar:default', 'baz:default'])
-        self.heaven.avatars[a.avatarId] = a
-        a2 = FakeComponentAvatar(name='bar', port=1000, listen_host='bar-host')
-        self.heaven.avatars[a2.avatarId] = a2
-        a3 = FakeComponentAvatar(name='baz', port=1001, listen_host='baz-host')
-        self.heaven.avatars[a2.avatarId] = a3
 
 class FakeTransport:
+
     def getPeer(self):
         from twisted.internet.address import IPv4Address
         return IPv4Address('TCP', 'nullhost', 1)
+
     def getHost(self):
         from twisted.internet.address import IPv4Address
         return IPv4Address('TCP', 'nullhost', 1)
 
+    def loseConnection(self):
+        pass
+
+
 class FakeBroker:
+
     def __init__(self):
         self.transport = FakeTransport()
 
+
 class FakeMind(log.Loggable):
+
     def __init__(self, testcase):
         self.broker = FakeBroker()
         self.testcase = testcase
 
+    def notifyOnDisconnect(self, proc):
+        pass
+
     def callRemote(self, name, *args, **kwargs):
         self.debug('callRemote(%s, %r, %r)' % (name, args, kwargs))
         #print "callRemote(%s, %r, %r)" % (name, args, kwargs)
@@ -273,10 +221,13 @@ class FakeMind(log.Loggable):
         if hasattr(self, method):
             m = getattr(self, method)
             try:
-                result = m(*args, **kwargs)
-                self.debug('callRemote(%s) succeeded with %r' % (
-                    name, result))
-                return defer.succeed(result)
+
+                def gotResult(res):
+                    self.debug('callRemote(%s) succeeded with %r', name, res)
+                    return res
+                d = defer.maybeDeferred(m, *args, **kwargs)
+                d.addCallback(gotResult)
+                return d
             except Exception, e:
                 self.warning('callRemote(%s) failed with %s' % (
                     name, log.getExceptionMessage(e)))
@@ -284,29 +235,48 @@ class FakeMind(log.Loggable):
         else:
             raise AttributeError('no method %s on self %r' % (name, self))
 
+
 class FakeWorkerMind(FakeMind):
 
     logCategory = 'fakeworkermind'
-    
+
     def __init__(self, testcase, avatarId):
         FakeMind.__init__(self, testcase)
         self.avatarId = avatarId
+        self._createDeferreds = []
 
     def remote_getPorts(self):
-        return range(7600,7608)
+        return (range(7600,
+                      7608), False)
 
     def remote_getFeedServerPort(self):
         return 7609
 
-    def remote_create(self, avatarId, type, moduleName, methodName, config):
+    def remote_create(self, avatarId, type, moduleName, methodName, nice,
+            config):
         self.debug('remote_create(%s): logging in component' % avatarId)
-        avatar = self.testcase._loginComponent(self.avatarId,
+        d = self.testcase._loginComponent(self.avatarId,
             avatarId, moduleName, methodName, type, config)
+
+        d2 = defer.Deferred()
+        self._createDeferreds.append(d2)
+        d.addCallback(lambda _: d2.callback(avatarId))
+
         # need to return the avatarId for comparison
-        return avatarId
-    
+        d.addCallback(lambda _: avatarId)
+        return d
+
+    def waitForComponentsCreate(self):
+        d = defer.DeferredList(self._createDeferreds)
+        self._createDeferreds = []
+        return d
+
     def remote_getComponents(self):
-        return []
+        # Fire only asychronously; with an empty list.
+        d = defer.Deferred()
+        reactor.callLater(0, d.callback, [])
+        return d
+
 
 class FakeComponentMind(FakeMind):
 
@@ -317,20 +287,19 @@ class FakeComponentMind(FakeMind):
         FakeMind.__init__(self, testcase)
         self.avatarId = avatarId
         self.logName = avatarId
-        self.config = None
+        self.config = config
 
         self.info('Creating component mind for %s' % avatarId)
         state = planet.ManagerJobState()
         state._dict = {
             'type': type,
             'pid': 1,
-            'cpu': 0.1,
             'mood': moods.waking.value,
             'manager-ip': '0.0.0.0',
             'workerName': workerName,
             'feederNames': [],
             'eaterNames': [],
-            'messages': [] }
+            'messages': []}
         self.state = state
 
     def remote_getState(self):
@@ -340,30 +309,32 @@ class FakeComponentMind(FakeMind):
     def remote_getConfig(self):
         return self.config
 
+    def remote_getMasterClockInfo(self):
+        return None
+
     def remote_provideMasterClock(self, port):
+        # Turn happy; we must do this asynchronously, otherwise the tests
+        # synchronously log out a client where that shouldn't be possible.
+
+        def turnHappy():
+            self.state.observe_set('mood', moods.happy.value)
+        reactor.callLater(0, turnHappy)
+
         return ("127.0.0.1", port, 0L)
 
-    def remote_setup(self, config):
-        self.debug('remote_setup(%r)', config)
-        self.config = config
+    def remote_setMasterClock(self, ip, port, base_time):
+        return None
 
-    def remote_start(self, clocking):
-        self.debug('remote_start(%r)' % clocking)
-        self.testcase.failUnless(hasattr(self, 'state'))
-        self.testcase.failUnless(hasattr(self.state, 'observe_set'))
-        
-        self.state.observe_set('mood', moods.happy.value)
-
-    def remote_eatFrom(self, fullFeedId, host, port):
+    def remote_eatFrom(self, eaterAlias, fullFeedId, host, port):
         # pretend this works
         return
 
     def remote_feedTo(self, componentId, feedId, host, port):
         # pretend this works
         return
-    
-    
-class TestVishnu(log.Loggable, unittest.TestCase):
+
+
+class TestVishnu(testsuite.TestCase):
 
     logCategory = "TestVishnu"
 
@@ -377,6 +348,7 @@ class TestVishnu(log.Loggable, unittest.TestCase):
         self._components = {} # id -> avatar
 
     # helper functions
+
     def _requestAvatar(self, avatarId, mind, iface, avatarDict):
         d = self.vishnu.dispatcher.requestAvatar(avatarId, None,
             mind, pb.IPerspective, iface)
@@ -387,29 +359,23 @@ class TestVishnu(log.Loggable, unittest.TestCase):
             avatar._mind = mind
             avatar._avatarId = avatarId
 
-            # trigger attached
-            # twisted 2.2.0 TestCase does not have a runReactor method
-            # and according to twisted changeset 15556 it was always
-            # deprecated
-            from twisted.internet import reactor
-            reactor.iterate()
             avatarDict[avatarId] = avatar
             return avatar
         d.addCallback(got_result)
         return d
-        
+
     def _loginWorker(self, avatarId):
         # create a worker and log it in
         # return the avatar
 
         # log in a worker
         return self._requestAvatar(avatarId,
-                                   FakeWorkerMind(self, avatarId), 
+                                   FakeWorkerMind(self, avatarId),
                                    interfaces.IWorkerMedium,
                                    self._workers)
-        
+
     def _loginComponent(self, workerName, avatarId, type, moduleName,
-        methodName, config):
+            methodName, config):
         # create a component and log it in
         # return the avatar
 
@@ -427,13 +393,6 @@ class TestVishnu(log.Loggable, unittest.TestCase):
         avatarId = avatar._avatarId
 
         logout(avatarId, avatar, mind)
-        
-        # trigger detached
-        # twisted 2.2.0 TestCase does not have a runReactor method
-        # and according to twisted changeset 15556 it was always
-        # deprecated
-        from twisted.internet import reactor
-        reactor.iterate()
 
     def testWorker(self):
         names = self.vishnu.workerHeaven.state.get('names')
@@ -454,43 +413,118 @@ class TestVishnu(log.Loggable, unittest.TestCase):
         d = self._loginWorker('worker')
         d.addCallback(got_avatar)
         return d
-    testWorker = deferred_result(testWorker)
 
     def testLoadConfiguration(self):
         __thisdir = os.path.dirname(os.path.abspath(__file__))
         file = os.path.join(__thisdir, 'test.xml')
-        
-        self.vishnu.loadConfigurationXML(file, manager.LOCAL_IDENTITY)
+
+        self.vishnu.loadComponentConfigurationXML(file, manager.LOCAL_IDENTITY)
         s = self.vishnu.state
-        
+
         l = s.get('flows')
         self.failUnless(l)
         f = l[0]
         self.failUnlessEqual(f.get('name'), 'testflow')
-        
+
         l = f.get('components')
         self.failUnless(l)
 
         # FIXME: why a second time ? Maybe to check that reloading doesn't
         # change things ?
-        self.vishnu.loadConfigurationXML(file, manager.LOCAL_IDENTITY)
+        self.vishnu.loadComponentConfigurationXML(file, manager.LOCAL_IDENTITY)
 
-        # now lets empty planet and make sure theres nothing in the depgraph
-        d = self.vishnu.emptyPlanet()
+        # now lets empty planet
+        return self.vishnu.emptyPlanet()
 
-        def verifyEmptyDAG(result):
-            # remove workers that components in test.xml depend on
-            self.vishnu._depgraph.removeWorker("streamer")
-            self.vishnu._depgraph.removeWorker("worker")
-            # make sure the depgraph is empty
-            self.assertEqual(self.vishnu._depgraph._dag._nodes,{})
+    def testLoadComponentWithSynchronization(self):
 
-        if weHaveAnOldTwisted():
-            unittest.deferredResult(d)
-            verifyEmptyDAG(None)
-        else:
-            d.addCallback(verifyEmptyDAG)
-            return d
+        def loadProducer():
+            compType = "pipeline-producer"
+            compId = common.componentId("testflow", "producer-video-test")
+            compProps = [
+                ("pipeline", "videotestsrc ! video/x-raw-yuv,width=320,"
+                 "height=240,framerate=5/1,format=(fourcc)I420")]
+            return self.vishnu.loadComponent(manager.LOCAL_IDENTITY,
+                                             compType, compId, None, compProps,
+                                             "worker", [], [], False, [])
+        # Add more tests if you implement handling of sync-needing components
+        self.assertRaises(NotImplementedError, loadProducer)
+
+    def testLoadComponent(self):
+
+        def loadProducerFromFile():
+            __thisdir = os.path.dirname(os.path.abspath(__file__))
+            file = os.path.join(__thisdir, 'testLoadComponent.xml')
+            return self.vishnu.loadComponentConfigurationXML(
+                file, manager.LOCAL_IDENTITY)
+
+        def loadConverter(_):
+            flows = self.vishnu.state.get('flows')
+            self.assertEqual(len(flows), 1)
+            flow = flows[0]
+            self.assertEqual(flow.get('name'), 'testflow')
+            components = flow.get('components')
+            self.assertEqual(len(components), 1)
+            self.assertEqual(components[0].get('name'),
+                             "producer-video-test")
+
+            compType = "pipeline-converter"
+            compId = common.componentId("testflow", "converter-ogg-theora")
+            compEaters = [("default", "producer-video-test")]
+            compProps = [("pipeline", "ffmpegcolorspace ! theoraenc "
+                         "keyframe-force=5 ! oggmux")]
+            compState = self.vishnu.loadComponent(
+                manager.LOCAL_IDENTITY, compType, compId, None, compProps,
+                "worker", [], compEaters, False, [])
+
+            self.assertEqual(compState.get('config').get('name'),
+                             "converter-ogg-theora")
+            self.failIf('label' in compState.get('config'))
+            self.assertEqual(len(components), 2)
+            self.assertEqual(components[1].get('name'),
+                             "converter-ogg-theora")
+
+            # Loading the same component again raises an error
+            self.assertRaises(errors.ComponentAlreadyExistsError,
+                              self.vishnu.loadComponent,
+                              manager.LOCAL_IDENTITY, compType, compId,
+                              None, compProps, "worker", [], compEaters,
+                              False, [])
+
+            compType = "http-streamer"
+            compId = common.componentId("testflow", "streamer-ogg-theora")
+            compLabel = "Streamer OGG-Theora/Vorbis"
+            compEaters = [("default", "converter-ogg-theora")]
+            compProps = [("port", 8800)]
+            compState = self.vishnu.loadComponent(
+                manager.LOCAL_IDENTITY, compType, compId, compLabel,
+                compProps, "streamer", [], compEaters, False, [])
+
+            self.assertEqual(compState.get('config').get('name'),
+                             "streamer-ogg-theora")
+            self.assertEqual(compState.get('config').get('label'),
+                             "Streamer OGG-Theora/Vorbis")
+            self.assertEqual(len(components), 3)
+            self.assertEqual(components[2].get('name'),
+                             "streamer-ogg-theora")
+
+            # Load a component to atmosphere
+            compType = "ical-bouncer"
+            compId = common.componentId("atmosphere", "test-bouncer")
+            compProps = [("file", "icalfile")]
+            compState = self.vishnu.loadComponent(
+                manager.LOCAL_IDENTITY, compType, compId, None,
+                compProps, "worker", [], [], False, [])
+
+            atmosphere = self.vishnu.state.get('atmosphere')
+            components = atmosphere.get('components')
+            self.assertEquals(len(components), 1)
+            self.failUnlessEqual(components[0].get('name'),
+                                 'test-bouncer')
+
+        d = loadProducerFromFile()
+        d.addCallback(loadConverter)
+        return d
 
     def testConfigBeforeWorker(self):
         # test a config with three components being loaded before the worker
@@ -501,35 +535,24 @@ class TestVishnu(log.Loggable, unittest.TestCase):
         # and their cleanup
         __thisdir = os.path.dirname(os.path.abspath(__file__))
         file = os.path.join(__thisdir, 'test.xml')
-        
+
         def confLoaded(_):
             # verify component mapper
             # 3 component states + avatarId's gotten from the config
             self.assertEqual(len(mappers.keys()), 6)
-            # verify depgraph
             id = '/testflow/producer-video-test'
             state = mappers[id].state
             assert state, state
-            dag = self.vishnu._depgraph._dag
-            o = dag.getOffspringTyped(state, "COMPONENTSTART")
-            names = [s.get('name') for s,t in o]
-            self.failIf('producer-video-test' in names)
-            self.failUnless('converter-ogg-theora' in names)
-            self.failUnless('streamer-ogg-theora' in names)
 
-            # verify that nothing should be started
-            start = self.vishnu._depgraph.whatShouldBeStarted()
-            # should be nothing because we have no worker
-            assert start == []
-            
             # log in a worker and verify components get started
             return self._loginWorker('worker')
 
         def gotWorker(workerAvatar):
-            d = self._verifyConfigAndOneWorker()
+            d = workerAvatar.mind.waitForComponentsCreate()
+            d.addCallback(lambda _: self._verifyConfigAndOneWorker())
             d.addCallback(lambda _: workerAvatar)
             return d
-        
+
         def confChecked(workerAvatar):
             # log out the producer and verify the mapper
             id = '/testflow/producer-video-test'
@@ -562,27 +585,33 @@ class TestVishnu(log.Loggable, unittest.TestCase):
             # Now log out the worker.
             self._logoutAvatar(workerAvatar)
 
-        d = self.vishnu.loadConfigurationXML(file, manager.LOCAL_IDENTITY)
+        d = self.vishnu.loadComponentConfigurationXML(
+            file, manager.LOCAL_IDENTITY)
         d.addCallback(confLoaded)
         d.addCallback(gotWorker)
         d.addCallback(confChecked)
         return d
-    testConfigBeforeWorker = deferred_result(testConfigBeforeWorker)
+    testConfigBeforeWorker.skip = 'andy will definitely not fix this soon'
 
     def testConfigAfterWorker(self):
         # test a config with three components being loaded after the worker
         # logs in
+
         def loadConfigAndOneWorker(workerAvatar):
             log.debug('unittest', 'loadConfigAndOneWorker')
             self.failUnlessEqual(len(self._workers), 1)
             self.failUnlessEqual(len(self._components), 0)
-            
+
             # load configuration
-            d = self.vishnu.loadConfigurationXML(file, manager.LOCAL_IDENTITY)
+            d = self.vishnu.loadComponentConfigurationXML(
+                file, manager.LOCAL_IDENTITY)
+            d.addCallback(lambda _:
+                          self._workers['worker'].mind.waitForComponentsCreate(
+                ))
             d.addCallback(lambda _: self._verifyConfigAndOneWorker())
             d.addCallback(lambda _: workerAvatar)
             return d
-        
+
         def logoutComponent(workerAvatar):
             log.debug('unittest', 'logoutComponent: producer')
             # log out the producer and verify the mapper
@@ -599,7 +628,7 @@ class TestVishnu(log.Loggable, unittest.TestCase):
             # We logged it out without it doing a clean shutdown, so it should
             # now be lost.
             self._verifyComponentIdGone(id, moods.lost)
-            
+
             # log out the converter and verify
             log.debug('unittest', 'logoutComponent: converter')
             id = '/testflow/converter-ogg-theora'
@@ -621,15 +650,6 @@ class TestVishnu(log.Loggable, unittest.TestCase):
         def emptyPlanet(_):
             return self.vishnu.emptyPlanet()
 
-        def removeWorkersAndCheckDAG(result):
-            # make sure the depgraph is empty
-            # remove worker 'streamer' that is set as the worker for 
-            # http-streamer in test.xml
-            # and remove worker 'worker' that was logged in, in this test
-            self.vishnu._depgraph.removeWorker("streamer")
-            self.vishnu._depgraph.removeWorker("worker")
-
-            self.assertEqual(self.vishnu._depgraph._dag._nodes,{})
         def verifyMappersIsZero(result):
             self.assertEqual(len(mappers.keys()), 0)
 
@@ -640,10 +660,8 @@ class TestVishnu(log.Loggable, unittest.TestCase):
         d.addCallback(loadConfigAndOneWorker)
         d.addCallback(logoutComponent)
         d.addCallback(emptyPlanet)
-        d.addCallback(removeWorkersAndCheckDAG)
         d.addCallback(verifyMappersIsZero)
         return d
-    testConfigAfterWorker = deferred_result(testConfigAfterWorker)
 
     def _verifyConfigAndOneWorker(self):
         self.debug('verifying after having loaded config and started worker')
@@ -679,9 +697,11 @@ class TestVishnu(log.Loggable, unittest.TestCase):
 
         state = m.jobState
         l = MyListener(state)
+        self.debug("Waiting for component producer-video-test to go happy")
         d = l.notifyOnSet(state, 'mood', moods.happy.value)
 
         def verifyMoodIsHappy(result):
+            self.debug("Turned happy")
             self.assertEqual(state.get('mood'), moods.happy.value,
                              "mood of %s is not happy but %r" % (
                 m.state.get('name'), moods.get(state.get('mood'))))
@@ -690,7 +710,7 @@ class TestVishnu(log.Loggable, unittest.TestCase):
             self.failUnless(avatar.componentState)
         d.addCallback(verifyMoodIsHappy)
         return d
-        
+
     def _verifyConfigAndNoWorker(self):
         mappers = self.vishnu._componentMappers
 
@@ -705,11 +725,11 @@ class TestVishnu(log.Loggable, unittest.TestCase):
         mappers = self.vishnu._componentMappers
         m = mappers[id]
         avatar = self._components[id]
-  
+
         self.failUnless(id in mappers.keys())
         self.assertEqual(m.id, id)
         self.failIf(avatar in mappers.keys())
-        
+
         self.assertEqual(m.avatar, None)
         self.failUnless(m.state)
         state = m.state
@@ -733,3 +753,39 @@ class TestVishnu(log.Loggable, unittest.TestCase):
         self.failIf(avatar.jobState)
         self.failIf(avatar.componentState)
 
+    def testDeleteFlow(self):
+        __thisdir = os.path.dirname(os.path.abspath(__file__))
+        file = os.path.join(__thisdir, 'test.xml')
+
+        self.vishnu.loadComponentConfigurationXML(file, manager.LOCAL_IDENTITY)
+        s= self.vishnu.state
+        l = s.get('flows')
+        self.failUnless(l)
+        f = l[0]
+        self.failUnlessEqual(f.get('name'), 'testflow')
+        cs = f.get('components')
+        self.failUnless(cs)
+
+        self.assertRaises(ValueError, self.vishnu.deleteFlow, 'does-not-exist')
+
+        cs[0].addKey('moodPending', True)
+        self.assertRaises(errors.BusyComponentError,
+                          self.vishnu.deleteFlow, 'testflow')
+        cs[0].addKey('moodPending', None)
+
+        # Test atomicity
+        first = cs[0]
+        cs[1].addKey('moodPending', True)
+        self.assertRaises(errors.BusyComponentError,
+                          self.vishnu.deleteFlow, 'testflow')
+        cs[1].addKey('moodPending', None)
+        self.failUnless(first in cs)
+
+        self.vishnu.deleteFlow('testflow')
+        l = s.get('flows')
+        self.failIf(l)
+
+        self.assertRaises(ValueError, self.vishnu.deleteFlow, 'testflow')
+
+        # now lets empty planet
+        return self.vishnu.emptyPlanet()
