@@ -19,54 +19,53 @@
 
 # Headers in this file shall remain intact.
 
-import common
-from twisted.trial import unittest
-import twisted.copyright #T1.3
-
 import crypt
 
+from twisted.cred import portal
 from twisted.internet import defer, reactor
 from twisted.python import log as tlog
 from twisted.spread import pb as tpb
-from twisted.cred import credentials as tcredentials
-from twisted.cred import checkers as tcheckers
-from twisted.cred import portal, error
+from twisted.trial import unittest
+from zope.interface import implements
 
-from flumotion.twisted import checkers, credentials, pb
-from flumotion.twisted import portal as fportal
-from flumotion.twisted.compat import implements
-
-from flumotion.common import keycards, log, interfaces
+from flumotion.common import testsuite
+from flumotion.common import keycards, log, errors
 from flumotion.component.bouncers import htpasswdcrypt, saltsha256
+from flumotion.twisted import checkers, pb
+from flumotion.twisted import portal as fportal
+
+from flumotion.test.common import haveTwisted
 
 htpasswdcryptConf = {
-    'name':  'testbouncer',
+    'name': 'testbouncer',
     'plugs': {},
-    'properties': {'data': "user:qi1Lftt0GZC0o"}
-}
+    'properties': {'data': "user:qi1Lftt0GZC0o"}}
 
 saltsha256Conf = {
-    'name':  'testbouncer',
+    'name': 'testbouncer',
     'plugs': {},
-    'properties': {'data': "user:iamsalt:2f826124ada2b2cdf11f4fd427c9ca48de0ed49b41476266d8df08d2cf86120e"}
-}
-
-#T1.3
-def weHaveAnOldTwisted():
-    return twisted.copyright.version < '2.0.0'
+    'properties': {
+    'data':
+    ("user:"
+     "iamsalt:"
+     "2f826124ada2b2cdf11f4fd427c9ca48de0ed49b41476266d8df08d2cf86120e")}}
 
 ### lots of fake objects to have fun with
 
+
 class FakePortalWrapperPlaintext:
     # a fake wrapper with a checker that lets username, password in
+
     def __init__(self):
         self.broker = FakeBroker()
-        self.checker = tcheckers.InMemoryUsernamePasswordDatabaseDontUse()
+        self.checker = checkers.FlexibleCredentialsChecker()
         self.checker.addUser("username", "password")
         self.portal = portal.Portal(FakeTRealm(), (self.checker, ))
 
+
 class FakePortalWrapperCrypt:
     # a fake wrapper with a checker that lets username, crypt(password, iq) in
+
     def __init__(self):
         self.checker = checkers.CryptChecker()
         cryptPassword = crypt.crypt('password', 'iq')
@@ -74,52 +73,72 @@ class FakePortalWrapperCrypt:
         self.portal = portal.Portal(FakeTRealm(), (self.checker, ))
 
 # FIXME: using real portal
+
+
 class FakeBouncerPortal:
     # a fake wrapper implementing BouncerPortal lookalike
+
     def __init__(self, bouncer):
         self.bouncer = bouncer
 
     def login(self, keycard, mind, interfaces):
         return self.bouncer.authenticate(keycard)
 
+
 class FakeAvatar(tpb.Avatar):
     implements(tpb.IPerspective)
     loggedIn = loggedOut = False
-    
+
     def __init__(self):
         pass
 
     def logout(self):
         self.loggedOut = True
 
+
 class FakeTRealm:
+
     def __init__(self):
         self.avatar = FakeAvatar()
+
     def requestAvatar(self, avatarId, mind, *ifaces):
         interface = ifaces[0]
-        assert interface == tpb.IPerspective, "interface is %r and not IPerspective" % interface
+        assert interface == tpb.IPerspective, (
+            "interface is %r and not IPerspective" % interface)
         self.avatar.loggedIn = True
         # we can return a deferred, or return directly
-        return defer.succeed((tpb.IPerspective, self.avatar, self.avatar.logout))
+        return defer.succeed((tpb.IPerspective,
+                              self.avatar, self.avatar.logout))
+
 
 class FakeFRealm(FakeTRealm):
+
     def requestAvatar(self, avatarId, keycard, mind, *interfaces):
         return FakeTRealm.requestAvatar(self, avatarId, mind, *interfaces)
 
+
 class FakeMind(tpb.Referenceable):
     pass
 
+
 class FakeBroker(tpb.Broker):
     pass
 
 # our test for twisted's challenger
 # this is done for comparison with our challenger
-class TestTwisted_PortalAuthChallenger(unittest.TestCase):
+
+
+class TestTwisted_PortalAuthChallenger(testsuite.TestCase):
+    if haveTwisted(8, 0):
+        # FIXME: don't directly use twisted's private classes!
+        skip = 'Test broken since twisted 8.0.'
+
     def setUp(self):
         # PB server creates a challenge
         self.challenge = tpb.challenge()
         # and a challenger to send to the client
-        self.challenger = tpb._PortalAuthChallenger(FakePortalWrapperPlaintext(), 
+        self.challenger = tpb._PortalAuthChallenger(
+            FakePortalWrapperPlaintext(),
             'username', self.challenge)
 
     def testRightPassword(self):
@@ -132,30 +151,29 @@ class TestTwisted_PortalAuthChallenger(unittest.TestCase):
         # client is asked to respond, so generate the response
         response = tpb.respond(self.challenge, 'wrong')
         d = self.challenger.remote_respond(response, None)
-        
-        def wrongPasswordErrback(wrongpasserror):
-            self.assert_(isinstance(wrongpasserror.type(), error.UnauthorizedLogin))
 
-        if weHaveAnOldTwisted(): #T1.3
-            failure = unittest.deferredError(d)
-            failure.trap(error.UnauthorizedLogin)
-        else:
-            d.addErrback(wrongPasswordErrback)
-            return d
+        def wrongPasswordErrback(wrongpasserror):
+            self.assert_(isinstance(wrongpasserror.type(),
+                                    errors.NotAuthenticatedError))
+
+        d.addErrback(wrongPasswordErrback)
+        return d
 
 ### SHINY NEW FPB
-class Test_BouncerWrapper(unittest.TestCase):
+
+
+class Test_BouncerWrapper(testsuite.TestCase):
+
     def setUp(self):
         broker = FakeBroker()
 
-        self.bouncer = htpasswdcrypt.HTPasswdCrypt()
-        self.bouncer.setup(htpasswdcryptConf)
+        self.bouncer = htpasswdcrypt.HTPasswdCrypt(htpasswdcryptConf)
         self.bouncerPortal = fportal.BouncerPortal(FakeFRealm(), self.bouncer)
         self.wrapper = pb._BouncerWrapper(self.bouncerPortal, broker)
 
     def tearDown(self):
         self.bouncer.stop()
-        
+
     def testUACPPOk(self):
         mind = FakeMind()
         keycard = keycards.KeycardUACPP('user', 'test', '127.0.0.1')
@@ -165,60 +183,52 @@ class Test_BouncerWrapper(unittest.TestCase):
         def uacppOkCallback(result):
             self.assert_(isinstance(result, tpb.AsReferenceable))
             return result
-        
+
         d.addCallback(uacppOkCallback)
-        if weHaveAnOldTwisted(): #1.3
-            result = unittest.deferredResult(d)
-        else:
-            return d
-    
+        return d
+
     def testUACPPWrongPassword(self):
         keycard = keycards.KeycardUACPP('user', 'tes', '127.0.0.1')
-        d = self.wrapper.remote_login(keycard, "avatarId", 
+        d = self.wrapper.remote_login(keycard, "avatarId",
             'twisted.spread.pb.IPerspective')
-        
+
         def uacppWrongPasswordErrback(wrongpasserror):
-            self.assert_(isinstance(wrongpasserror.type(), error.UnauthorizedLogin))
-        
-        if weHaveAnOldTwisted(): #T1.3
-            failure = unittest.deferredError(d)
-            failure.trap(error.UnauthorizedLogin)
-        else:
-            d.addErrback(uacppWrongPasswordErrback)
-            return d
+            self.assert_(isinstance(wrongpasserror.type(),
+                                    errors.NotAuthenticatedError))
+
+        d.addErrback(uacppWrongPasswordErrback)
+        return d
 
     def testUACPCCOk(self):
         # create
         keycard = keycards.KeycardUACPCC('user', '127.0.0.1')
 
         # send
-        d = self.wrapper.remote_login(keycard, None, 
+        d = self.wrapper.remote_login(keycard, None,
             'twisted.spread.pb.IPerspective')
-        
+
         def uacpccOkCallback(keycard):
             self.assertEquals(keycard.state, keycards.REQUESTING)
             # respond to challenge
             keycard.setPassword('test')
-            d = self.wrapper.remote_login(keycard, None, 
+            d = self.wrapper.remote_login(keycard, None,
                 'twisted.spread.pb.IPerspective')
+
             def uacpccOkCallback2(result):
                 self.assert_(isinstance(result, tpb.AsReferenceable))
                 return result
             d.addCallback(uacpccOkCallback2)
             return d
-        
+
         d.addCallback(uacpccOkCallback)
-        if weHaveAnOldTwisted(): #T1.3
-            unittest.deferredResult(d)
-        else:
-            return d
-            
+        return d
+
     def testUACPCCWrongUser(self):
         # create
         keycard = keycards.KeycardUACPCC('wronguser', '127.0.0.1')
 
         # send
-        d = self.wrapper.remote_login(keycard, "avatarId", 
+        d = self.wrapper.remote_login(keycard, "avatarId",
             'twisted.spread.pb.IPerspective')
 
         def uacpccWrongUserCallback(keycard):
@@ -227,19 +237,16 @@ class Test_BouncerWrapper(unittest.TestCase):
             keycard.setPassword('test')
             d = self.wrapper.remote_login(keycard, "avatarId",
                 'twisted.spread.pb.IPerspective')
+
             def uacpccWrongUserErrback(failure):
                 self.assert_(isinstance(failure.type(),
-                    error.UnauthorizedLogin))
+                    errors.NotAuthenticatedError))
                 return True
             d.addErrback(uacpccWrongUserErrback)
             return d
-        
+
         d.addCallback(uacpccWrongUserCallback)
-        if weHaveAnOldTwisted(): #T1.3
-            result = unittest.deferredResult(d)
-            self.assertEquals(result, True)
-        else:
-            return d
+        return d
 
     def testUACPCCWrongPassword(self):
         # create
@@ -255,19 +262,16 @@ class Test_BouncerWrapper(unittest.TestCase):
             keycard.setPassword('wrong')
             d = self.wrapper.remote_login(keycard, "avatarId",
                 'twisted.spread.pb.IPerspective')
+
             def uacpccWrongPasswordErrback(failure):
                 self.assert_(isinstance(failure.type(),
-                    error.UnauthorizedLogin))
+                    errors.NotAuthenticatedError))
                 return True
             d.addErrback(uacpccWrongPasswordErrback)
             return d
 
         d.addCallback(uacpccWrongPasswordCallback)
-        if weHaveAnOldTwisted(): #T1.3
-            result = unittest.deferredResult(d)
-            self.assertEquals(result, True)
-        else:
-            return d
+        return d
 
     def testUACPCCTamperWithChallenge(self):
         # create challenger
@@ -287,19 +291,19 @@ class Test_BouncerWrapper(unittest.TestCase):
             keycard.setPassword('test')
             d = self.wrapper.remote_login(keycard, "avatarId",
                 'twisted.spread.pb.IPerspective')
+
             def uacpccTamperErrback(failure):
                 self.assert_(isinstance(failure.type(),
-                    error.UnauthorizedLogin))
+                    errors.NotAuthenticatedError))
             d.addErrback(uacpccTamperErrback)
             return d
 
         d.addCallback(uacpccTamperCallback)
-        if weHaveAnOldTwisted(): #T1.3
-            unittest.deferredResult(d)
-        else:
-            return d
+        return d
+
+
+class Test_FPortalRoot(testsuite.TestCase):
 
-class Test_FPortalRoot(unittest.TestCase):
     def setUp(self):
         self.bouncerPortal = fportal.BouncerPortal(FakeFRealm(), 'bouncer')
         self.root = pb._FPortalRoot(self.bouncerPortal)
@@ -309,12 +313,15 @@ class Test_FPortalRoot(unittest.TestCase):
         self.failUnless(isinstance(root, pb._BouncerWrapper))
         self.assertEquals(root.broker, 'a')
 
-class TestAuthenticator(unittest.TestCase):
+
+class TestAuthenticator(testsuite.TestCase):
+
     def testIssueNoInfo(self):
         # not setting any useful auth info on the authenticator does not
         # allow us to issue a keycard
         a = pb.Authenticator(username="tarzan")
-        d = a.issue(["flumotion.common.keycards.KeycardUACPP",])
+        d = a.issue([
+            "flumotion.common.keycards.KeycardUACPP", ])
         d.addCallback(lambda r: self.failIf(r))
         return d
 
@@ -322,41 +329,45 @@ class TestAuthenticator(unittest.TestCase):
         # our authenticator by default only does challenge-based keycards
         a = pb.Authenticator(username="tarzan", password="jane",
             address="localhost")
-        d = a.issue(["flumotion.common.keycards.KeycardUACPP",])
+        d = a.issue([
+            "flumotion.common.keycards.KeycardUACPP", ])
         d.addCallback(lambda r: self.failIf(r))
 
     def testIssueUACPCC(self):
         a = pb.Authenticator(username="tarzan", password="jane",
             address="localhost")
-        d = a.issue(["flumotion.common.keycards.KeycardUACPCC",])
+        d = a.issue([
+            "flumotion.common.keycards.KeycardUACPCC", ])
         d.addCallback(lambda r: self.failUnless(isinstance(r,
             keycards.KeycardUACPCC)))
         return d
 
 # time for the big kahuna
 # base class so we can use different bouncers
-class Test_FPBClientFactory(unittest.TestCase):
+
+
+class Test_FPBClientFactory(testsuite.TestCase):
 
     def setUp(self):
         self.realm = FakeFRealm()
-        self.bouncer = self.bouncerClass()
-        self.bouncer.setup(self.bouncerConf)
+        self.bouncer = self.bouncerClass(self.bouncerConf)
         self.portal = fportal.BouncerPortal(self.realm, self.bouncer)
         self.serverFactory = tpb.PBServerFactory(self.portal,
             unsafeTracebacks=0)
         self.port = reactor.listenTCP(0, self.serverFactory,
             interface="127.0.0.1")
         self.portno = self.port.getHost().port
-        # don't output Twisted tracebacks for PB errors we will trigger
-        log._getTheFluLogObserver().ignoreErrors(error.UnauthorizedLogin)
+
+    def flushNotAuthenticatedError(self):
+        try:
+            self.flushLoggedErrors(errors.NotAuthenticatedError)
+        except AttributeError:
+            tlog.flushErrors(errors.NotAuthenticatedError)
 
     def tearDown(self):
         self.bouncer.stop()
-        tlog.flushErrors(error.UnauthorizedLogin)
-        log._getTheFluLogObserver().clearIgnores()
+        self.flushNotAuthenticatedError()
         self.port.stopListening()
-        reactor.iterate()
-        reactor.iterate()
 
     def clientDisconnect(self, factory, reference):
         # clean up broker by waiting on Disconnect notify
@@ -368,29 +379,30 @@ class Test_FPBClientFactory(unittest.TestCase):
             return d
 
 # test with htpasswdcrypt bouncer first
+
+
 class Test_FPBClientFactoryHTPasswdCrypt(Test_FPBClientFactory):
     bouncerClass = htpasswdcrypt.HTPasswdCrypt
     bouncerConf = htpasswdcryptConf
+
     def testOk(self):
         factory = pb.FPBClientFactory()
         a = pb.Authenticator(username="user", password="test",
             address="127.0.0.1")
 
-        # send 
+        # send
         d = factory.login(a)
         c = reactor.connectTCP("127.0.0.1", self.portno, factory)
-        
+
         def OkCallback(result):
             # make sure we really used challenge/response keycard
-            self.failUnless(isinstance(factory.keycard, keycards.KeycardUACPCC))
+            self.failUnless(isinstance(factory.keycard,
+                                       keycards.KeycardUACPCC))
             self.assert_(isinstance(result, tpb.RemoteReference))
             return self.clientDisconnect(factory, result)
-        
+
         d.addCallback(OkCallback)
-        if weHaveAnOldTwisted():
-            result = unittest.deferredResult(d)
-        else:
-            return d
+        return d
 
     def testWrongPassword(self):
         factory = pb.FPBClientFactory()
@@ -402,47 +414,45 @@ class Test_FPBClientFactoryHTPasswdCrypt(Test_FPBClientFactory):
         c = reactor.connectTCP("127.0.0.1", self.portno, factory)
 
         log.debug("trial", "wait for result")
+
         def WrongPasswordErrback(failure):
-            self.failUnless(isinstance(factory.keycard, keycards.KeycardUACPCC))
+            self.failUnless(isinstance(factory.keycard,
+                                       keycards.KeycardUACPCC))
             # This is a CopiedFailure
             self.assert_(failure.check(
-                "twisted.cred.error.UnauthorizedLogin"))
+                "flumotion.common.errors.NotAuthenticatedError"))
             log.debug("trial", "got failure %r" % failure)
             c.disconnect()
             return True
-        if weHaveAnOldTwisted():
-            unittest.deferredError(d)
-        else:
-            d.addErrback(WrongPasswordErrback)
-            return d
-        
+        d.addErrback(WrongPasswordErrback)
+        return d
+
 # FIXME: rewrite such that we can enforce a challenger, possibly
 # by setting a property on the bouncer
+
     def notestUACPCCOk(self):
         factory = pb.FPBClientFactory()
 
         # send
         d = factory.login(self.authenticator, 'MIND')
         c = reactor.connectTCP("127.0.0.1", self.portno, factory)
-        
-        def OkCallback(keycard):
+
+        def uacpccOkCallback(keycard):
             # get result
             self.assertEquals(keycard.state, keycards.REQUESTING)
             # respond to challenge
             keycard.setPassword('test')
             d = factory.login(keycard, 'MIND')
             # check if we have a remote reference
+
             def uacpccOkCallback2(result):
                 self.assert_(isinstance(result, tpb.RemoteReference))
                 return self.clientDisconnect(factory, result)
             d.addCallback(uacpccOkCallback2)
             return d
-        
+
         d.addCallback(uacpccOkCallback)
-        if weHaveAnOldTwisted():
-            unittest.deferredResult(d)
-        else:
-            return d
+        return d
 
     def testWrongUser(self):
         factory = pb.FPBClientFactory()
@@ -454,25 +464,20 @@ class Test_FPBClientFactoryHTPasswdCrypt(Test_FPBClientFactory):
         # send
         d = factory.login(a)
         c = reactor.connectTCP("127.0.0.1", self.portno, factory)
-        
+
         def WrongUserCb(keycard):
-            self.fail("Should have returned UnauthorizedLogin")
-            
+            self.fail("Should have returned NotAuthenticatedError")
+
         def WrongUserEb(failure):
             # find copied failure
             self.failUnless(failure.check(
-                "twisted.cred.error.UnauthorizedLogin"))
-            from twisted.cred.error import UnauthorizedLogin
-            tlog.flushErrors(UnauthorizedLogin)
+                "flumotion.common.errors.NotAuthenticatedError"))
             return self.clientDisconnect(factory, None)
-    
+
         d.addCallback(WrongUserCb)
         d.addErrback(WrongUserEb)
-    
-        if weHaveAnOldTwisted():
-            unittest.deferredResult(d)
-        else:
-            return d
+
+        return d
 
     def notestUACPCCWrongPassword(self):
         factory = pb.FPBClientFactory()
@@ -483,7 +488,7 @@ class Test_FPBClientFactoryHTPasswdCrypt(Test_FPBClientFactory):
         # send
         d = factory.login(keycard, 'MIND')
         c = reactor.connectTCP("127.0.0.1", self.portno, factory)
-        
+
         def uacpccWrongPasswordCallback(keycard):
             self.assertEquals(keycard.state, keycards.REQUESTING)
 
@@ -494,19 +499,14 @@ class Test_FPBClientFactoryHTPasswdCrypt(Test_FPBClientFactory):
             def uacpccWrongPasswordErrback(failure):
                 # find copied failure
                 self.failUnless(failure.check(
-                    "twisted.cred.error.UnauthorizedLogin"))
-                from twisted.cred.error import UnauthorizedLogin
-                tlog.flushErrors(UnauthorizedLogin)
+                    "flumotion.common.errors.NotAuthenticatedError"))
                 return self.clientDisconnect(factory, None)
-            
+
             d.addErrback(uacpccWrongPasswordErrback)
             return d
-        
+
         d.addCallback(uacpccWrongPasswordCallback)
-        if weHaveAnOldTwisted():
-            unittest.deferredResult(d)
-        else:
-            return d
+        return d
 
     def notestUACPCCTamperWithChallenge(self):
         factory = pb.FPBClientFactory()
@@ -519,7 +519,7 @@ class Test_FPBClientFactoryHTPasswdCrypt(Test_FPBClientFactory):
         # send
         d = factory.login(keycard, 'MIND')
         c = reactor.connectTCP("127.0.0.1", self.portno, factory)
-        
+
         def uacpccTamperCallback(keycard):
             self.assertEquals(keycard.state, keycards.REQUESTING)
 
@@ -527,24 +527,21 @@ class Test_FPBClientFactoryHTPasswdCrypt(Test_FPBClientFactory):
             keycard.challenge = "I am a h4x0r"
             keycard.setPassword('test')
             d = factory.login(keycard, 'MIND')
-            
+
             def uacpccTamperErrback(failure):
                 # find copied failure
                 self.failUnless(failure.check(
-                    "twisted.cred.error.UnauthorizedLogin"))
-                from twisted.cred.error import UnauthorizedLogin
-                tlog.flushErrors(UnauthorizedLogin)
+                    "flumotion.common.errors.NotAuthenticatedError"))
                 return self.clientDisconnect(factory, None)
 
             d.addErrback(uacpccTamperErrback)
             return d
         d.addCallback(uacpccTamperCallback)
-        if weHaveAnOldTwisted():
-            unittest.deferredResult(d)
-        else:
-            return d
+        return d
 
 # test with sha256 bouncer
+
+
 class Test_FPBClientFactorySaltSha256(Test_FPBClientFactory):
     bouncerClass = saltsha256.SaltSha256
     bouncerConf = saltsha256Conf
@@ -553,21 +550,19 @@ class Test_FPBClientFactorySaltSha256(Test_FPBClientFactory):
         factory = pb.FPBClientFactory()
         a = pb.Authenticator(username="user", password="test",
             address="127.0.0.1")
-        # send 
+        # send
         d = factory.login(a)
         c = reactor.connectTCP("127.0.0.1", self.portno, factory)
-        
+
         def OkCallback(result):
             # make sure we really used an SHA256 challenge/response keycard
-            self.failUnless(isinstance(factory.keycard, keycards.KeycardUASPCC))
+            self.failUnless(isinstance(factory.keycard,
+                                       keycards.KeycardUASPCC))
             self.assert_(isinstance(result, tpb.RemoteReference))
             return self.clientDisconnect(factory, result)
-        
+
         d.addCallback(OkCallback)
-        if weHaveAnOldTwisted():
-            result = unittest.deferredResult(d)
-        else:
-            return d
+        return d
 
     def testWrongPassword(self):
         factory = pb.FPBClientFactory()
@@ -578,21 +573,20 @@ class Test_FPBClientFactorySaltSha256(Test_FPBClientFactory):
         c = reactor.connectTCP("127.0.0.1", self.portno, factory)
 
         log.debug("trial", "wait for result")
+
         def WrongPasswordErrback(failure):
             # make sure we really used an SHA256 challenge/response keycard
-            self.failUnless(isinstance(factory.keycard, keycards.KeycardUASPCC))
+            self.failUnless(isinstance(factory.keycard,
+                                       keycards.KeycardUASPCC))
             # This is a CopiedFailure
             self.assert_(failure.check(
-                "twisted.cred.error.UnauthorizedLogin"))
+                "flumotion.common.errors.NotAuthenticatedError"))
             log.debug("trial", "got failure %r" % failure)
             c.disconnect()
             return True
-        if weHaveAnOldTwisted():
-            unittest.deferredError(d)
-        else:
-            d.addErrback(WrongPasswordErrback)
-            return d
-        
+        d.addErrback(WrongPasswordErrback)
+        return d
+
     def testWrongUser(self):
         factory = pb.FPBClientFactory()
 
@@ -603,27 +597,23 @@ class Test_FPBClientFactorySaltSha256(Test_FPBClientFactory):
         # send
         d = factory.login(a)
         c = reactor.connectTCP("127.0.0.1", self.portno, factory)
-        
+
         def WrongUserCb(keycard):
-            self.fail("Should have returned UnauthorizedLogin")
-            
+            self.fail("Should have returned NotAuthenticatedError")
+
         def WrongUserEb(failure):
             # find copied failure
             self.failUnless(failure.check(
-                "twisted.cred.error.UnauthorizedLogin"))
-            from twisted.cred.error import UnauthorizedLogin
-            tlog.flushErrors(UnauthorizedLogin)
+                "flumotion.common.errors.NotAuthenticatedError"))
             return self.clientDisconnect(factory, None)
-    
+
         d.addCallback(WrongUserCb)
         d.addErrback(WrongUserEb)
-    
-        if weHaveAnOldTwisted():
-            unittest.deferredResult(d)
-        else:
-            return d
+
+        return d
 
 # FIXME: do this with a fake authenticator that tampers with the challenge
+
     def notestUACPCCTamperWithChallenge(self):
         factory = pb.FPBClientFactory()
 
@@ -635,7 +625,7 @@ class Test_FPBClientFactorySaltSha256(Test_FPBClientFactory):
         # send
         d = factory.login(keycard, 'MIND')
         c = reactor.connectTCP("127.0.0.1", self.portno, factory)
-        
+
         def uacpccTamperCallback(keycard):
             self.assertEquals(keycard.state, keycards.REQUESTING)
 
@@ -643,23 +633,18 @@ class Test_FPBClientFactorySaltSha256(Test_FPBClientFactory):
             keycard.challenge = "I am a h4x0r"
             keycard.setPassword('test')
             d = factory.login(keycard, 'MIND')
-            
+
             def uacpccTamperErrback(failure):
                 # find copied failure
                 self.failUnless(failure.check(
-                    "twisted.cred.error.UnauthorizedLogin"))
-                from twisted.cred.error import UnauthorizedLogin
-                tlog.flushErrors(UnauthorizedLogin)
+                    "flumotion.common.errors.NotAuthenticatedError"))
                 return self.clientDisconnect(factory, None)
 
             d.addErrback(uacpccTamperErrback)
             return d
         d.addCallback(uacpccTamperCallback)
-        if weHaveAnOldTwisted():
-            unittest.deferredResult(d)
-        else:
-            return d
+        return d
 
 
 if __name__ == '__main__':
-     unittest.main()
+    unittest.main()
