@@ -19,11 +19,13 @@
 
 # Headers in this file shall remain intact.
 
+__version__ = "$Rev: 5969 $"
+
 # enum for result of stopped
-STOPPED_SUCCESS = 0
-STOPPED_ERROR = 1
-STOPPED_REFUSED = 2
-STOPPED_CONNECT_ERROR = 3
-STOPPED_READ_ERROR = 4
-STOPPED_INTERNAL_ERROR = 5
-STOPPED_LAST = 6
+(STOPPED_SUCCESS,
+ STOPPED_ERROR,
+ STOPPED_REFUSED,
+ STOPPED_CONNECT_ERROR,
+ STOPPED_READ_ERROR,
+ STOPPED_INTERNAL_ERROR,
+ STOPPED_LAST) = range(7)
