@@ -23,20 +23,26 @@
 Flumotion Twisted credentials
 """
 
-import crypt
-import md5
-
 import random
 
-from flumotion.common import log
+from flumotion.common import log, python
 from twisted.cred import credentials
-from flumotion.twisted.compat import implements
+from zope.interface import implements
+
+try:
+    import crypt
+except ImportError:
+    from flumotion.extern import unixcrypt as crypt
+
+__version__ = "$Rev: 7990 $"
+
 
 class Username:
     """
     I am your average username and password credentials.
     """
     implements(credentials.IUsernamePassword)
+
     def __init__(self, username, password=''):
         self.username = username
         self.password = password
@@ -44,6 +50,11 @@ class Username:
     def checkPassword(self, password):
         return password == self.password
 
+IUsernamePassword = credentials.IUsernamePassword
+
+IUsernameHashedPassword = credentials.IUsernameHashedPassword
+
+
 class IUsernameCryptPassword(credentials.ICredentials):
     """
     I encapsulate a username and check crypted passwords.
@@ -60,21 +71,23 @@ class IUsernameCryptPassword(credentials.ICredentials):
     def checkCryptPassword(self, cryptPassword):
         """
         Validate these credentials against the correct crypt password.
-                                                                                
+
         @param cryptPassword: The correct, crypt password against which to
         check.
-                                                                                
+
         @return: a deferred which becomes, or a boolean indicating if the
         password matches.
         """
 
+
 class UsernameCryptPasswordPlaintext:
     """
     I take a username and a plaintext password.
     I implement IUsernameCryptPassword.
     """
-    
+
     implements(IUsernameCryptPassword)
+
     def __init__(self, username, password):
         self.username = username
         self.password = password
@@ -85,6 +98,7 @@ class UsernameCryptPasswordPlaintext:
         encrypted = crypt.crypt(self.password, salt)
         return encrypted == cryptPassword
 
+
 class UsernameCryptPasswordCrypt:
     """
     I take a username and a crypt password.
@@ -94,8 +108,9 @@ class UsernameCryptPasswordCrypt:
     with a different salt.
     I implement IUsernameCryptPassword.
     """
-    
+
     implements(IUsernameCryptPassword)
+
     def __init__(self, username, cryptPassword=None):
         self.username = username
         self.cryptPassword = cryptPassword
@@ -115,16 +130,17 @@ class UsernameCryptPasswordCrypt:
         """
         return self.cryptPassword == cryptPassword
 
+
 def cryptRespond(challenge, cryptPassword):
     """
     Respond to a given crypt challenge with our cryptPassword.
     """
-    import md5
-    md = md5.new()
+    md = python.md5()
     md.update(cryptPassword)
     md.update(challenge)
     return md.digest()
 
+
 def dataToHex(data):
     """
     Take a string of bytes, and return a string of two-digit hex values.
@@ -133,22 +149,25 @@ def dataToHex(data):
     for c in data:
         l.append("%02x" % ord(c))
     return "".join(l)
-        
+
 # copied from twisted.spread.pb.challenge()
+
+
 def cryptChallenge():
     """
     I return some random data.
     """
     crap = ''
-    for x in range(random.randrange(15,25)):
-        crap = crap + chr(random.randint(65,90) + x - x) # pychecker madness
-    crap = md5.new(crap).digest()
+    for x in range(random.randrange(15, 25)):
+        crap = crap + chr(random.randint(65, 90) + x - x) # pychecker madness
+    crap = python.md5(crap).digest()
     return crap
-    
+
+
 class UsernameCryptPasswordCryptChallenger:
     """
     I take a username.
-    
+
     Authenticator will give me a salt and challenge me.
     Requester will respond to the challenge.
     At that point I'm ready to be used by a checker.
@@ -157,7 +176,7 @@ class UsernameCryptPasswordCryptChallenger:
 
     I implement IUsernameCryptPassword.
     """
-    
+
     implements(IUsernameCryptPassword)
 
     def __init__(self, username):
@@ -187,6 +206,7 @@ class UsernameCryptPasswordCryptChallenger:
         expected = cryptRespond(self.challenge, cryptPassword)
         return self.response == expected
 
+
 class IToken(credentials.ICredentials):
     """I encapsulate a token.
 
@@ -197,12 +217,32 @@ class IToken(credentials.ICredentials):
     @ivar token: The token associated with these credentials.
     """
 
+
 class Token:
     implements(IToken)
 
     def __init__(self, token):
         self.token = token
 
+
+class IHTTPGetArguments(credentials.ICredentials):
+    """I encapsulate HTTP GET request arguments.
+
+    This credential is used when authentication
+    depend on HTTP request arguments.
+
+    @type arguments: C{dict}
+    @ivar arguments: The HTTP request arguments.
+    """
+
+
+class HTTPGetArguments:
+    implements(IHTTPGetArguments)
+
+    def __init__(self, arguments):
+        self.arguments = arguments
+
+
 class IUsernameSha256Password(credentials.ICredentials):
     """
     I encapsulate a username and check SHA-256 passwords.
@@ -219,7 +259,7 @@ class IUsernameSha256Password(credentials.ICredentials):
     def checkSha256Password(self, sha256Password):
         """
         Validate these credentials against the correct SHA-256 password.
-                                                                                
+
         @param sha256Password: The correct SHA-256 password against which to
         check.
 
@@ -229,10 +269,12 @@ class IUsernameSha256Password(credentials.ICredentials):
 
 # our Sha256 passwords are salted;
 # ie the password string is salt + dataToHex(SHA256 digest(salt + password))
+
+
 class UsernameSha256PasswordCryptChallenger:
     """
     I take a username.
-    
+
     Authenticator will give me a salt and challenge me.
     Requester will respond to the challenge.
     At that point I'm ready to be used by a checker.
@@ -241,7 +283,7 @@ class UsernameSha256PasswordCryptChallenger:
 
     I implement IUsernameSha256Password.
     """
-    
+
     implements(IUsernameSha256Password)
 
     def __init__(self, username):
@@ -273,3 +315,91 @@ class UsernameSha256PasswordCryptChallenger:
 
         expected = cryptRespond(self.challenge, sha256Password)
         return self.response == expected
+
+
+class HTTPDigestChallenger(log.Loggable):
+    _algorithm = "MD5" # MD5-sess also supported
+
+    def __init__(self, username):
+        self.username = username
+        self.nonce = None
+        self.method = None
+        self.uri = None
+
+        self.qop = None # If non-None, the next two must be set
+        self.cnonce = None
+        self.ncvalue = None
+
+        self.response = None
+
+    def checkHTTPDigestResponse(self, ha1):
+        expectedResponse = self._calculateRequestDigest(
+            self.username, ha1, self.nonce, self.cnonce,
+            self.method, self.uri, self.ncvalue, self.qop)
+
+        self.debug(
+            "Attempting to check calculated response %s against "
+            " provided response %r", expectedResponse, self.response)
+        self.debug("Username %s, nonce %s, method %s, uri %s, qop %s, "
+                   "cnonce %s, ncvalue %s", self.username, self.nonce,
+                   self.method, self.uri, self.qop, self.cnonce,
+                   self.ncvalue)
+        self.debug("Using H(A1): %s", ha1)
+
+        if not self.response:
+            return False
+
+        return self.response == expectedResponse
+
+    def _calculateHA1(self, ha1, nonce, cnonce):
+        """
+        Calculate H(A1) as from specification (RFC2617) section 3.2.2, given
+        the initial hash H(username:realm:passwd), hex-encoded.
+
+        This basically applies the second-level hashing for MD5-sess, if
+        required.
+        """
+        if self._algorithm == 'MD5':
+            return ha1
+        elif self._algorithm == 'MD5-sess':
+            HA1 = ha1.decode('hex')
+
+            m = python.md5()
+            m.update(HA1)
+            m.update(':')
+            m.update(nonce)
+            m.update(':')
+            m.update(cnonce)
+            return m.digest().encode('hex')
+        else:
+            raise NotImplementedError("Unimplemented algorithm")
+
+    def _calculateHA2(self, method, uri):
+        # We don't support auth-int, otherwise we'd optionally need to do
+        # some more work here
+        m = python.md5()
+        m.update(method)
+        m.update(':')
+        m.update(uri)
+        return m.digest().encode('hex')
+
+    def _calculateRequestDigest(self, username, ha1, nonce, cnonce, method,
+            uri, ncvalue, qop):
+        HA1 = self._calculateHA1(ha1, nonce, cnonce)
+        HA2 = self._calculateHA2(method, uri)
+
+        m = python.md5()
+        m.update(HA1)
+        m.update(':')
+        m.update(nonce)
+        if qop:
+            m.update(':')
+            m.update(ncvalue)
+            m.update(':')
+            m.update(cnonce)
+            m.update(':')
+            m.update(qop) # Must be 'auth', others not supported
+        m.update(':')
+        m.update(HA2)
+
+        return m.digest().encode('hex')
