@@ -26,21 +26,22 @@ Inspired by L{twisted.spread.flavors}
 """
 
 from twisted.internet import defer
-from twisted.python import components
 from twisted.spread import pb
+from zope.interface import Interface
+from flumotion.common import log
 
-# T1.3: suppress components warnings in Twisted 2.0
-from flumotion.twisted import compat
-compat.filterWarnings(components, 'ComponentsDeprecationWarning')
+__version__ = "$Rev: 7740 $"
 
-from flumotion.twisted.compat import Interface
 
 ### Generice Cacheable/RemoteCache for state objects
-class IStateListener(compat.Interface):
+
+
+class IStateListener(Interface):
     """
     I am an interface for objects that want to listen to changes on
     cached states.
     """
+
     def stateSet(self, object, key, value):
         """
         @type  object: L{StateRemoteCache}
@@ -48,7 +49,7 @@ class IStateListener(compat.Interface):
         @type  key:    string
         @param key:    the key being set
         @param value:  the value the key is being set to
-        
+
         The given key on the given object has been set to the given value.
         """
 
@@ -74,6 +75,7 @@ class IStateListener(compat.Interface):
         The given value has been removed from the list given by the key.
         """
 
+
 class StateCacheable(pb.Cacheable):
     """
     I am a cacheable state object.
@@ -81,11 +83,13 @@ class StateCacheable(pb.Cacheable):
     I cache key-value pairs, where values can be either single objects
     or list of objects.
     """
+
     def __init__(self):
         self._observers = []
         self._dict = {}
 
     # our methods
+
     def addKey(self, key, value=None):
         """
         Add a key to the state cache so it can be used with set.
@@ -94,6 +98,7 @@ class StateCacheable(pb.Cacheable):
 
     # don't use [] as the default value, it creates only one reference and
     # reuses it
+
     def addListKey(self, key, value=None):
         """
         Add a key for a list of objects to the state cache.
@@ -104,6 +109,7 @@ class StateCacheable(pb.Cacheable):
 
     # don't use {} as the default value, it creates only one reference and
     # reuses it
+
     def addDictKey(self, key, value=None):
         """
         Add a key for a dict value to the state cache.
@@ -143,9 +149,9 @@ class StateCacheable(pb.Cacheable):
             raise KeyError('%s in %r' % (key, self))
 
         self._dict[key] = value
-        list = [o.callRemote('set', key, value) for o in self._observers]
-        return defer.DeferredList(list)
-        
+        dList = [o.callRemote('set', key, value) for o in self._observers]
+        return defer.DeferredList(dList)
+
     def append(self, key, value):
         """
         Append the given object to the given list.
@@ -155,9 +161,9 @@ class StateCacheable(pb.Cacheable):
             raise KeyError('%s in %r' % (key, self))
 
         self._dict[key].append(value)
-        list = [o.callRemote('append', key, value) for o in self._observers]
-        return defer.DeferredList(list)
- 
+        dList = [o.callRemote('append', key, value) for o in self._observers]
+        return defer.DeferredList(dList)
+
     def remove(self, key, value):
         """
         Remove the given object from the given list.
@@ -171,10 +177,10 @@ class StateCacheable(pb.Cacheable):
         except ValueError:
             raise ValueError('value %r not in list %r for key %r' % (
                 value, self._dict[key], key))
-        list = [o.callRemote('remove', key, value) for o in self._observers]
-        dl = defer.DeferredList(list)
+        dList = [o.callRemote('remove', key, value) for o in self._observers]
+        dl = defer.DeferredList(dList)
         return dl
- 
+
     def setitem(self, key, subkey, value):
         """
         Set a value in the given dict.
@@ -184,10 +190,10 @@ class StateCacheable(pb.Cacheable):
             raise KeyError('%s in %r' % (key, self))
 
         self._dict[key][subkey] = value
-        list = [o.callRemote('setitem', key, subkey, value)
+        dList = [o.callRemote('setitem', key, subkey, value)
                 for o in self._observers]
-        return defer.DeferredList(list)
- 
+        return defer.DeferredList(dList)
+
     def delitem(self, key, subkey):
         """
         Removes an element from the given dict. Note that the key refers
@@ -202,12 +208,13 @@ class StateCacheable(pb.Cacheable):
         except KeyError:
             raise KeyError('key %r not in dict %r for key %r' % (
                 subkey, self._dict[key], key))
-        list = [o.callRemote('delitem', key, subkey, value) for o in
+        dList = [o.callRemote('delitem', key, subkey, value) for o in
                 self._observers]
-        dl = defer.DeferredList(list)
+        dl = defer.DeferredList(dList)
         return dl
- 
+
     # pb.Cacheable methods
+
     def getStateToCacheAndObserveFor(self, perspective, observer):
         self._observers.append(observer)
         return self._dict
@@ -229,16 +236,19 @@ class StateCacheable(pb.Cacheable):
 # Instead, invalidation will only be performed by the application, if at
 # all, via an explicit call to invalidate().
 
+
 class StateRemoteCache(pb.RemoteCache):
     """
     I am a remote cache of a state object.
     """
+
     def __init__(self):
         self._listeners = {}
         # no constructor
         # pb.RemoteCache.__init__(self)
 
     # our methods
+
     def hasKey(self, key):
         return key in self._dict.keys()
 
@@ -265,12 +275,15 @@ class StateRemoteCache(pb.RemoteCache):
         # when this is created through serialization from a JobCS,
         # __init__ does not seem to get called, so create self._listeners
         if not hasattr(self, '_listeners'):
-            # fixme: this means that callbacks will be fired in
+            # FIXME: this means that callbacks will be fired in
             # arbitrary order; should be fired in order of connecting.
+            # Use twisted.python.util.OrderedDict instead
             self._listeners = {}
 
+    #F0.8: remove set=None and move set_=None there
+
     def addListener(self, listener, set=None, append=None, remove=None,
-                    setitem=None, delitem=None, invalidate=None):
+                    setitem=None, delitem=None, invalidate=None, set_=None):
         """
         Adds a listener to the remote cache.
 
@@ -278,41 +291,55 @@ class StateRemoteCache(pb.RemoteCache):
         given as the 'set', 'append', and 'remove', 'setitem', and
         'delitem' keyword arguments.
 
+        Always call this method using keyword arguments for the functions;
+        calling them with positional arguments is not supported.
+
         Setting one of the event handlers to None will ignore that
         event. It is an error for all event handlers to be None.
 
-        @param listener: A new listener object that wants to receive
-                       cache state change notifications.
-        @type listener: object implementing
-                        L{flumotion.twisted.flavors.IStateListener}
-        @param    set: A procedure to call when a value is set
-        @type     set: procedure(object, key, value) -> None
-        @param append: A procedure to call when a value is appended to a
-                       list
-        @type  append: procedure(object, key, value) -> None
-        @param remove: A procedure to call when a value is removed from
-                       a list
-        @type  remove: procedure(object, key, value) -> None
-        @param setitem: A procedure to call when a value is set in a
-                       dict.
-        @type  setitem: procedure(object, key, subkey, value) -> None
-        @param delitem: A procedure to call when a value is removed
-                       from a dict.
-        @type  delitem: procedure(object, key, subkey, value) -> None
-        @param invalidate: A procedure to call when this cache has been
-                       invalidated.
+        @param listener:   new listener object that wants to receive
+                           cache state change notifications.
+        @type  listener:   object implementing
+                           L{flumotion.twisted.flavors.IStateListener}
+        @param set_:       procedure to call when a value is set
+        @type  set_:       procedure(object, key, value) -> None
+        @param append:     procedure to call when a value is appended to a list
+        @type  append:     procedure(object, key, value) -> None
+        @param remove:     procedure to call when a value is removed from
+                           a list
+        @type  remove:     procedure(object, key, value) -> None
+        @param setitem:    procedure to call when a value is set in a dict
+        @type  setitem:    procedure(object, key, subkey, value) -> None
+        @param delitem:    procedure to call when a value is removed
+                           from a dict.
+        @type  delitem:    procedure(object, key, subkey, value) -> None
+        @param invalidate: procedure to call when this cache has been
+                           invalidated.
         @type  invalidate: procedure(object) -> None
         """
-        if not (set or append or remove or setitem or delitem or invalidate):
-            print ("Warning: Use of deprecated %r.addListener(%r) without "
-                   "explicit event handlers" % (self, listener))
-            set = listener.stateSet
+        # F0.8: remove set
+        if set:
+            import warnings
+            warnings.warn('Please use the set_ kwarg instead',
+                DeprecationWarning, stacklevel=2)
+            set_ = set
+
+        if not (set_ or append or remove or setitem or delitem or invalidate):
+            # FIXME: remove this behavior in F0.6
+            import sys
+            log.safeprintf(sys.stderr,
+                           "Warning: Use of deprecated %r.addListener(%r)"
+                           " without explicit event handlers\n", self,
+                           listener)
+            set_ = listener.stateSet
             append = listener.stateAppend
             remove = listener.stateRemove
+
         self._ensureListeners()
         if listener in self._listeners:
-            raise KeyError, listener
-        self._listeners[listener] = [set, append, remove, setitem,
+            raise KeyError(
+                "%r is already a listener of %r" % (listener, self))
+        self._listeners[listener] = [set_, append, remove, setitem,
                                      delitem, invalidate]
         if invalidate and hasattr(self, '_cache_invalid'):
             invalidate(self)
@@ -320,25 +347,35 @@ class StateRemoteCache(pb.RemoteCache):
     def removeListener(self, listener):
         self._ensureListeners()
         if listener not in self._listeners:
-            raise KeyError, listener
+            raise KeyError(listener)
         del self._listeners[listener]
 
     # pb.RemoteCache methods
+
     def setCopyableState(self, dict):
         self._dict = dict
-        
+
+    def _notifyListeners(self, index, *args):
+        # notify our local listeners; compute set of procs first, so as
+        # to allow the listeners set to change during the calls
+        self._ensureListeners()
+        for proc in [tup[index] for tup in self._listeners.values()]:
+            if proc:
+                try:
+                    proc(self, *args)
+                except Exception, e:
+                    # These are all programming errors
+                    log.warning("stateremotecache",
+                                'Exception in StateCache handler: %s',
+                                log.getExceptionMessage(e))
+
     def observe_set(self, key, value):
         self._dict[key] = value
         # if we also subclass from Cacheable, then we're a proxy, so proxy
         if hasattr(self, 'set'):
             StateCacheable.set(self, key, value)
 
-        # notify our local listeners
-        self._ensureListeners()
-        for l in self._listeners:
-            stateSet = self._listeners[l][0]
-            if stateSet: 
-                stateSet(self, key, value)
+        self._notifyListeners(0, key, value)
 
     def observe_append(self, key, value):
         # if we also subclass from Cacheable, then we're a proxy, so proxy
@@ -347,12 +384,7 @@ class StateRemoteCache(pb.RemoteCache):
         else:
             self._dict[key].append(value)
 
-        # notify our local listeners
-        self._ensureListeners()
-        for l in self._listeners:
-            stateAppend = self._listeners[l][1]
-            if stateAppend: 
-                stateAppend(self, key, value)
+        self._notifyListeners(1, key, value)
 
     def observe_remove(self, key, value):
         # if we also subclass from Cacheable, then we're a proxy, so proxy
@@ -365,12 +397,7 @@ class StateRemoteCache(pb.RemoteCache):
                 raise ValueError("value %r not under key %r with values %r" %
                     (value, key, self._dict[key]))
 
-        # notify our local listeners
-        self._ensureListeners()
-        for l in self._listeners:
-            stateRemove = self._listeners[l][2]
-            if stateRemove: 
-                stateRemove(self, key, value)
+        self._notifyListeners(2, key, value)
 
     def observe_setitem(self, key, subkey, value):
         # if we also subclass from Cacheable, then we're a proxy, so proxy
@@ -379,12 +406,7 @@ class StateRemoteCache(pb.RemoteCache):
         else:
             self._dict[key][subkey] = value
 
-        # notify our local listeners
-        self._ensureListeners()
-        for l in self._listeners:
-            stateSetitem = self._listeners[l][3]
-            if stateSetitem: 
-                stateSetitem(self, key, subkey, value)
+        self._notifyListeners(3, key, subkey, value)
 
     def observe_delitem(self, key, subkey, value):
         # if we also subclass from Cacheable, then we're a proxy, so proxy
@@ -397,12 +419,7 @@ class StateRemoteCache(pb.RemoteCache):
                 raise KeyError("key %r not in dict %r for state dict %r" %
                     (subkey, self._dict[key], self._dict))
 
-        # notify our local listeners
-        self._ensureListeners()
-        for l in self._listeners:
-            stateDelitem = self._listeners[l][4]
-            if stateDelitem: 
-                stateDelitem(self, key, subkey, value)
+        self._notifyListeners(4, key, subkey, value)
 
     def invalidate(self):
         """Invalidate this StateRemoteCache.
@@ -418,11 +435,6 @@ class StateRemoteCache(pb.RemoteCache):
         # to remotely invalidate the cache. that's ok though, because
         # double-caches are currently only used by the manager, which
         # does not call invalidate() on its caches.
-        setattr(self, '_cache_invalid', True)
+        self._cache_invalid = True
 
-        self._ensureListeners()
-        # copy of keys, because dict could change during iteration
-        for l in self._listeners.keys():
-            invalidate = self._listeners[l][5]
-            if invalidate: 
-                invalidate(self)
+        self._notifyListeners(5)
