@@ -19,14 +19,6 @@
 
 # Headers in this file shall remain intact.
 
-import os
-import signal
-
-from twisted.python import failure
-import twisted.copyright
-from twisted.internet import reactor, protocol, defer
-from flumotion.common import log as flog
-
 """
 Framework for writing automated integration tests.
 
@@ -37,7 +29,7 @@ L{twisted.trial.unittest.TestCase} class.
 
 Integration tests look like normal test methods, except that they are
 decorated with L{integration.test}, take an extra "plan" argument, and
-do not return anything. For example:
+do not return anything. For example::
 
   from twisted.trial import unittest
   from flumotion.twisted import integration
@@ -73,7 +65,7 @@ having failed.
 Also note that while the test is running, the stdout and stderr of each
 spawned process is redirected into log files in a subdirectory of where
 the test is located. For example, in the previous example, the following
-files will be created:
+files will be created::
 
   $testdir/IntegrationTestExample-$date/testEchoFunctionality/echo.stdout
   $testdir/IntegrationTestExample-$date/testEchoFunctionality/echo.stderr
@@ -84,6 +76,16 @@ successful completion of the test case, the log directory will be
 deleted.
 """
 
+import os
+import signal
+
+from twisted.python import failure
+from twisted.internet import reactor, protocol, defer
+from flumotion.common import log as flog
+
+__version__ = "$Rev: 7162 $"
+
+
 # Twisted's reactor.iterate() is defined like this:
 #
 #     def iterate(self, delay=0):
@@ -106,17 +108,27 @@ deleted.
 # the waker. This is twisted bug #1997.
 reactor.wakeUp = lambda: reactor.waker and reactor.waker.wakeUp()
 
+
 def log(format, *args):
     flog.doLog(flog.LOG, None, 'integration', format, args, -2)
+
+
 def debug(format, *args):
     flog.doLog(flog.DEBUG, None, 'integration', format, args, -2)
+
+
 def info(format, *args):
     flog.doLog(flog.INFO, None, 'integration', format, args, -2)
+
+
 def warning(format, *args):
     flog.doLog(flog.WARN, None, 'integration', format, args, -2)
+
+
 def error(format, *args):
     flog.doLog(flog.ERROR, None, 'integration', format, args, -2)
 
+
 def _which(executable):
     if os.sep in executable:
         if os.access(os.path.abspath(executable), os.X_OK):
@@ -127,39 +139,53 @@ def _which(executable):
                 return os.path.join(path, executable)
     raise CommandNotFoundException(executable)
 
+
 class UnexpectedExitCodeException(Exception):
+
     def __init__(self, process, expectedCode, actualCode):
         Exception.__init__(self)
         self.process = process
         self.expected = expectedCode
         self.actual = actualCode
+
     def __str__(self):
         return ('Expected exit code %r from %r, but got %r'
                 % (self.expected, self.process, self.actual))
 
+
 class UnexpectedExitException(Exception):
+
     def __init__(self, process):
         Exception.__init__(self)
         self.process = process
+
     def __str__(self):
         return 'The process %r exited prematurely.' % self.process
 
+
 class CommandNotFoundException(Exception):
+
     def __init__(self, command):
         Exception.__init__(self)
         self.command = command
+
     def __str__(self):
         return 'Command %r not found in the PATH.' % self.command
 
+
 class ProcessesStillRunningException(Exception):
+
     def __init__(self, processes):
         Exception.__init__(self)
         self.processes = processes
+
     def __str__(self):
         return ('Processes still running at end of test: %r'
-                % (self.processes,))
+                % (self.processes, ))
+
 
 class TimeoutException(Exception):
+
     def __init__(self, process, status):
         self.process = process
         self.status = status
@@ -168,7 +194,9 @@ class TimeoutException(Exception):
         return ('Timed out waiting for %r to exit with status %r'
                 % (self.process, self.status))
 
+
 class ProcessProtocol(protocol.ProcessProtocol):
+
     def __init__(self):
         self.exitDeferred = defer.Deferred()
         self.timedOut = False
@@ -180,22 +208,25 @@ class ProcessProtocol(protocol.ProcessProtocol):
         info('forcing timeout for process protocol %r', self)
         self.timedOut = True
         self.exitDeferred.errback(TimeoutException(process, status))
-        
+
     def processEnded(self, status):
-        info('process ended with status %r, exit code %r', status, status.value.exitCode)
+        info('process ended with status %r, exit code %r',
+             status, status.value.exitCode)
         if self.timedOut:
             warning('already timed out??')
             print 'already timed out quoi?'
         else:
-            info('process ended with status %r, exit code %r', status, status.value.exitCode)
+            info('process ended with status %r, exit code %r',
+                 status, status.value.exitCode)
             self.exitDeferred.callback(status.value.exitCode)
 
+
 class Process:
     NOT_STARTED, STARTED, STOPPED = 'NOT-STARTED', 'STARTED', 'STOPPED'
 
     def __init__(self, name, argv, testDir):
         self.name = name
-        self.argv = (_which(argv[0]),) + argv[1:]
+        self.argv = (_which(argv[0]), ) + argv[1:]
         self.testDir = testDir
 
         self.pid = None
@@ -209,7 +240,7 @@ class Process:
         assert self.state == self.NOT_STARTED
 
         self.protocol = ProcessProtocol()
-        
+
         stdout = open(os.path.join(self.testDir, self.name + '.stdout'), 'w')
         stderr = open(os.path.join(self.testDir, self.name + '.stderr'), 'w')
         # don't give it a stdin, output to log files
@@ -245,7 +276,7 @@ class Process:
         self.state = self.STARTED
 
         def got_exit(res):
-            self.state = self.STOPPED 
+            self.state = self.STOPPED
             info('process %r has stopped', self)
             return res
         self.protocol.getDeferred().addCallback(got_exit)
@@ -259,6 +290,7 @@ class Process:
         assert self.state != self.NOT_STARTED
         info('waiting for process %r to exit', self)
         d = self.protocol.getDeferred()
+
         def got_exit(res):
             debug('process %r exited with status %r', self, res)
             if res != status:
@@ -271,6 +303,7 @@ class Process:
                                                 self.protocol.timeout,
                                                 self,
                                                 status)
+
             def cancel_timeout(res):
                 debug('cancelling timeout for %r', self)
                 if self._timeoutDC.active():
@@ -282,6 +315,7 @@ class Process:
     def __repr__(self):
         return '<Process %s in state %s>' % (self.name, self.state)
 
+
 class PlanExecutor:
     # both the vm and its ops
 
@@ -308,6 +342,7 @@ class PlanExecutor:
 
     def wait(self, process, exitCode):
         assert process in self.processes
+
         def remove_from_processes_list(_):
             self.processes.remove(process)
         d = process.wait(exitCode, timeout=self.timeout)
@@ -326,11 +361,13 @@ class PlanExecutor:
                     continue
                 d = defer.Deferred()
                 dlist.append(d)
+
                 def callbacker(d):
                     return lambda status: d.callback(status.value.exitCode)
                 p.protocol.processEnded = callbacker(d)
                 p.kill(sig=signal.SIGKILL)
             d = defer.DeferredList(dlist)
+
             def error(_):
                 if failure:
                     return failure
@@ -339,12 +376,13 @@ class PlanExecutor:
             d.addCallback(error)
             return d
         return failure
-            
+
     def run(self, ops, timeout=20):
         self.timeout = timeout
         d = defer.Deferred()
+
         def run_op(_, op):
-            # print 'Last result: %r' % (_,)
+            # print 'Last result: %r' % (_, )
             # print 'Now running: %s(%r)' % (op[0].__name__, op[1:])
             return op[0](*op[1:])
         for op in ops:
@@ -359,7 +397,9 @@ class PlanExecutor:
         reactor.callLater(0, d.callback, None)
         return d
 
+
 class Plan:
+
     def __init__(self, testCase, testName):
         self.name = testName
         self.testCaseName = testCase.__class__.__name__
@@ -410,10 +450,10 @@ class Plan:
         self.timeout = timeout
 
     def spawn(self, command, *args):
-        allArgs = (command,) + args
+        allArgs = (command, ) + args
         process, = self.spawnPar(allArgs)
         return process
-        
+
     def spawnPar(self, *argvs):
         processes = []
         self._appendOp(self.vm.checkExits, ())
@@ -430,9 +470,9 @@ class Plan:
 
     def wait(self, process, status):
         self.waitPar((process, status))
-        
+
     def waitPar(self, *processStatusPairs):
-        processes = tuple([p for p,s in processStatusPairs])
+        processes = tuple([p for p, s in processStatusPairs])
         self._appendOp(self.vm.checkExits, processes)
         for process, status in processStatusPairs:
             self._appendOp(self.vm.wait, process, status)
@@ -447,21 +487,17 @@ class Plan:
         d.addCallback(lambda _: self._cleanOutputDir())
         return d
 
+
 def test(proc):
     testName = proc.__name__
+
     def wrappedtest(self):
         plan = Plan(self, testName)
         proc(self, plan)
-        if twisted.copyright.version < '2.0':
-            # FIXME T1.3
-            info('using deferredResult for old trial')
-            from twisted.trial import unittest
-            return unittest.deferredResult(plan.execute())
-        else:
-            return plan.execute()
+        return plan.execute()
     try:
         wrappedtest.__name__ = testName
-    except Exception:
+    except TypeError:
         # can only set procedure names in python >= 2.4
         pass
     # trial seems to require a timeout, at least in twisted 2.4, so give
