@@ -27,19 +27,25 @@ Inspired by L{twisted.spread.pb}
 
 import time
 
-from twisted.cred import checkers, credentials, error
+from twisted.cred import checkers, credentials
 from twisted.cred.portal import IRealm, Portal
 from twisted.internet import protocol, defer, reactor
+from twisted.internet import error as terror
 from twisted.python import log, reflect, failure
 from twisted.spread import pb, flavors
 from twisted.spread.pb import PBClientFactory
+from zope.interface import implements
 
 from flumotion.configure import configure
 from flumotion.common import keycards, interfaces, common, errors
 from flumotion.common import log as flog
+from flumotion.common.netutils import addressGetHost
 from flumotion.twisted import reflect as freflect
 from flumotion.twisted import credentials as fcredentials
-from flumotion.twisted.compat import implements
+
+__version__ = "$Rev: 7942 $"
+
+
 # TODO:
 #   merge FMCF back into twisted
 
@@ -54,6 +60,7 @@ from flumotion.twisted.compat import implements
 # this way you can request a different avatarId than the user you authenticate
 # with, or you can login without a username
 
+
 class FPBClientFactory(pb.PBClientFactory, flog.Loggable):
     """
     I am an extended Perspective Broker client factory using generic
@@ -76,20 +83,38 @@ class FPBClientFactory(pb.PBClientFactory, flog.Loggable):
     keycard = None
     medium = None
     perspectiveInterface = None # override in subclass
+    _fpbconnector = None
+
+    ## from protocol.ClientFactory
+
+    def startedConnecting(self, connector):
+        self._fpbconnector = connector
+        return pb.PBClientFactory.startedConnecting(self, connector)
+
+    ## from twisted.spread.pb.ClientFactory
+
+    def disconnect(self):
+        if self._fpbconnector:
+            try:
+                self._fpbconnector.stopConnecting()
+            except terror.NotConnectingError:
+                pass
+        return pb.PBClientFactory.disconnect(self)
 
     def getKeycardClasses(self):
         """
         Ask the remote PB server for all the keycard interfaces it supports.
 
-        @rtype: L{defer.Deferred} returning list of str
+        @rtype: L{twisted.internet.defer.Deferred} returning list of str
         """
+
         def getRootObjectCb(root):
             return root.callRemote('getKeycardClasses')
 
         d = self.getRootObject()
         d.addCallback(getRootObjectCb)
         return d
-        
+
     def login(self, authenticator):
         """
         Login, respond to challenges, and eventually get perspective
@@ -104,7 +129,8 @@ class FPBClientFactory(pb.PBClientFactory, flog.Loggable):
         assert not isinstance(authenticator, keycards.Keycard)
         interfaces = []
         if self.perspectiveInterface:
-            self.debug('perspectiveInterface is %r' % self.perspectiveInterface)
+            self.debug('perspectiveInterface is %r' %
+                       self.perspectiveInterface)
             interfaces.append(self.perspectiveInterface)
         else:
             self.warning('No perspectiveInterface set on %r' % self)
@@ -112,9 +138,9 @@ class FPBClientFactory(pb.PBClientFactory, flog.Loggable):
             interfaces.append(pb.IPerspective)
         interfaces = [reflect.qual(interface)
                           for interface in interfaces]
-            
+
         def getKeycardClassesCb(keycardClasses):
-            self.debug('supported keycard classes: %r' % keycardClasses)
+            self.log('supported keycard classes: %r' % keycardClasses)
             d = authenticator.issue(keycardClasses)
             return d
 
@@ -132,52 +158,53 @@ class FPBClientFactory(pb.PBClientFactory, flog.Loggable):
         return d
 
     # we are a different kind of PB client, so warn
-    def _cbSendUsername(self, root, username, password, avatarId, client, interfaces):
+
+    def _cbSendUsername(self, root, username, password,
+                        avatarId, client, interfaces):
         self.warning("you really want to use cbSendKeycard")
 
-        
     def _cbSendKeycard(self, root, authenticator, client, interfaces, count=0):
-        self.debug("_cbSendKeycard(root=%r, authenticator=%r, client=%r, " \
-            "interfaces=%r, count=%d" % (
-            root, authenticator, client, interfaces, count))
+        self.log("_cbSendKeycard(root=%r, authenticator=%r, client=%r, "
+                 "interfaces=%r, count=%d", root, authenticator, client,
+                 interfaces, count)
         count = count + 1
         d = root.callRemote("login", self.keycard, client, *interfaces)
-        return d.addCallback(self._cbLoginCallback, root, authenticator, client,
-            interfaces, count)
+        return d.addCallback(self._cbLoginCallback, root,
+                             authenticator, client, interfaces, count)
 
     # we can get either a keycard, None (?) or a remote reference
+
     def _cbLoginCallback(self, result, root, authenticator, client, interfaces,
         count):
         if count > 5:
             # too many recursions, server is h0rked
             self.warning('Too many recursions, internal error.')
-        self.debug("FPBClientFactory(): result %r" % result)
-
-        if not result:
-            self.warning('No result, raising.')
-            raise error.UnauthorizedLogin()
+        self.log("FPBClientFactory(): result %r" % result)
 
         if isinstance(result, pb.RemoteReference):
             # everything done, return reference
-            self.debug('Done, returning result %r' % result)
+            self.debug('login successful, returning %r', result)
             return result
 
         # must be a keycard
         keycard = result
         if not keycard.state == keycards.AUTHENTICATED:
-            self.debug("FPBClientFactory(): requester needs to resend %r" %
-                keycard)
+            self.log("FPBClientFactory(): requester needs to resend %r",
+                     keycard)
             d = authenticator.respond(keycard)
+
             def _loginAgainCb(keycard):
                 d = root.callRemote("login", keycard, client, *interfaces)
-                return d.addCallback(self._cbLoginCallback, root, authenticator,
-                    client, interfaces, count)
+                return d.addCallback(self._cbLoginCallback, root,
+                                     authenticator, client,
+                                     interfaces, count)
             d.addCallback(_loginAgainCb)
             return d
 
         self.debug("FPBClientFactory(): authenticated %r" % keycard)
         return keycard
 
+
 class ReconnectingPBClientFactory(pb.PBClientFactory, flog.Loggable,
                                   protocol.ReconnectingClientFactory):
     """
@@ -193,13 +220,15 @@ class ReconnectingPBClientFactory(pb.PBClientFactory, flog.Loggable,
         self._doingLogin = False
 
     def clientConnectionFailed(self, connector, reason):
-        log.msg("connection failed, reason %r" % reason)
+        log.msg("connection failed to %s, reason %r" % (
+            connector.getDestination(), reason))
         pb.PBClientFactory.clientConnectionFailed(self, connector, reason)
         RCF = protocol.ReconnectingClientFactory
         RCF.clientConnectionFailed(self, connector, reason)
 
     def clientConnectionLost(self, connector, reason):
-        log.msg("connection lost, reason %r" % reason)
+        log.msg("connection lost to %s, reason %r" % (
+            connector.getDestination(), reason))
         pb.PBClientFactory.clientConnectionLost(self, connector, reason,
                                              reconnecting=True)
         RCF = protocol.ReconnectingClientFactory
@@ -218,14 +247,16 @@ class ReconnectingPBClientFactory(pb.PBClientFactory, flog.Loggable,
         self._client = client
 
         self._doingLogin = True
-        
+
     # methods to override
+
     def gotDeferredLogin(self, deferred):
         """
         The deferred from login is now available.
         """
         raise NotImplementedError
 
+
 class ReconnectingFPBClientFactory(FPBClientFactory,
                                    protocol.ReconnectingClientFactory):
     """
@@ -240,15 +271,21 @@ class ReconnectingFPBClientFactory(FPBClientFactory,
         FPBClientFactory.__init__(self)
         self._doingLogin = False
         self._doingGetPerspective = False
-        
+
     def clientConnectionFailed(self, connector, reason):
-        log.msg("connection failed, reason %r" % reason)
+        log.msg("connection failed to %s, reason %r" % (
+            connector.getDestination(), reason))
         FPBClientFactory.clientConnectionFailed(self, connector, reason)
         RCF = protocol.ReconnectingClientFactory
         RCF.clientConnectionFailed(self, connector, reason)
+        if self.continueTrying:
+            self.debug("will try reconnect in %f seconds", self.delay)
+        else:
+            self.debug("not trying to reconnect")
 
     def clientConnectionLost(self, connector, reason):
-        log.msg("connection lost, reason %r" % reason)
+        log.msg("connection lost to %s, reason %r" % (
+            connector.getDestination(), reason))
         FPBClientFactory.clientConnectionLost(self, connector, reason,
                                              reconnecting=True)
         RCF = protocol.ReconnectingClientFactory
@@ -264,12 +301,14 @@ class ReconnectingFPBClientFactory(FPBClientFactory,
 
     # TODO: This is a poorly named method; it just provides the appropriate
     # authentication information, and doesn't actually _start_ login at all.
+
     def startLogin(self, authenticator):
         assert not isinstance(authenticator, keycards.Keycard)
         self._authenticator = authenticator
         self._doingLogin = True
-        
+
     # methods to override
+
     def gotDeferredLogin(self, deferred):
         """
         The deferred from login is now available.
@@ -282,13 +321,15 @@ class ReconnectingFPBClientFactory(FPBClientFactory,
 # in other terms, you can request different "kinds" of avatars from the same
 # PB server.
 # this code needs to be sent upstream to Twisted
+
+
 class _FPortalRoot:
     """
     Root object, used to login to bouncer.
     """
 
     implements(flavors.IPBRoot)
-    
+
     def __init__(self, bouncerPortal):
         """
         @type bouncerPortal: L{flumotion.twisted.portal.BouncerPortal}
@@ -298,6 +339,7 @@ class _FPortalRoot:
     def rootObject(self, broker):
         return _BouncerWrapper(self.bouncerPortal, broker)
 
+
 class _BouncerWrapper(pb.Referenceable, flog.Loggable):
 
     logCategory = "_BouncerWrapper"
@@ -310,7 +352,7 @@ class _BouncerWrapper(pb.Referenceable, flog.Loggable):
         """
         @returns: the fully-qualified class names of supported keycard
                   interfaces
-        @rtype:   L{defer.Deferred} firing list of str
+        @rtype:   L{twisted.internet.defer.Deferred} firing list of str
         """
         return self.bouncerPortal.getKeycardClasses()
 
@@ -323,39 +365,32 @@ class _BouncerWrapper(pb.Referenceable, flog.Loggable):
         @returns: one of
             - a L{flumotion.common.keycards.Keycard} when more steps
               need to be performed
-            - a L{twisted.spread.pb.AsReferenceable} when authentication 
+            - a L{twisted.spread.pb.AsReferenceable} when authentication
               has succeeded, which will turn into a
               L{twisted.spread.pb.RemoteReference} on the client side
-            - a L{twisted.cred.error.UnauthorizedLogin} when authentication
-              is denied
+            - a L{flumotion.common.errors.NotAuthenticatedError} when
+              authentication is denied
         """
+
+        def loginResponse(result):
+            self.log("loginResponse: result=%r", result)
+            # if the result is a keycard, we're not yet ready
+            if isinstance(result, keycards.Keycard):
+                return result
+            else:
+                # authenticated, so the result is the tuple
+                interface, perspective, logout = result
+                self.broker.notifyOnDisconnect(logout)
+                return pb.AsReferenceable(perspective, "perspective")
+
         # corresponds with FPBClientFactory._cbSendKeycard
-        self.log("remote_login(keycard=%s, *interfaces=%r" % (keycard, interfaces))
+        self.log("remote_login(keycard=%s, *interfaces=%r" % (
+            keycard, interfaces))
         interfaces = [freflect.namedAny(interface) for interface in interfaces]
         d = self.bouncerPortal.login(keycard, mind, *interfaces)
-        d.addCallback(self._authenticateCallback, mind, *interfaces)
+        d.addCallback(loginResponse)
         return d
 
-    def _authenticateCallback(self, result, mind, *interfaces):
-        self.log("_authenticateCallback(result=%r, mind=%r, interfaces=%r" % (result, mind, interfaces))
-        # FIXME: coverage indicates that "not result" does not happen,
-        # presumably because a Failure is triggered before us
-        if not result:
-            return failure.Failure(error.UnauthorizedLogin())
-
-        # if the result is a keycard, we're not yet ready
-        if isinstance(result, keycards.Keycard):
-            return result
-
-        # authenticated, so the result is the tuple
-        # FIXME: our keycard should be stored higher up since it was authd
-        # then cleaned up sometime in the future
-        # for that we probably need to pass it along
-        return self._loggedIn(result)
-
-    def _loggedIn(self, (interface, perspective, logout)):
-        self.broker.notifyOnDisconnect(logout)
-        return pb.AsReferenceable(perspective, "perspective")
 
 class Authenticator(flog.Loggable, pb.Referenceable):
     """
@@ -384,6 +419,7 @@ class Authenticator(flog.Loggable, pb.Referenceable):
     username = None
     password = None
     address = None
+    ttl = 30
     # FIXME: we can add ssh keys and similar here later on
 
     def __init__(self, **kwargs):
@@ -397,11 +433,11 @@ class Authenticator(flog.Loggable, pb.Referenceable):
         @param keycardClasses: list of fully qualified keycard classes
         @type  keycardClasses: list of str
 
-        @rtype: L{defer.Deferred} firing L{keycards.Keycard}
+        @rtype: L{twisted.internet.defer.Deferred} firing L{keycards.Keycard}
         """
         # this method returns a deferred so we present the same interface
         # as the RemoteAuthenticator adapter
-    
+
         # construct a list of keycard interfaces we can support right now
         supported = []
         # address is allowed to be None
@@ -417,25 +453,29 @@ class Authenticator(flog.Loggable, pb.Referenceable):
 
         for i in keycardClasses:
             if i in supported:
-                self.debug('Keycard interface %s supported, looking up' % i)
+                self.log('Keycard interface %s supported, looking up', i)
                 name = i.split(".")[-1]
                 methodName = "issue_%s" % name
                 method = getattr(self, methodName)
                 keycard = method()
-                self.debug('Issuing keycard %r of class %s' % (
-                    keycard, name))
+                self.debug('Issuing keycard %r of class %s', keycard,
+                           name)
                 keycard.avatarId = self.avatarId
+                if self.ttl is not None:
+                    keycard.ttl = self.ttl
                 return defer.succeed(keycard)
 
         self.debug('Could not issue a keycard')
         return defer.succeed(None)
 
     # non-challenge types
+
     def issue_KeycardUACPP(self):
         return keycards.KeycardUACPP(self.username, self.password,
             self.address)
 
     # challenge types
+
     def issue_KeycardUACPCC(self):
         return keycards.KeycardUACPCC(self.username, self.address)
 
@@ -450,7 +490,8 @@ class Authenticator(flog.Loggable, pb.Referenceable):
         @param keycard: the keycard with the challenge to respond to
         @type  keycard: L{keycards.Keycard}
 
-        @rtype:   L{defer.Deferred} firing a {keycards.Keycard}
+        @rtype:   L{twisted.internet.defer.Deferred} firing
+                  a {keycards.Keycard}
         @returns: a deferred firing the keycard with a response set
         """
         self.debug('responding to challenge on keycard %r' % keycard)
@@ -459,22 +500,24 @@ class Authenticator(flog.Loggable, pb.Referenceable):
         return defer.succeed(method(keycard))
 
     def respond_KeycardUACPCC(self, keycard):
-        self.debug('setting password')
+        self.log('setting password')
         keycard.setPassword(self.password)
         return keycard
 
     def respond_KeycardUASPCC(self, keycard):
-        self.debug('setting password')
+        self.log('setting password')
         keycard.setPassword(self.password)
         return keycard
 
     ### pb.Referenceable methods
+
     def remote_issue(self, interfaces):
         return self.issue(interfaces)
 
     def remote_respond(self, keycard):
         return self.respond(keycard)
 
+
 class RemoteAuthenticator:
     """
     I am an adapter for a pb.RemoteReference to present the same interface
@@ -488,7 +531,13 @@ class RemoteAuthenticator:
     def __init__(self, remoteReference):
         self._remote = remoteReference
 
+    def copy(self, avatarId=None):
+        ret = RemoteAuthenticator(self._remote)
+        ret.avatarId = avatarId or self.avatarId
+        return ret
+
     def issue(self, interfaces):
+
         def issueCb(keycard):
             keycard.avatarId = self.avatarId
             return keycard
@@ -511,15 +560,17 @@ class Referenceable(pb.Referenceable, flog.Loggable):
 
 
     # a referenceable that logs receiving remote messages
+
     def remoteMessageReceived(self, broker, message, args, kwargs):
         args = broker.unserialize(args)
         kwargs = broker.unserialize(kwargs)
         method = getattr(self, "remote_%s" % message, None)
         if method is None:
-            raise pb.NoSuchMethod("No such method: remote_%s" % (message,))
+            raise pb.NoSuchMethod("No such method: remote_%s" % (message, ))
 
         level = flog.DEBUG
-        if message == 'ping': level = flog.LOG
+        if message == 'ping':
+            level = flog.LOG
 
         debugClass = self.logCategory.upper()
         # all this malarkey is to avoid actually interpolating variables
@@ -532,38 +583,27 @@ class Referenceable(pb.Referenceable, flog.Loggable):
         logKwArgs = self.doLog(level, method, format, *debugArgs)
 
         # invoke the remote_ method
-        try:
-            state = method(*args, **kwargs)
-        except TypeError:
-            self.warning("%s didn't accept %s and %s" % (method, args, kwargs))
-            raise
+        d = defer.maybeDeferred(method, *args, **kwargs)
 
         # log coming out of the method
-        if isinstance(state, defer.Deferred):
-            # for a deferred, we currently can't log a better location than
-            # the def line for the function/instance that we called above
-            def callback(result):
-                format, debugArgs = flog.getFormatArgs(
-                    '%s <-- %s: remote_%s(', startArgs,
-                    '): %r', (flog.ellipsize(result), ), args, kwargs)
-                self.doLog(level, -1, format, *debugArgs, **logKwArgs)
-                return result
-            def errback(failure):
-                format, debugArgs = flog.getFormatArgs(
-                    '%s <-- %s: remote_%s(', startArgs,
-                    '): failure %r', (failure, ), args, kwargs)
-                self.doLog(level, -1, format, *debugArgs, **logKwArgs)
-                return failure
 
-            state.addCallback(callback)
-            state.addErrback(errback)
-        else:
+        def callback(result):
             format, debugArgs = flog.getFormatArgs(
                 '%s <-- %s: remote_%s(', startArgs,
-                '): %r', (flog.ellipsize(state), ), args, kwargs)
+                '): %r', (flog.ellipsize(result), ), args, kwargs)
             self.doLog(level, -1, format, *debugArgs, **logKwArgs)
+            return result
+
+        def errback(failure):
+            format, debugArgs = flog.getFormatArgs(
+                '%s <-- %s: remote_%s(', startArgs,
+                '): failure %r', (failure, ), args, kwargs)
+            self.doLog(level, -1, format, *debugArgs, **logKwArgs)
+            return failure
+
+        d.addCallbacks(callback, errback)
+        return broker.serialize(d, self.perspective)
 
-        return broker.serialize(state, self.perspective)
 
 class Avatar(pb.Avatar, flog.Loggable):
     """
@@ -580,15 +620,23 @@ class Avatar(pb.Avatar, flog.Loggable):
         self.debug("created new Avatar with id %s", avatarId)
 
     # a referenceable that logs receiving remote messages
+
     def perspectiveMessageReceived(self, broker, message, args, kwargs):
         args = broker.unserialize(args)
         kwargs = broker.unserialize(kwargs)
+        return self.perspectiveMessageReceivedUnserialised(broker, message,
+            args, kwargs)
+
+    def perspectiveMessageReceivedUnserialised(self, broker, message,
+            args, kwargs):
         method = getattr(self, "perspective_%s" % message, None)
         if method is None:
-            raise pb.NoSuchMethod("No such method: perspective_%s" % (message,))
+            raise pb.NoSuchMethod("No such method: perspective_%s" % (
+                message, ))
 
         level = flog.DEBUG
-        if message == 'ping': level = flog.LOG
+        if message == 'ping':
+            level = flog.LOG
         debugClass = self.logCategory.upper()
         startArgs = [self.remoteLogName, debugClass, message]
         format, debugArgs = flog.getFormatArgs(
@@ -598,55 +646,41 @@ class Avatar(pb.Avatar, flog.Loggable):
         logKwArgs = self.doLog(level, method, format, *debugArgs)
 
         # invoke the perspective_ method
-        try:
-            state = method(*args, **kwargs)
-        except TypeError:
-            self.debug("%s didn't accept %s and %s" % (method, args, kwargs))
-            raise
-        except pb.Error, e:
-            format, debugArgs = flog.getFormatArgs(
-                '%s <-- %s: perspective_%s(', startArgs,
-                '): pb.Error %r', (e, ), args, kwargs)
-            self.doLog(level, -1, format, *debugArgs, **logKwArgs)
-            raise e
+        d = defer.maybeDeferred(method, *args, **kwargs)
 
         # log coming out of the method
-        if isinstance(state, defer.Deferred):
-            # for a deferred, we currently can't log a better location than
-            # the def line for the function/instance that we called above
-            def callback(result):
-                format, debugArgs = flog.getFormatArgs(
-                    '%s <-- %s: perspective_%s(', startArgs,
-                    '): %r', (flog.ellipsize(result), ), args, kwargs)
-                self.doLog(level, -1, format, *debugArgs, **logKwArgs)
-                return result
-            def errback(failure):
-                format, debugArgs = flog.getFormatArgs(
-                    '%s <-- %s: perspective_%s(', startArgs,
-                    '): failure %r', (failure, ), args, kwargs)
-                self.doLog(level, -1, format, *debugArgs, **logKwArgs)
-                return failure
 
-            state.addCallback(callback)
-            state.addErrback(errback)
-        else:
+        def callback(result):
             format, debugArgs = flog.getFormatArgs(
                 '%s <-- %s: perspective_%s(', startArgs,
-                '): %r', (flog.ellipsize(state), ), args, kwargs)
+                '): %r', (flog.ellipsize(result), ), args, kwargs)
             self.doLog(level, -1, format, *debugArgs, **logKwArgs)
+            return result
 
-        return broker.serialize(state, self, method, args, kwargs)
+        def errback(failure):
+            format, debugArgs = flog.getFormatArgs(
+                '%s <-- %s: perspective_%s(', startArgs,
+                '): failure %r', (failure, ), args, kwargs)
+            self.doLog(level, -1, format, *debugArgs, **logKwArgs)
+            return failure
+
+        d.addCallbacks(callback, errback)
+
+        return broker.serialize(d, self, method, args, kwargs)
 
     def setMind(self, mind):
         """
         Tell the avatar that the given mind has been attached.
         This gives the avatar a way to call remotely to the client that
         requested this avatar.
-        This is scheduled by the portal after the client has logged in.
+
+        It is best to call setMind() from within the avatar's __init__
+        method. Some old code still does this via a callLater, however.
 
         @type mind: L{twisted.spread.pb.RemoteReference}
         """
         self.mind = mind
+
         def nullMind(x):
             self.debug('%r: disconnected from %r' % (self, self.mind))
             self.mind = None
@@ -656,9 +690,10 @@ class Avatar(pb.Avatar, flog.Loggable):
         tarzan = transport.getHost()
         jane = transport.getPeer()
         if tarzan and jane:
-            self.debug("PB client connection seen by me is from me %s to %s" % (
-                common.addressGetHost(tarzan),
-                common.addressGetHost(jane)))
+            self.debug(
+                "PB client connection seen by me is from me %s to %s" % (
+                addressGetHost(tarzan),
+                addressGetHost(jane)))
         self.log('Client attached is mind %s', mind)
 
     def mindCallRemoteLogging(self, level, stackDepth, name, *args,
@@ -697,7 +732,7 @@ class Avatar(pb.Avatar, flog.Loggable):
 
         def errback(failure):
             format, debugArgs = flog.getFormatArgs(
-                '%s <-- %s: callRemote(%s', startArgs,
+                '%s <-- %s: callRemote(%s, ', startArgs,
                 '): %r', (failure, ), args, kwargs)
             self.doLog(level, -1, format, *debugArgs, **logKwArgs)
             return failure
@@ -725,8 +760,10 @@ class Avatar(pb.Avatar, flog.Loggable):
         if self.mind:
             return self.mind.broker.transport.loseConnection()
 
+
 class PingableAvatar(Avatar):
-    _pingCheckInterval = configure.heartbeatInterval * 2.5
+    _pingCheckInterval = (configure.heartbeatInterval *
+                          configure.pingTimeoutMultiplier)
 
     def perspective_ping(self):
         self._lastPing = time.time()
@@ -752,6 +789,10 @@ class PingableAvatar(Avatar):
             self._pingCheckDC.cancel()
         self._pingCheckDC = None
 
+        # release the disconnect function, too, to help break any
+        # potential cycles
+        self._pingCheckDisconnect = None
+
     def setMind(self, mind):
         # chain up
         Avatar.setMind(self, mind)
@@ -762,8 +803,8 @@ class PingableAvatar(Avatar):
         self.mind.notifyOnDisconnect(stopPingCheckingCb)
 
         # Now we have a remote reference, so start checking pings.
+
         def _disconnect():
             if self.mind:
                 self.mind.broker.transport.loseConnection()
         self.startPingChecking(_disconnect)
-
