@@ -19,30 +19,26 @@
 
 # Headers in this file shall remain intact.
 
-import optparse
 import os
 import sys
 
 from twisted.internet import reactor
 
 from flumotion.configure import configure
-from flumotion.common import log, keycards, common, errors
+from flumotion.common import log, errors
+from flumotion.common import connection
+from flumotion.common.options import OptionGroup, OptionParser
+from flumotion.common.process import startup
 from flumotion.worker import worker, config
 from flumotion.twisted import pb
 
-def _createParser():
-    parser = optparse.OptionParser()
-    parser.add_option('-d', '--debug',
-                      action="store", type="string", dest="debug",
-                      help="set debug levels")
-    parser.add_option('-v', '--verbose',
-                      action="store_true", dest="verbose",
-                      help="be verbose")
-    parser.add_option('', '--version',
-                      action="store_true", dest="version",
-                      help="show version information")
+__version__ = "$Rev: 7162 $"
 
-    group = optparse.OptionGroup(parser, "worker options")
+
+def _createParser():
+    parser = OptionParser(domain="flumotion-worker")
+
+    group = OptionGroup(parser, "worker options")
     group.add_option('-H', '--host',
                      action="store", type="string", dest="host",
                      help="manager host to connect to [default localhost]")
@@ -99,17 +95,18 @@ def _createParser():
 
     return parser
 
+
 def _readConfig(workerFile, options):
     # modifies options dict in-place
     log.info('worker', 'Reading configuration from %s' % workerFile)
     try:
         cfg = config.WorkerConfigXML(workerFile)
     except config.ConfigError, value:
-        raise errors.SystemError(
+        raise errors.FatalError(
             "Could not load configuration from %s: %s" % (
             workerFile, value))
     except IOError, e:
-        raise errors.SystemError(
+        raise errors.FatalError(
             "Could not load configuration from %s: %s" % (
             workerFile, e.strerror))
 
@@ -143,6 +140,8 @@ def _readConfig(workerFile, options):
     # XML could specify it as empty, meaning "don't use any"
     if not options.feederports and cfg.feederports is not None:
         options.feederports = cfg.feederports
+    if options.randomFeederports is None:
+        options.randomFeederports = cfg.randomFeederports
     if options.randomFeederports:
         options.feederports = None
         log.debug('worker', 'Using random feederports')
@@ -152,9 +151,10 @@ def _readConfig(workerFile, options):
     # general
     # command-line debug > environment debug > config file debug
     if not options.debug and cfg.fludebug \
-        and not os.environ.has_key('FLU_DEBUG'):
+        and not 'FLU_DEBUG' in os.environ:
         options.debug = cfg.fludebug
-    
+
+
 def main(args):
     parser = _createParser()
     log.debug('worker', 'Parsing arguments (%r)' % ', '.join(args))
@@ -167,14 +167,6 @@ def main(args):
             log.debug('worker', 'Setting configure.%s to %s' % (d, o))
             setattr(configure, d, o)
 
-    # verbose overrides --debug; is only a command-line option
-    if options.verbose:
-        options.debug = "*:3"
- 
-    # apply the command-line debug level if is given through --verbose or -d
-    if options.debug:
-        log.setFluDebug(options.debug)
-
     # translate feederports string to range
     if options.feederports:
         if not '-' in options.feederports:
@@ -221,11 +213,6 @@ def main(args):
             options.transport)
         return 1
 
-    # handle all options
-    if options.version:
-        print common.version("flumotion-worker")
-        return 0
-
     # reset FLU_DEBUG which could be different after parsing XML file
     if options.debug:
         log.setFluDebug(options.debug)
@@ -251,47 +238,36 @@ def main(args):
     if options.daemonize:
         if options.serviceName:
             name = options.serviceName
+        if not options.daemonizeTo:
+            options.daemonizeTo = "/"
 
-    common.startup("worker", name, options.daemonize, options.daemonizeTo)
+    startup("worker", name, options.daemonize, options.daemonizeTo)
 
     log.debug('worker', 'Running Flumotion version %s' %
         configure.version)
     import twisted.copyright
     log.debug('worker', 'Running against Twisted version %s' %
         twisted.copyright.version)
-    log.debug('worker', 'Running against GStreamer version %s' %
-        configure.gst_version)
 
     # register all package paths (FIXME: this should go away when
     # components come from manager)
     from flumotion.common import setup
     setup.setupPackagePath()
 
-    # create a brain and have it remember the manager to direct jobs to
-    # connect the brain to the manager
-    if options.transport == "tcp":
-        reactor.connectTCP(options.host, options.port,
-            brain.workerClientFactory)
-    elif options.transport == "ssl":
-        from twisted.internet import ssl
-        reactor.connectSSL(options.host, options.port,
-            brain.workerClientFactory,
-            ssl.ClientContextFactory())
+    # FIXME: why address='localhost' ?
+    authenticator = pb.Authenticator(username=options.username,
+                                     password=options.password,
+                                     address='localhost',
+                                     avatarId=options.name)
+    info = connection.PBConnectionInfo(options.host, options.port,
+                                       options.transport == "ssl",
+                                       authenticator)
+    brain.login(info)
 
     log.info('worker',
-             'Connecting to manager %s:%d using %s' % (options.host,
-                                                       options.port,
-                                                       options.transport.upper()))
+             'Connecting to manager %s using %s' % (info,
+                                                    options.transport.upper()))
 
-    authenticator = pb.Authenticator(
-        username=options.username,
-        password=options.password,
-        address='localhost', # FIXME: why localhost ?
-        avatarId=options.name
-    )
-    brain.login(authenticator)
-
-    reactor.addSystemEventTrigger('before', 'shutdown', brain.shutdownHandler)
 
     # go into the reactor main loop
     reactor.run()
